@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -15,29 +15,49 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
+""" The hints module should contain functions to call in your code.
+
+In reality, right now it does only contain an import tracing mechanism
+that helps us with debugging.
+"""
 
 from __future__ import print_function
-import sys, os
+
+import os
+import sys
 
 original_import = __import__
 
 _indentation = 0
 
+
 def _normalizePath(path):
     path = os.path.abspath(path)
 
     best = None
 
-    for path_entry in sys.path:
-        if path.startswith(path_entry):
+    paths = list(sys.path)
+
+    # Nuitka standalone mode.
+    try:
+        paths.append(__nuitka_binary_dir)
+        paths.append(os.getcwd())
+    except NameError:
+        pass
+
+    for path_entry in paths:
+        path_entry = os.path.normcase(path_entry)
+
+        if os.path.normcase(path).startswith(path_entry):
             if best is None or len(path_entry) > len(best):
                 best = path_entry
 
     if best is not None:
-        path = path.replace(best, "$PYTHONPATH")
+        path = "$PYTHONPATH" + path[len(best) :]
 
     return path
 
+
 def _moduleRepr(module):
     try:
         module_file = module.__file__
@@ -46,29 +66,37 @@ def _moduleRepr(module):
         if module_file.endswith(".so"):
             module_file = os.path.join(
                 os.path.dirname(module_file),
-                os.path.basename(module_file).split(".")[0] + ".so"
+                os.path.basename(module_file).split(".")[0] + ".so",
             )
 
         file_desc = "file " + _normalizePath(module_file).replace(".pyc", ".py")
     except AttributeError:
         file_desc = "built-in"
 
-    return "<module %s %s>" % (
-        module.__name__,
-        file_desc
-    )
+    return "<module %s %s>" % (module.__name__, file_desc)
 
-def enableImportTracing(normalize_paths = True, show_source = False):
 
-    def _ourimport(name, globals = None, locals = None, fromlist = None,  # @ReservedAssignment
-                   level = -1 if sys.version_info[0] < 2 else 0):
+def enableImportTracing(normalize_paths=True, show_source=False):
+    # pylint: disable=redefined-builtin
+
+    def _ourimport(
+        name,
+        globals=None,
+        locals=None,
+        fromlist=None,
+        level=-1 if sys.version_info[0] < 3 else 0,
+    ):
         builtins.__import__ = original_import
 
+        # Singleton, pylint: disable=global-statement
         global _indentation
         try:
             _indentation += 1
 
-            print(_indentation * " " + "called with: name=%r level=%d fromlist=%s" % (name, level, fromlist))
+            print(
+                _indentation * " "
+                + "called with: name=%r level=%d fromlist=%s" % (name, level, fromlist)
+            )
 
             for entry in traceback.extract_stack()[:-1]:
                 if entry[2] == "_ourimport":
@@ -104,10 +132,12 @@ def enableImportTracing(normalize_paths = True, show_source = False):
             return result
         finally:
             _indentation -= 1
+
     try:
         import __builtin__ as builtins
     except ImportError:
         import builtins
 
     import traceback
+
     builtins.__import__ = _ourimport
