@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -33,7 +33,7 @@ from nuitka.finalizations.FinalizeMarkups import getImportedNames
 from nuitka.freezer.IncludedEntryPoints import (
     addIncludedEntryPoints,
     addShlibEntryPoint,
-    getStandardEntryPoints,
+    getStandaloneEntryPoints,
     setMainEntryPoint,
 )
 from nuitka.freezer.Standalone import copyDataFiles
@@ -50,12 +50,11 @@ from nuitka.PythonVersions import (
 from nuitka.Tracing import general, inclusion_logger
 from nuitka.tree import SyntaxErrors
 from nuitka.utils import Execution, InstanceCounters, MemoryUsage, Utils
-from nuitka.utils.AppDirs import getCacheDir
 from nuitka.utils.FileOperations import (
     deleteFile,
-    hasFilenameExtension,
-    listDir,
+    getDirectoryRealPath,
     makePath,
+    putTextFileContents,
     removeDirectory,
 )
 from nuitka.utils.Importing import getSharedLibrarySuffix
@@ -65,12 +64,13 @@ from . import ModuleRegistry, Options, OutputDirectories, TreeXML
 from .build import SconsInterface
 from .codegen import CodeGeneration, LoaderCodes, Reports
 from .finalizations import Finalization
+from .freezer.Onefile import packDistFolderToOnefile
 from .freezer.Standalone import copyUsedDLLs
 from .optimizations import Optimization
 from .tree import Building
 
 
-def createNodeTree(filename):
+def _createNodeTree(filename):
     """Create a node tree.
 
     Turn that source code into a node tree structure. If recursion into
@@ -93,7 +93,7 @@ def createNodeTree(filename):
     source_dir = OutputDirectories.getSourceDirectoryPath()
 
     if not Options.shallOnlyExecCCompilerCall():
-        cleanSourceDirectory(source_dir)
+        SconsInterface.cleanSconsDirectory(source_dir)
 
     # Prepare the ".dist" directory, throwing away what was there before.
     if Options.isStandaloneMode():
@@ -120,7 +120,10 @@ def createNodeTree(filename):
         )
 
         if kind != "absolute":
-            sys.exit("Error, failed to locate package %r." % package_name)
+            inclusion_logger.sysexit(
+                "Error, failed to locate package %r you asked to include."
+                % package_name
+            )
 
         Recursion.checkPluginPath(
             plugin_filename=package_directory, module_package=package_package
@@ -136,12 +139,17 @@ def createNodeTree(filename):
         )
 
         if kind != "absolute":
-            sys.exit("Error, failed to locate module %r." % module_name)
+            inclusion_logger.sysexit(
+                "Error, failed to locate module %r you asked to include." % module_name
+            )
 
         Recursion.checkPluginSinglePath(
             plugin_filename=module_filename, module_package=module_package
         )
 
+    # Allow plugins to add more modules based on the initial set being complete.
+    Plugins.onModuleInitialSet()
+
     # Then optimize the tree and potentially recursed modules.
     Optimization.optimize(main_module.getOutputFilename())
 
@@ -161,46 +169,6 @@ def dumpTreeXML(tree):
     TreeXML.dump(xml_root)
 
 
-def cleanSourceDirectory(source_dir):
-    extensions = (
-        ".bin",
-        ".c",
-        ".cpp",
-        ".exp",
-        ".h",
-        ".lib",
-        ".manifest",
-        ".o",
-        ".obj",
-        ".os",
-        ".rc",
-        ".res",
-        ".S",
-        ".txt",
-        ".const",
-    )
-
-    def check(path):
-        if hasFilenameExtension(path, extensions):
-            deleteFile(path, must_exist=True)
-
-    if os.path.isdir(source_dir):
-        for path, _filename in listDir(source_dir):
-            check(path)
-
-        static_dir = os.path.join(source_dir, "static_src")
-
-        if os.path.exists(static_dir):
-            for path, _filename in listDir(static_dir):
-                check(path)
-
-        plugins_dir = os.path.join(source_dir, "plugins")
-
-        if os.path.exists(plugins_dir):
-            for path, _filename in listDir(plugins_dir):
-                check(path)
-
-
 def pickSourceFilenames(source_dir, modules):
     """Pick the names for the C files of each module.
 
@@ -276,13 +244,11 @@ def pickSourceFilenames(source_dir, modules):
 standalone_entry_points = []
 
 
-def makeSourceDirectory(main_module):
+def makeSourceDirectory():
     """Get the full list of modules imported, create code for all of them."""
     # We deal with a lot of details here, but rather one by one, and split makes
     # no sense, pylint: disable=too-many-branches
 
-    assert main_module.isCompiledPythonModule()
-
     # assert main_module in ModuleRegistry.getDoneModules()
 
     # We might have chosen to include it as bytecode, and only compiled it for
@@ -402,33 +368,27 @@ def makeSourceDirectory(main_module):
         )
 
 
-def _asBoolStr(value):
-    return "true" if value else "false"
-
-
-def runScons(main_module, quiet):
+def runSconsBackend(quiet):
     # Scons gets transported many details, that we express as variables, and
     # have checks for them, leading to many branches and statements,
     # pylint: disable=too-many-branches,too-many-statements
 
+    asBoolStr = SconsInterface.asBoolStr
+
     options = {
-        "name": os.path.basename(
-            OutputDirectories.getTreeFilenameWithSuffix(main_module, "")
-        ),
-        "result_name": OutputDirectories.getResultBasepath(),
+        "result_name": OutputDirectories.getResultBasepath(onefile=False),
         "source_dir": OutputDirectories.getSourceDirectoryPath(),
-        "debug_mode": _asBoolStr(Options.isDebug()),
-        "python_debug": _asBoolStr(Options.isPythonDebug()),
-        "unstripped_mode": _asBoolStr(Options.isUnstripped()),
-        "module_mode": _asBoolStr(Options.shallMakeModule()),
-        "full_compat": _asBoolStr(Options.isFullCompat()),
+        "debug_mode": asBoolStr(Options.is_debug),
+        "python_debug": asBoolStr(Options.isPythonDebug()),
+        "unstripped_mode": asBoolStr(Options.isUnstripped()),
+        "module_mode": asBoolStr(Options.shallMakeModule()),
+        "full_compat": asBoolStr(Options.is_fullcompat),
         "experimental": ",".join(Options.getExperimentalIndications()),
-        "trace_mode": _asBoolStr(Options.shallTraceExecution()),
+        "trace_mode": asBoolStr(Options.shallTraceExecution()),
         "python_version": python_version_str,
         "target_arch": Utils.getArchitecture(),
-        "python_prefix": sys.prefix,
+        "python_prefix": getDirectoryRealPath(sys.prefix),
         "nuitka_src": SconsInterface.getSconsDataPath(),
-        "nuitka_cache": getCacheDir(),
         "module_count": "%d"
         % (
             1
@@ -446,19 +406,19 @@ def runScons(main_module, quiet):
         options["cache_mode"] = "true"
 
     if Options.isLto():
-        options["lto_mode"] = "true"
+        options["lto_mode"] = asBoolStr(True)
 
     if Options.shallUseStaticLibPython():
-        options["static_libpython"] = "true"
+        options["static_libpython"] = asBoolStr(True)
 
     if Options.shallDisableConsoleWindow():
-        options["win_disable_console"] = "true"
+        options["win_disable_console"] = asBoolStr(True)
 
     if Options.isStandaloneMode():
-        options["standalone_mode"] = "true"
+        options["standalone_mode"] = asBoolStr(True)
 
     if Options.shallTreatUninstalledPython():
-        options["uninstalled_python"] = "true"
+        options["uninstalled_python"] = asBoolStr(True)
 
     if ModuleRegistry.getUncompiledTechnicalModules():
         options["frozen_modules"] = str(
@@ -466,10 +426,10 @@ def runScons(main_module, quiet):
         )
 
     if Options.isShowScons():
-        options["show_scons"] = "true"
+        options["show_scons"] = asBoolStr(True)
 
     if Options.isMingw64():
-        options["mingw_mode"] = "true"
+        options["mingw_mode"] = asBoolStr(True)
 
     if Options.getMsvcVersion():
         msvc_version = Options.getMsvcVersion()
@@ -481,46 +441,45 @@ def runScons(main_module, quiet):
         options["msvc_version"] = msvc_version
 
     if Utils.getOS() == "Windows":
-        options["noelf_mode"] = "true"
+        options["noelf_mode"] = asBoolStr(True)
 
     if Options.isClang():
-        options["clang_mode"] = "true"
-
-    if Options.getIconPath():
-        options["icon_path"] = Options.getIconPath()
+        options["clang_mode"] = asBoolStr(True)
 
     if Options.isProfile():
-        options["profile_mode"] = "true"
+        options["profile_mode"] = asBoolStr(True)
 
     if "no_warnings" in getPythonFlags():
-        options["no_python_warnings"] = "true"
+        options["no_python_warnings"] = asBoolStr(True)
 
     if "no_asserts" in getPythonFlags():
-        options["python_sysflag_optimize"] = "true"
+        options["python_sysflag_optimize"] = asBoolStr(True)
 
-    if python_version < 300 and sys.flags.py3k_warning:
-        options["python_sysflag_py3k_warning"] = "true"
+    if python_version < 0x300 and sys.flags.py3k_warning:
+        options["python_sysflag_py3k_warning"] = asBoolStr(True)
 
-    if python_version < 300 and (sys.flags.division_warning or sys.flags.py3k_warning):
-        options["python_sysflag_division_warning"] = "true"
+    if python_version < 0x300 and (
+        sys.flags.division_warning or sys.flags.py3k_warning
+    ):
+        options["python_sysflag_division_warning"] = asBoolStr(True)
 
     if sys.flags.bytes_warning:
-        options["python_sysflag_bytes_warning"] = "true"
+        options["python_sysflag_bytes_warning"] = asBoolStr(True)
 
     if int(os.environ.get("NUITKA_SITE_FLAG", "no_site" in Options.getPythonFlags())):
-        options["python_sysflag_no_site"] = "true"
+        options["python_sysflag_no_site"] = asBoolStr(True)
 
     if "trace_imports" in Options.getPythonFlags():
-        options["python_sysflag_verbose"] = "true"
+        options["python_sysflag_verbose"] = asBoolStr(True)
 
     if "no_randomization" in Options.getPythonFlags():
-        options["python_sysflag_no_randomization"] = "true"
+        options["python_sysflag_no_randomization"] = asBoolStr(True)
 
-    if python_version < 300 and sys.flags.unicode:
-        options["python_sysflag_unicode"] = "true"
+    if python_version < 0x300 and sys.flags.unicode:
+        options["python_sysflag_unicode"] = asBoolStr(True)
 
-    if python_version >= 370 and sys.flags.utf8_mode:
-        options["python_sysflag_utf8"] = "true"
+    if python_version >= 0x370 and sys.flags.utf8_mode:
+        options["python_sysflag_utf8"] = asBoolStr(True)
 
     abiflags = getPythonABI()
     if abiflags:
@@ -541,9 +500,17 @@ def runScons(main_module, quiet):
         options["module_suffix"] = getSharedLibrarySuffix(preferred=True)
 
     if Options.shallRunInDebugger():
-        options["full_names"] = "true"
+        options["full_names"] = asBoolStr(True)
 
-    return SconsInterface.runScons(options, quiet), options
+    if Options.assumeYesForDownloads():
+        options["assume_yes_for_downloads"] = asBoolStr(True)
+
+    return (
+        SconsInterface.runScons(
+            options=options, quiet=quiet, scons_filename="Backend.scons"
+        ),
+        options,
+    )
 
 
 def writeSourceCode(filename, source_code):
@@ -551,12 +518,7 @@ def writeSourceCode(filename, source_code):
     # or something else has failed.
     assert not os.path.isfile(filename), filename
 
-    if python_version >= 300:
-        with open(filename, "wb") as output_file:
-            output_file.write(source_code.encode("latin1"))
-    else:
-        with open(filename, "w") as output_file:
-            output_file.write(source_code)
+    putTextFileContents(filename=filename, contents=source_code, encoding="latin1")
 
 
 def writeBinaryData(filename, binary_data):
@@ -571,7 +533,7 @@ def writeBinaryData(filename, binary_data):
 
 
 def callExecPython(args, clean_path, add_path):
-    old_python_path = os.environ.get("PYTHONPATH", None)
+    old_python_path = os.environ.get("PYTHONPATH")
 
     if clean_path and old_python_path is not None:
         os.environ["PYTHONPATH"] = ""
@@ -614,10 +576,14 @@ def executeModule(tree, clean_path):
     callExecPython(clean_path=clean_path, add_path=True, args=args)
 
 
-def compileTree(main_module):
+def compileTree():
     source_dir = OutputDirectories.getSourceDirectoryPath()
 
+    general.info("Completed Python level compilation and optimization.")
+
     if not Options.shallOnlyExecCCompilerCall():
+        general.info("Generating source code for C backend compiler.")
+
         if Options.isShowProgress() or Options.isShowMemory():
             general.info(
                 "Total memory usage before generating C code: {memory}:".format(
@@ -625,7 +591,7 @@ def compileTree(main_module):
                 )
             )
         # Now build the target language code for the whole tree.
-        makeSourceDirectory(main_module=main_module)
+        makeSourceDirectory()
 
         bytecode_accessor = ConstantAccessor(
             data_filename="__bytecode.const", top_level_name="bytecode_data"
@@ -643,7 +609,7 @@ def compileTree(main_module):
         source_dir = OutputDirectories.getSourceDirectoryPath()
 
         if not os.path.isfile(os.path.join(source_dir, "__helpers.h")):
-            sys.exit("Error, no previous build directory exists.")
+            general.sysexit("Error, no previous build directory exists.")
 
     if Options.isShowProgress() or Options.isShowMemory():
         general.info(
@@ -655,18 +621,22 @@ def compileTree(main_module):
     if Options.isShowMemory():
         InstanceCounters.printStats()
 
-    if Options.isDebug():
+    if Options.is_debug:
         Reports.doMissingOptimizationReport()
 
     if Options.shallNotDoExecCCompilerCall():
         return True, {}
 
+    general.info("Running data composer tool for optimal constant value handling.")
+
     # TODO: On Windows, we could run this in parallel to Scons, on Linux we need it
     # for linking.
     runDataComposer(source_dir)
 
+    general.info("Running C level backend compilation via Scons.")
+
     # Run the Scons to build things.
-    result, options = runScons(main_module=main_module, quiet=not Options.isShowScons())
+    result, options = runSconsBackend(quiet=not Options.isShowScons())
 
     return result, options
 
@@ -677,8 +647,8 @@ def handleSyntaxError(e):
     # versions he wants, tell him about the potential version problem.
     error_message = SyntaxErrors.formatOutput(e)
 
-    if not Options.isFullCompat():
-        if python_version < 300:
+    if not Options.is_fullcompat:
+        if python_version < 0x300:
             suggested_python_version_str = getSupportedPythonVersions()[-1]
         else:
             suggested_python_version_str = "2.7"
@@ -709,7 +679,10 @@ def main():
     """
 
     # Main has to fulfill many options, leading to many branches and statements
-    # to deal with them.  pylint: disable=too-many-branches
+    # to deal with them.  pylint: disable=too-many-branches,too-many-statements
+    if not Options.shallDumpBuiltTreeXML():
+        general.info("Starting Python compilation.")
+
     filename = Options.getPositionalArgs()[0]
 
     # Inform the importing layer about the main script directory, so it can use
@@ -720,7 +693,7 @@ def main():
 
     # Turn that source code into a node tree structure.
     try:
-        main_module = createNodeTree(filename=filename)
+        main_module = _createNodeTree(filename=filename)
     except (SyntaxError, IndentationError) as e:
         handleSyntaxError(e)
 
@@ -730,7 +703,7 @@ def main():
             dumpTreeXML(module)
     else:
         # Make the actual compilation.
-        result, options = compileTree(main_module=main_module)
+        result, options = compileTree()
 
         # Exit if compilation failed.
         if not result:
@@ -744,38 +717,12 @@ def main():
 
         executePostProcessing()
 
-        if Options.isStandaloneMode():
-            binary_filename = options["result_exe"]
-
-            setMainEntryPoint(binary_filename)
-
-            dist_dir = OutputDirectories.getStandaloneDirectoryPath()
-
-            for module in ModuleRegistry.getDoneModules():
-                addIncludedEntryPoints(Plugins.considerExtraDlls(dist_dir, module))
-
-            copyUsedDLLs(
-                source_dir=OutputDirectories.getSourceDirectoryPath(),
-                dist_dir=dist_dir,
-                standalone_entry_points=getStandardEntryPoints(),
-            )
-
-            copyDataFiles(dist_dir=dist_dir)
-
-            Plugins.onStandaloneDistributionFinished(dist_dir)
-
-        # Remove the source directory (now build directory too) if asked to.
-        if Options.isRemoveBuildDir():
-            removeDirectory(
-                path=OutputDirectories.getSourceDirectoryPath(), ignore_errors=False
-            )
-
         if Options.shallMakeModule() and Options.shallCreatePyiFile():
             pyi_filename = OutputDirectories.getResultBasepath() + ".pyi"
 
-            with open(pyi_filename, "w") as pyi_file:
-                pyi_file.write(
-                    """\
+            putTextFileContents(
+                filename=pyi_filename,
+                contents="""\
 # This file was generated by Nuitka and describes the types of the
 # created shared library.
 
@@ -795,13 +742,64 @@ def main():
 __name__ = ...
 
 """
-                    % {
-                        "imports": "\n".join(
-                            "import %s" % module_name
-                            for module_name in getImportedNames()
-                        )
-                    }
-                )
+                % {
+                    "imports": "\n".join(
+                        "import %s" % module_name for module_name in getImportedNames()
+                    )
+                },
+            )
+
+        if Options.isStandaloneMode():
+            binary_filename = options["result_exe"]
+
+            setMainEntryPoint(binary_filename)
+
+            dist_dir = OutputDirectories.getStandaloneDirectoryPath()
+
+            for module in ModuleRegistry.getDoneModules():
+                addIncludedEntryPoints(Plugins.considerExtraDlls(dist_dir, module))
+
+            copyUsedDLLs(
+                source_dir=OutputDirectories.getSourceDirectoryPath(),
+                dist_dir=dist_dir,
+                standalone_entry_points=getStandaloneEntryPoints(),
+            )
+
+            copyDataFiles(dist_dir=dist_dir)
+
+            Plugins.onStandaloneDistributionFinished(dist_dir)
+
+            if Options.isOnefileMode():
+                packDistFolderToOnefile(dist_dir, binary_filename)
+
+                if Options.isRemoveBuildDir():
+                    general.info("Removing dist folder %r." % dist_dir)
+
+                    removeDirectory(path=dist_dir, ignore_errors=False)
+                else:
+                    general.info(
+                        "Keeping dist folder %r for inspection, no need to use it."
+                        % dist_dir
+                    )
+
+        # Remove the source directory (now build directory too) if asked to.
+        source_dir = OutputDirectories.getSourceDirectoryPath()
+
+        if Options.isRemoveBuildDir():
+            general.info("Removing build directory %r." % source_dir)
+
+            removeDirectory(path=source_dir, ignore_errors=False)
+            assert not os.path.exists(source_dir)
+        else:
+            general.info("Keeping build directory %r." % source_dir)
+
+        final_filename = OutputDirectories.getResultFullpath(
+            onefile=Options.isOnefileMode()
+        )
+
+        Plugins.onFinalResult(final_filename)
+
+        general.info("Successfully created %r." % final_filename)
 
         # Execute the module immediately if option was given.
         if Options.shallExecuteImmediately():
@@ -812,6 +810,8 @@ __name__ = ...
                 )
             else:
                 executeMain(
-                    binary_filename=OutputDirectories.getResultFullpath(),
+                    binary_filename=OutputDirectories.getResultFullpath(
+                        onefile=Options.isOnefileMode()
+                    ),
                     clean_path=Options.shallClearPythonPathEnvironment(),
                 )
