@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -49,6 +49,9 @@ class SourceCodeReference(object):
     def __repr__(self):
         return "<%s to %s:%s>" % (self.__class__.__name__, self.filename, self.line)
 
+    def __hash__(self):
+        return hash((self.filename, self.line, self.column))
+
     def __lt__(self, other):
         # Many cases decide early, pylint: disable=too-many-return-statements
         if other is None:
@@ -97,19 +100,14 @@ class SourceCodeReference(object):
         return self.isInternal() is other.isInternal()
 
     def _clone(self, line):
-        """ Make a copy it itself.
-
-        """
-        return self.fromFilenameAndLine(
-            filename = self.filename,
-            line     = line
-        )
+        """Make a copy it itself."""
+        return self.fromFilenameAndLine(filename=self.filename, line=line)
 
     def atInternal(self):
-        """ Make a copy it itself but mark as internal code.
+        """Make a copy it itself but mark as internal code.
 
-            Avoids useless copies, by returning an internal object again if
-            it is already internal.
+        Avoids useless copies, by returning an internal object again if
+        it is already internal.
         """
         if not self.isInternal():
             result = self._clone(self.line)
@@ -118,12 +116,11 @@ class SourceCodeReference(object):
         else:
             return self
 
-
     def atLineNumber(self, line):
-        """ Make a reference to the same file, but different line.
+        """Make a reference to the same file, but different line.
 
-            Avoids useless copies, by returning same object if the line is
-            the same.
+        Avoids useless copies, by returning same object if the line is
+        the same.
         """
 
         assert type(line) is int, line
@@ -175,7 +172,4 @@ class SourceCodeReferenceInternal(SourceCodeReference):
 
 
 def fromFilename(filename):
-    return SourceCodeReference.fromFilenameAndLine(
-        filename = filename,
-        line     = 1
-    )
+    return SourceCodeReference.fromFilenameAndLine(filename=filename, line=1)
