@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,18 +23,28 @@ module variable references.
 
 """
 
-from nuitka.nodes.shapes.StandardShapes import ShapeUnknown
+from abc import abstractmethod
+
+from nuitka.__past__ import getMetaClassBase, iterItems
+from nuitka.nodes.shapes.StandardShapes import tshape_unknown
 from nuitka.utils import InstanceCounters, Utils
 
 complete = False
 
-class Variable(object):
+
+class Variable(getMetaClassBase("Variable")):
 
     # We will need all of these attributes, since we track the global
-    # state and cache some decisions as attributes, pylint: disable=too-many-instance-attributes
+    # state and cache some decisions as attributes. TODO: But in some
+    # cases, part of the these might be moved to the outside.
     __slots__ = (
-        "variable_name", "owner", "version_number", "shared_users", "shared_scopes",
-        "traces", "users", "writers"
+        "variable_name",
+        "owner",
+        "version_number",
+        "shared_users",
+        "traces",
+        "users",
+        "writers",
     )
 
     @InstanceCounters.counted_init
@@ -48,7 +58,6 @@ class Variable(object):
         self.version_number = 0
 
         self.shared_users = False
-        self.shared_scopes = False
 
         self.traces = set()
 
@@ -58,6 +67,23 @@ class Variable(object):
 
     __del__ = InstanceCounters.counted_del()
 
+    def finalize(self):
+        del self.users
+        del self.writers
+        del self.traces
+        del self.owner
+
+    def __repr__(self):
+        return "<%s '%s' of '%s'>" % (
+            self.__class__.__name__,
+            self.variable_name,
+            self.owner.getName(),
+        )
+
+    @abstractmethod
+    def getVariableType(self):
+        pass
+
     def getDescription(self):
         return "variable '%s'" % self.variable_name
 
@@ -72,7 +98,7 @@ class Variable(object):
 
     def getCodeName(self):
         var_name = self.variable_name
-        var_name = var_name.replace('.', '$')
+        var_name = var_name.replace(".", "$")
         var_name = Utils.encodeNonAscii(var_name)
 
         return var_name
@@ -82,36 +108,45 @@ class Variable(object):
 
         return self.version_number
 
-    # pylint: disable=no-self-use
-    def isLocalVariable(self):
+    @staticmethod
+    def isLocalVariable():
         return False
 
-    def isParameterVariable(self):
+    @staticmethod
+    def isParameterVariable():
         return False
 
-    def isModuleVariable(self):
+    @staticmethod
+    def isModuleVariable():
         return False
 
-    def isTempVariable(self):
+    @staticmethod
+    def isTempVariable():
+        return False
+
+    @staticmethod
+    def isTempVariableBool():
+        return False
+
+    @staticmethod
+    def isLocalsDictVariable():
         return False
-    # pylint: enable=R0201
 
     def addVariableUser(self, user):
         # Update the shared scopes flag.
         if user is not self.owner:
-            # These are not really scopes.
             self.shared_users = True
 
-            if user.isExpressionGeneratorObjectBody() or \
-               user.isExpressionCoroutineObjectBody() or \
-               user.isExpressionAsyncgenObjectBody():
+            # These are not really scopes, just shared uses.
+            if (
+                user.isExpressionGeneratorObjectBody()
+                or user.isExpressionCoroutineObjectBody()
+                or user.isExpressionAsyncgenObjectBody()
+            ):
                 if self.owner is user.getParentVariableProvider():
                     return
 
-            self.shared_scopes = True
-
-    def isSharedAmongScopes(self):
-        return self.shared_scopes
+            _variables_in_shared_scopes.add(self)
 
     def isSharedTechnically(self):
         if not self.shared_users:
@@ -128,11 +163,10 @@ class Variable(object):
         for user in self.users:
             user = user.getEntryPoint()
 
-            while user is not owner and \
-                  (
-                   (user.isExpressionFunctionBody() and not user.needsCreation()) or \
-                   user.isExpressionClassBody()
-                  ):
+            while user is not owner and (
+                (user.isExpressionFunctionBody() and not user.needsCreation())
+                or user.isExpressionClassBody()
+            ):
                 user = user.getParentVariableProvider()
 
             if user is not owner:
@@ -144,10 +178,6 @@ class Variable(object):
         self.traces.add(variable_trace)
 
     def removeTrace(self, variable_trace):
-        # Make it unusable, and break GC cycles while at it.
-        variable_trace.variable = None
-        variable_trace.previous = None
-
         self.traces.remove(variable_trace)
 
     def updateUsageState(self):
@@ -160,6 +190,8 @@ class Variable(object):
 
             if trace.isAssignTrace():
                 writers.add(owner)
+            elif trace.isDeletedTrace() and owner is not self.owner:
+                writers.add(owner)
 
         self.writers = writers
         self.users = users
@@ -167,7 +199,7 @@ class Variable(object):
     def hasWritesOutsideOf(self, user):
         if not complete:
             return None
-        elif user in self.writers:
+        elif self.writers is not None and user in self.writers:
             return len(self.writers) > 1
         else:
             return bool(self.writers)
@@ -195,23 +227,30 @@ class Variable(object):
 
         return None
 
+    def getMatchingDelTrace(self, del_node):
+        for trace in self.traces:
+            if trace.isDeletedTrace() and trace.getDelNode() is del_node:
+                return trace
+
+        return None
+
     def getTypeShapes(self):
         result = set()
 
         for trace in self.traces:
             if trace.isAssignTrace():
-                result.add(
-                    trace.getAssignNode().getAssignSource().getTypeShape()
-                )
+                result.add(trace.getAssignNode().getTypeShape())
             elif trace.isUnknownTrace():
-                result.add(ShapeUnknown)
-            elif trace.isUninitTrace():
-                if trace.hasDefiniteUsages() or trace.hasPotentialUsages():
-                    result.add(ShapeUnknown)
+                result.add(tshape_unknown)
             elif trace.isInitTrace():
-                result.add(ShapeUnknown)
+                result.add(tshape_unknown)
+            elif trace.isUnassignedTrace():
+                pass
             elif trace.isMergeTrace():
                 pass
+            # TODO: Remove this and be not unknown.
+            elif trace.isLoopTrace():
+                trace.getTypeShape().emitAlternatives(result.add)
             else:
                 assert False, trace
 
@@ -222,37 +261,28 @@ class LocalVariable(Variable):
     __slots__ = ()
 
     def __init__(self, owner, variable_name):
-        Variable.__init__(
-            self,
-            owner         = owner,
-            variable_name = variable_name
-        )
+        Variable.__init__(self, owner=owner, variable_name=variable_name)
 
-    def __repr__(self):
-        return "<%s '%s' of '%s'>" % (
-            self.__class__.__name__,
-            self.variable_name,
-            self.owner.getName()
-        )
-
-    def isLocalVariable(self):
+    @staticmethod
+    def isLocalVariable():
         return True
 
+    @staticmethod
+    def getVariableType():
+        return "object"
+
 
 class ParameterVariable(LocalVariable):
     __slots__ = ()
 
     def __init__(self, owner, parameter_name):
-        LocalVariable.__init__(
-            self,
-            owner         = owner,
-            variable_name = parameter_name
-        )
+        LocalVariable.__init__(self, owner=owner, variable_name=parameter_name)
 
     def getDescription(self):
         return "parameter variable '%s'" % self.variable_name
 
-    def isParameterVariable(self):
+    @staticmethod
+    def isParameterVariable():
         return True
 
 
@@ -263,72 +293,106 @@ class ModuleVariable(Variable):
         assert type(variable_name) is str, repr(variable_name)
         assert module.isCompiledPythonModule()
 
-        Variable.__init__(
-            self,
-            owner         = module,
-            variable_name = variable_name
-        )
+        Variable.__init__(self, owner=module, variable_name=variable_name)
 
         self.module = module
 
     def __repr__(self):
         return "<ModuleVariable '%s' of '%s'>" % (
             self.variable_name,
-            self.getModule().getFullName()
+            self.module.getFullName(),
         )
 
     def getDescription(self):
         return "global variable '%s'" % self.variable_name
 
-    def isModuleVariable(self):
+    @staticmethod
+    def isModuleVariable():
         return True
 
     def getModule(self):
         return self.module
 
+    @staticmethod
+    def getVariableType():
+        return "object"
+
 
 class TempVariable(Variable):
     __slots__ = ()
 
     def __init__(self, owner, variable_name):
-        Variable.__init__(
-            self,
-            owner         = owner,
-            variable_name = variable_name
-        )
-
-    def __repr__(self):
-        return "<TempVariable '%s' of '%s'>" % (
-            self.getName(),
-            self.getOwner().getName()
-        )
+        Variable.__init__(self, owner=owner, variable_name=variable_name)
 
     def getDescription(self):
         return "temp variable '%s'" % self.variable_name
 
-    def isTempVariable(self):
+    @staticmethod
+    def isTempVariable():
         return True
 
+    @staticmethod
+    def getVariableType():
+        return "object"
 
-def updateVariablesFromCollection(old_collection, new_collection):
+
+class TempVariableBool(TempVariable):
+    __slots__ = ()
+
+    def getDescription(self):
+        return "temp bool variable '%s'" % self.variable_name
+
+    @staticmethod
+    def isTempVariableBool():
+        return True
+
+    @staticmethod
+    def getVariableType():
+        return "bool"
+
+
+class LocalsDictVariable(Variable):
+    __slots__ = ()
+
+    def __init__(self, owner, variable_name):
+        Variable.__init__(self, owner=owner, variable_name=variable_name)
+
+    @staticmethod
+    def isLocalsDictVariable():
+        return True
+
+    @staticmethod
+    def getVariableType():
+        return "object"
+
+
+def updateVariablesFromCollection(old_collection, new_collection, source_ref):
     # After removing/adding traces, we need to pre-compute the users state
     # information.
     touched_variables = set()
+    loop_trace_removal = set()
 
     if old_collection is not None:
-        for variable_trace in old_collection.getVariableTracesAll().values():
-            variable = variable_trace.getVariable()
-
+        for (variable, _version), variable_trace in iterItems(
+            old_collection.getVariableTracesAll()
+        ):
             variable.removeTrace(variable_trace)
             touched_variables.add(variable)
 
-    if new_collection is not None:
-        for variable_trace in new_collection.getVariableTracesAll().values():
-            variable = variable_trace.getVariable()
+            if variable_trace.isLoopTrace():
+                loop_trace_removal.add(variable)
 
+    if new_collection is not None:
+        for (variable, _version), variable_trace in iterItems(
+            new_collection.getVariableTracesAll()
+        ):
             variable.addTrace(variable_trace)
             touched_variables.add(variable)
 
+            if variable_trace.isLoopTrace():
+                if variable in loop_trace_removal:
+                    loop_trace_removal.remove(variable)
+
         # Release the memory, and prevent the "active" state from being ever
         # inspected, it's useless now.
         new_collection.variable_actives.clear()
@@ -336,3 +400,34 @@ def updateVariablesFromCollection(old_collection, new_collection):
 
     for variable in touched_variables:
         variable.updateUsageState()
+
+    if loop_trace_removal:
+        if new_collection is not None:
+            new_collection.signalChange(
+                "var_usage",
+                source_ref,
+                lambda: "Loop variable '%s' usage ceased."
+                % ",".join(variable.getName() for variable in loop_trace_removal),
+            )
+
+
+# To detect the Python2 shared variable deletion, that would be a syntax
+# error
+_variables_in_shared_scopes = set()
+
+
+def isSharedAmongScopes(variable):
+    return variable in _variables_in_shared_scopes
+
+
+def releaseSharedScopeInformation(tree):
+    # Singleton, pylint: disable=global-statement
+
+    assert tree.isCompiledPythonModule()
+
+    global _variables_in_shared_scopes
+    _variables_in_shared_scopes = set(
+        variable
+        for variable in _variables_in_shared_scopes
+        if variable.getOwner().getParentModule() is not tree
+    )
