@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,52 +26,70 @@ be a "in (str, unicode)" rather than making useless version checks.
 """
 
 import sys
+from abc import ABCMeta
 
-# pylint: disable=I0021,invalid-name,redefined-builtin
+# pylint: disable=I0021,invalid-name,redefined-builtin,self-assigning-variable
 
 if str is bytes:
-    import __builtin__ as builtins  # @UnresolvedImport @UnusedImport pylint: disable=I0021,import-error
+    import __builtin__ as builtins  # pylint: disable=I0021,import-error
 else:
-    import builtins  # @UnresolvedImport @Reimport pylint: disable=I0021,import-error
+    import builtins  # pylint: disable=I0021,import-error
 
 # Work around for CPython 3.x renaming "long" to "int".
 if str is bytes:
-    long = long  # @ReservedAssignment pylint: disable=I0021,undefined-variable
+    long = long  # pylint: disable=I0021,undefined-variable
 else:
-    long = int   # @ReservedAssignment
+    long = int
 
 # Work around for CPython 3.x renaming "unicode" to "str".
 if str is bytes:
-    unicode = unicode  # @ReservedAssignment pylint: disable=I0021,undefined-variable
+    unicode = unicode  # pylint: disable=I0021,undefined-variable
 else:
-    unicode = str      # @ReservedAssignment
+    unicode = str
 
-def iterItems(d):
-    try:
+
+if str is bytes:
+
+    def iterItems(d):
         return d.iteritems()
-    except AttributeError:
+
+
+else:
+
+    def iterItems(d):
         return d.items()
 
+
 if str is not bytes:
-    raw_input = input      # @ReservedAssignment
+    raw_input = input
+    xrange = range
+    basestring = str
 else:
-    raw_input = raw_input  # @ReservedAssignment
-
-if str is bytes:
-    xrange = xrange # @ReservedAssignment pylint: disable=I0021,undefined-variable
-else:
-    xrange = range  # @ReservedAssignment
+    raw_input = raw_input
+    xrange = xrange  # pylint: disable=I0021,undefined-variable
+    basestring = basestring  # pylint: disable=I0021,undefined-variable
 
 
 if str is bytes:
-    from urllib import urlretrieve # @UnresolvedImport @UnusedImport pylint: disable=I0021,import-error,no-name-in-module
+    from urllib import (  # pylint: disable=I0021,import-error,no-name-in-module
+        urlretrieve,
+    )
 else:
-    from urllib.request import urlretrieve # @UnresolvedImport @Reimport pylint: disable=I0021,import-error,no-name-in-module
+    from urllib.request import (  # pylint: disable=I0021,import-error,no-name-in-module
+        urlretrieve,
+    )
 
 if str is bytes:
-    from cStringIO import StringIO # @UnresolvedImport @UnusedImport pylint: disable=I0021,import-error
+    from cStringIO import (  # pylint: disable=I0021,import-error,ungrouped-imports
+        StringIO,
+    )
 else:
-    from io import StringIO # @UnresolvedImport @Reimport pylint: disable=I0021,import-error
+    from io import StringIO  # pylint: disable=I0021,import-error,ungrouped-imports
+
+if str is bytes:
+    BytesIO = StringIO
+else:
+    from io import BytesIO  # pylint: disable=I0021,import-error
 
 try:
     from functools import total_ordering
@@ -86,17 +104,54 @@ except ImportError:
 
         return cls
 
+
 if str is bytes:
-    intern = intern      # @ReservedAssignment pylint: disable=I0021,undefined-variable
+    from collections import (  # pylint: disable=I0021,import-error,no-name-in-module
+        Iterable,
+        MutableSet,
+    )
 else:
-    intern = sys.intern  # @ReservedAssignment @UndefinedVariable
+    from collections.abc import (  # pylint: disable=I0021,import-error,import-error,no-name-in-module
+        Iterable,
+        MutableSet,
+    )
+
+if str is bytes:
+    intern = intern  # pylint: disable=I0021,undefined-variable
+else:
+    intern = sys.intern
+
+if str is bytes:
+    to_byte = chr
+else:
+
+    def to_byte(value):
+        assert type(value) is int and 0 <= value < 256
+        return bytes((value,))
+
+
+def getMetaClassBase(meta_class_prefix):
+    """For Python2/3 compatible source, we create a base class that has the metaclass
+    used and doesn't require making a choice.
+    """
+
+    class MetaClass(ABCMeta):
+        pass
+
+    MetaClassBase = MetaClass("%sMetaClassBase" % meta_class_prefix, (object,), {})
+
+    return MetaClassBase
+
 
 # For PyLint to be happy.
 assert long
 assert unicode
 assert urlretrieve
 assert StringIO
+assert BytesIO
 assert type(xrange) is type, xrange
 assert total_ordering
 assert intern
 assert builtins
+assert Iterable
+assert MutableSet
