@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -31,11 +31,12 @@ And releasing of values, as this is what the error case commonly does.
 from nuitka.PythonVersions import python_version
 
 from .ExceptionCodes import getExceptionIdentifier
-from .Indentation import getCommentCode, indented
+from .Indentation import indented
 from .LineNumberCodes import getErrorLineNumberUpdateCode
 from .templates.CodeTemplatesExceptions import (
     template_error_catch_exception,
     template_error_catch_quick_exception,
+    template_error_format_name_error_exception,
     template_error_format_string_exception,
 )
 
@@ -91,9 +92,12 @@ def getErrorExitBoolCode(
         getAssertionCode("!(%s)" % condition, emit)
         return
 
-    exception_type, exception_value, exception_tb, _exception_lineno = (
-        context.variable_storage.getExceptionVariableDescriptions()
-    )
+    (
+        exception_type,
+        exception_value,
+        exception_tb,
+        _exception_lineno,
+    ) = context.variable_storage.getExceptionVariableDescriptions()
 
     if quick_exception:
         emit(
@@ -146,7 +150,7 @@ def getErrorExitCode(
     needs_check=True,
 ):
     getErrorExitBoolCode(
-        condition="%s == NULL" % check_name,
+        condition=check_name.getCType().getExceptionCheckCondition(check_name),
         release_names=release_names,
         release_name=release_name,
         needs_check=needs_check,
@@ -156,48 +160,86 @@ def getErrorExitCode(
     )
 
 
-def getErrorFormatExitBoolCode(condition, exception, args, emit, context):
-    assert not condition.endswith(";")
+def _getExceptionChainingCode(context):
+    (
+        exception_type,
+        exception_value,
+        exception_tb,
+        _exception_lineno,
+    ) = context.variable_storage.getExceptionVariableDescriptions()
 
-    exception_type, exception_value, exception_tb, _exception_lineno = (
-        context.variable_storage.getExceptionVariableDescriptions()
+    keeper_vars = context.getExceptionKeeperVariables()
+
+    if keeper_vars[0] is not None:
+        return ("ADD_EXCEPTION_CONTEXT(&%s, &%s);" % (keeper_vars[0], keeper_vars[1]),)
+    else:
+        return (
+            "NORMALIZE_EXCEPTION(&%s, &%s, &%s);"
+            % (exception_type, exception_value, exception_tb),
+            "CHAIN_EXCEPTION(%s);" % exception_value,
+        )
+
+
+def getTakeReferenceCode(value_name, emit):
+    value_name.getCType().getTakeReferenceCode(value_name=value_name, emit=emit)
+
+
+def getReleaseCode(release_name, emit, context):
+    if context.needsCleanup(release_name):
+        release_name.getCType().getReleaseCode(
+            value_name=release_name, needs_check=False, emit=emit
+        )
+
+        context.removeCleanupTempName(release_name)
+
+
+def getReleaseCodes(release_names, emit, context):
+    for release_name in release_names:
+        getReleaseCode(release_name=release_name, emit=emit, context=context)
+
+
+def getMustNotGetHereCode(reason, emit):
+    emit(
+        """\
+NUITKA_CANNOT_GET_HERE("%s");
+return NULL;"""
+        % reason
     )
 
-    if len(args) == 1 and type(args[0]) is str:
-        from .ConstantCodes import getModuleConstantCode
 
-        set_exception = [
-            "%s = %s;" % (exception_type, exception),
-            "Py_INCREF(%s);" % exception_type,
-            "%s = %s;" % (exception_value, getModuleConstantCode(constant=args[0])),
-            "%s = NULL;" % exception_tb,
-        ]
+def getAssertionCode(check, emit):
+    emit("assert(%s);" % check)
+
+
+def getLocalVariableReferenceErrorCode(variable, condition, emit, context):
+    variable_name = variable.getName()
+
+    (
+        exception_type,
+        exception_value,
+        exception_tb,
+        _exception_lineno,
+    ) = context.variable_storage.getExceptionVariableDescriptions()
+
+    if variable.getOwner() is not context.getOwner():
+        helper_code = "FORMAT_UNBOUND_CLOSURE_ERROR"
     else:
-        set_exception = [
-            "%s = %s;" % (exception_type, exception),
-            "Py_INCREF(%s);" % exception_type,
-            "%s = Py%s_FromFormat(%s);"
-            % (
-                exception_value,
-                "String" if python_version < 300 else "Unicode",
-                ", ".join('"%s"' % arg for arg in args),
-            ),
-            "%s = NULL;" % exception_tb,
-        ]
+        helper_code = "FORMAT_UNBOUND_LOCAL_ERROR"
 
-    if python_version >= 300:
-        keeper_vars = context.getExceptionKeeperVariables()
+    set_exception = [
+        "%s(&%s, &%s, %s);"
+        % (
+            helper_code,
+            exception_type,
+            exception_value,
+            context.getConstantCode(variable_name),
+        ),
+        "%s = NULL;" % exception_tb,
+    ]
 
-        if keeper_vars[0] is not None:
-            set_exception.append(
-                "ADD_EXCEPTION_CONTEXT(&%s, &%s);" % (keeper_vars[0], keeper_vars[1])
-            )
-        else:
-            set_exception.append(
-                "NORMALIZE_EXCEPTION(&%s, &%s, &%s);"
-                % (exception_type, exception_value, exception_tb)
-            )
-            set_exception.append("CHAIN_EXCEPTION(%s);" % exception_value)
+    # TODO: Move this into the helper code.
+    if python_version >= 0x300:
+        set_exception.extend(_getExceptionChainingCode(context))
 
     emit(
         template_error_format_string_exception
@@ -214,81 +256,45 @@ def getErrorFormatExitBoolCode(condition, exception, args, emit, context):
     )
 
 
-def getReleaseCode(release_name, emit, context):
-    if context.needsCleanup(release_name):
-        emit("Py_DECREF(%s);" % release_name)
-        context.removeCleanupTempName(release_name)
-
-
-def getReleaseCodes(release_names, emit, context):
-    for release_name in release_names:
-        getReleaseCode(release_name=release_name, emit=emit, context=context)
-
-
-def getMustNotGetHereCode(reason, context, emit):
-    getCommentCode(reason, emit)
-
-    provider = context.getEntryPoint()
-
-    emit(
-        "NUITKA_CANNOT_GET_HERE(%(function_identifier)s);"
-        % {"function_identifier": provider.getCodeName()}
-    )
-    emit("return NULL;")
-
-
-def getAssertionCode(check, emit):
-    emit("assert(%s);" % check)
-
-
-def getCheckObjectCode(check_name, emit):
-    emit("CHECK_OBJECT(%s);" % check_name)
-
-
-def getLocalVariableReferenceErrorCode(variable, condition, emit, context):
-    if variable.getOwner() is not context.getOwner():
-        getErrorFormatExitBoolCode(
-            condition=condition,
-            exception="PyExc_NameError",
-            args=(
-                """\
-free variable '%s' referenced before assignment in enclosing scope""",
-                variable.getName(),
-            ),
-            emit=emit,
-            context=context,
-        )
-    else:
-        getErrorFormatExitBoolCode(
-            condition=condition,
-            exception="PyExc_UnboundLocalError",
-            args=(
-                """\
-local variable '%s' referenced before assignment""",
-                variable.getName(),
-            ),
-            emit=emit,
-            context=context,
-        )
-
-
+# TODO: Get rid of this function entirely.
 def getNameReferenceErrorCode(variable_name, condition, emit, context):
-    if python_version < 340:
+    helper_code = "FORMAT_NAME_ERROR"
+
+    if python_version < 0x340:
         owner = context.getOwner()
 
         if not owner.isCompiledPythonModule() and not owner.isExpressionClassBody():
-            error_message = "global name '%s' is not defined"
-        else:
-            error_message = "name '%s' is not defined"
-    else:
-        error_message = "name '%s' is not defined"
+            helper_code = "FORMAT_GLOBAL_NAME_ERROR"
 
-    error_message = error_message % variable_name
+    (
+        exception_type,
+        exception_value,
+        _exception_tb,
+        _exception_lineno,
+    ) = context.variable_storage.getExceptionVariableDescriptions()
 
-    getErrorFormatExitBoolCode(
-        condition=condition,
-        exception="PyExc_NameError",
-        args=(error_message,),
-        emit=emit,
-        context=context,
+    set_exception = "%s(&%s, &%s, %s);" % (
+        helper_code,
+        exception_type,
+        exception_value,
+        context.getConstantCode(variable_name),
+    )
+
+    # TODO: Make this part of the helper code as well.
+    if python_version >= 0x300:
+        set_exception = [set_exception]
+        set_exception.extend(_getExceptionChainingCode(context))
+
+    emit(
+        template_error_format_name_error_exception
+        % {
+            "condition": condition,
+            "exception_exit": context.getExceptionEscape(),
+            "set_exception": indented(set_exception),
+            "release_temps": indented(getErrorExitReleaseCode(context)),
+            "var_description_code": indented(
+                getFrameVariableTypeDescriptionCode(context)
+            ),
+            "line_number_code": indented(getErrorLineNumberUpdateCode(context)),
+        }
     )
