@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,41 +22,32 @@
 
 template_iterator_check = """\
 // Check if iterator has left-over elements.
-CHECK_OBJECT( %(iterator_name)s ); assert( HAS_ITERNEXT( %(iterator_name)s ) );
+CHECK_OBJECT(%(iterator_name)s); assert(HAS_ITERNEXT(%(iterator_name)s));
 
-%(attempt_name)s = (*Py_TYPE( %(iterator_name)s )->tp_iternext)( %(iterator_name)s );
+%(attempt_name)s = (*Py_TYPE(%(iterator_name)s)->tp_iternext)(%(iterator_name)s);
 
-if (likely( %(attempt_name)s == NULL ))
-{
+if (likely(%(attempt_name)s == NULL)) {
     PyObject *error = GET_ERROR_OCCURRED();
 
-    if ( error != NULL )
-    {
-        if ( EXCEPTION_MATCH_BOOL_SINGLE( error, PyExc_StopIteration ))
-        {
+    if (error != NULL) {
+        if (EXCEPTION_MATCH_BOOL_SINGLE(error, PyExc_StopIteration)) {
             CLEAR_ERROR_OCCURRED();
-        }
-        else
-        {
-            FETCH_ERROR_OCCURRED( &%(exception_type)s, &%(exception_value)s, &%(exception_tb)s );
+        } else {
+            FETCH_ERROR_OCCURRED(&%(exception_type)s, &%(exception_value)s, &%(exception_tb)s);
 %(release_temps_1)s
 %(var_description_code_1)s
 %(line_number_code_1)s
             goto %(exception_exit)s;
         }
     }
-}
-else
-{
-    Py_DECREF( %(attempt_name)s );
+} else {
+    Py_DECREF(%(attempt_name)s);
 
-    // TODO: Could avoid PyErr_Format.
-#if PYTHON_VERSION < 300
-    PyErr_Format( PyExc_ValueError, "too many values to unpack" );
-#else
-    PyErr_Format( PyExc_ValueError, "too many values to unpack (expected %(count)d)" );
-#endif
-    FETCH_ERROR_OCCURRED( &%(exception_type)s, &%(exception_value)s, &%(exception_tb)s );
+    %(exception_type)s = PyExc_ValueError;
+    Py_INCREF(PyExc_ValueError);
+    %(exception_value)s = %(too_many_values_error)s;
+    Py_INCREF(%(exception_value)s);
+    %(exception_tb)s = NULL;
 %(release_temps_2)s
 %(var_description_code_2)s
 %(line_number_code_2)s
@@ -64,17 +55,13 @@ else
 }"""
 
 template_loop_break_next = """\
-if ( %(to_name)s == NULL )
-{
-    if ( CHECK_AND_CLEAR_STOP_ITERATION_OCCURRED() )
-    {
+if (%(to_name)s == NULL) {
+    if (CHECK_AND_CLEAR_STOP_ITERATION_OCCURRED()) {
 %(break_indicator_code)s
         goto %(break_target)s;
-    }
-    else
-    {
+    } else {
 %(release_temps)s
-        FETCH_ERROR_OCCURRED( &%(exception_type)s, &%(exception_value)s, &%(exception_tb)s );
+        FETCH_ERROR_OCCURRED(&%(exception_type)s, &%(exception_value)s, &%(exception_tb)s);
 %(var_description_code)s
 %(line_number_code)s
         goto %(exception_target)s;
@@ -82,5 +69,6 @@ if ( %(to_name)s == NULL )
 }
 """
 
-from . import TemplateDebugWrapper # isort:skip
+from . import TemplateDebugWrapper  # isort:skip
+
 TemplateDebugWrapper.checkDebug(globals())
