@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -31,18 +31,16 @@ are in another context.
 
 """
 
-from logging import warning
 
-from nuitka import Options, Tracing
+from nuitka import Tracing
 from nuitka.__past__ import unicode  # pylint: disable=I0021,redefined-builtin
-from nuitka.importing.Importing import isWhiteListedImport
-from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
 
 from .FinalizeBase import FinalizationVisitorBase
 
 imported_names = set()
 
+
 def getImportedNames():
     return imported_names
 
@@ -53,31 +51,17 @@ class FinalizeMarkups(FinalizationVisitorBase):
             self._onEnterNode(node)
         except Exception:
             Tracing.printError(
-                "Problem with %r at %s" % (
-                    node,
-                    node.getSourceReference().getAsString()
-                )
+                "Problem with %r at %s"
+                % (node, node.getSourceReference().getAsString())
             )
             raise
 
     def _onEnterNode(self, node):
         # This has many different things it deals with, so there need to be a
-        # lot of branches and statements, pylint: disable=too-many-branches,too-many-statements
+        # lot of branches and statements, pylint: disable=too-many-branches
 
         # Also all self specific things have been done on the outside,
         # pylint: disable=no-self-use
-        if node.isExpressionFunctionBody():
-            if node.isUnoptimized():
-                node.markAsLocalsDict()
-
-        if node.needsLocalsDict():
-            provider = node.getParentVariableProvider()
-
-            provider.markAsLocalsDict()
-
-            if node.isStatementSetLocals():
-                provider.markAsForeignLocalsDict()
-
         if node.isStatementReturn() or node.isStatementGeneratorReturn():
             search = node
 
@@ -87,30 +71,18 @@ class FinalizeMarkups(FinalizationVisitorBase):
             # containing the "return" statement.
             search = search.getParentReturnConsumer()
 
-            if search.isExpressionGeneratorObjectBody() or \
-               search.isExpressionCoroutineObjectBody() or \
-               search.isExpressionAsyncgenObjectBody():
+            if (
+                search.isExpressionGeneratorObjectBody()
+                or search.isExpressionCoroutineObjectBody()
+                or search.isExpressionAsyncgenObjectBody()
+            ):
                 if in_tried_block:
                     search.markAsNeedsGeneratorReturnHandling(2)
                 else:
                     search.markAsNeedsGeneratorReturnHandling(1)
 
-        if node.isExpressionBuiltinImport() and \
-           not Options.getShallFollowExtra() and \
-           not Options.getShallFollowExtraFilePatterns() and \
-           not Options.shallFollowNoImports() and \
-           not isWhiteListedImport(node) and \
-           not node.recurse_attempted and \
-           not Plugins.suppressBuiltinImportWarning(node.getParentModule(), node.getSourceReference()):
-            warning("""Unresolved '__import__' call at '%s' may require use \
-of '--recurse-directory'.""" % (
-                    node.getSourceReference().getAsString()
-                )
-            )
-
-        if node.isExpressionBuiltinImport() and \
-           node.recurse_attempted:
-            module_name = node.getImportName()
+        if node.isExpressionBuiltinImport() and node.recurse_attempted:
+            module_name = node.subnode_name
 
             if module_name.isCompileTimeConstant():
                 imported_module_name = module_name.getCompileTimeConstant()
@@ -120,13 +92,14 @@ of '--recurse-directory'.""" % (
                         imported_names.add(imported_module_name)
 
         if node.isExpressionFunctionCreation():
-            if not node.getParent().isExpressionFunctionCall() or \
-               node.getParent().getFunction() is not node:
-                node.getFunctionRef().getFunctionBody().markAsNeedsCreation()
+            if (
+                not node.getParent().isExpressionFunctionCall()
+                or node.getParent().subnode_function is not node
+            ):
+                node.subnode_function_ref.getFunctionBody().markAsNeedsCreation()
 
         if node.isExpressionFunctionCall():
-            node.getFunction().getFunctionRef().getFunctionBody().\
-              markAsDirectlyCalled()
+            node.subnode_function.subnode_function_ref.getFunctionBody().markAsDirectlyCalled()
 
         if node.isExpressionFunctionRef():
             function_body = node.getFunctionBody()
@@ -140,36 +113,48 @@ of '--recurse-directory'.""" % (
 
         if node.isStatementAssignmentVariable():
             target_var = node.getVariable()
-            assign_source = node.getAssignSource()
+            assign_source = node.subnode_source
 
             if assign_source.isExpressionOperationBinary():
-                left_arg = assign_source.getLeft()
+                left_arg = assign_source.subnode_left
 
-                if left_arg.isExpressionVariableRef():
-                    if assign_source.getLeft().getVariable().isModuleVariable():
-                        assign_source.unmarkAsInplaceSuspect()
-                    elif assign_source.getLeft().getVariable() is target_var:
+                if (
+                    left_arg.isExpressionVariableRef()
+                    or left_arg.isExpressionTempVariableRef()
+                ):
+                    if assign_source.subnode_left.getVariable() is target_var:
                         if assign_source.isInplaceSuspect():
                             node.markAsInplaceSuspect()
+                elif left_arg.isExpressionLocalsVariableRefOrFallback():
+                    # TODO: This might be bad.
+                    assign_source.unmarkAsInplaceSuspect()
 
-        if node.isStatementPublishException():
+        if python_version < 0x300 and node.isStatementPublishException():
             node.getParentStatementsFrame().markAsFrameExceptionPreserving()
 
-        if python_version >= 300:
-            if node.isExpressionYield() or \
-               node.isExpressionYieldFrom() or \
-               node.isExpressionAsyncWait():
+        if python_version >= 0x300:
+            if (
+                node.isExpressionYield()
+                or node.isExpressionYieldFrom()
+                or node.isExpressionYieldFromWaitable()
+            ):
                 search = node.getParent()
 
-                while not search.isExpressionGeneratorObjectBody() and \
-                      not search.isExpressionCoroutineObjectBody() and \
-                      not search.isExpressionAsyncgenObjectBody():
-
+                # TODO: This is best achieved by having different yield nodes
+                # depending on containing function kind to begin with and should
+                # be discovered during the build.
 
+                while (
+                    not search.isExpressionGeneratorObjectBody()
+                    and not search.isExpressionCoroutineObjectBody()
+                    and not search.isExpressionAsyncgenObjectBody()
+                ):
                     last_search = search
                     search = search.getParent()
 
-                    if search.isStatementTry() and \
-                       last_search == search.getBlockExceptHandler():
+                    if (
+                        search.isStatementTry()
+                        and last_search == search.subnode_except_handler
+                    ):
                         node.markAsExceptionPreserving()
                         break
