@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -32,9 +32,8 @@ import pkgutil
 import shutil
 import subprocess
 import sys
-from logging import debug, info, warning
 
-from nuitka import Options, SourceCodeReferences, Tracing
+from nuitka import Options, SourceCodeReferences
 from nuitka.__past__ import iterItems
 from nuitka.containers.odict import OrderedDict
 from nuitka.containers.oset import OrderedSet
@@ -49,6 +48,7 @@ from nuitka.nodes.ModuleNodes import (
 )
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
+from nuitka.Tracing import general, inclusion_logger, printError
 from nuitka.tree.SourceReading import readSourceCodeFromFilename
 from nuitka.utils import Utils
 from nuitka.utils.AppDirs import getCacheDir
@@ -56,15 +56,21 @@ from nuitka.utils.Execution import withEnvironmentPathAdded
 from nuitka.utils.FileOperations import (
     areSamePaths,
     deleteFile,
+    getDirectoryRealPath,
     getExternalUsePath,
     getFileContentByLine,
     getFileContents,
+    getFileList,
     getSubDirectories,
+    haveSameFileContents,
     isPathBelow,
     listDir,
     makePath,
+    putTextFileContents,
+    resolveShellPatternToFilenames,
     withFileLock,
 )
+from nuitka.utils.Importing import getSharedLibrarySuffixes
 from nuitka.utils.ModuleNames import ModuleName
 from nuitka.utils.SharedLibraries import (
     callInstallNameTool,
@@ -78,6 +84,7 @@ from nuitka.utils.Timing import TimerReport
 from nuitka.utils.Utils import getArchitecture
 
 from .DependsExe import getDependsExePath
+from .IncludedDataFiles import IncludedDataFile
 
 
 def loadCodeObjectData(precompiled_filename):
@@ -104,7 +111,10 @@ def _detectedPrecompiledFile(filename, module_name, result, user_provided, techn
     if module_name in module_names:
         return
 
-    debug("Freezing module '%s' (from '%s').", module_name, filename)
+    if Options.isShowInclusion():
+        inclusion_logger.info(
+            "Freezing module '%s' (from '%s')." % (module_name, filename)
+        )
 
     uncompiled_module = makeUncompiledPythonModule(
         module_name=module_name,
@@ -157,14 +167,22 @@ __file__ = (__nuitka_binary_dir + '%s%s') if '__nuitka_binary_dir' in dict(__bui
             "def main():", "def main():return\n\nif 0:\n def _unused():"
         )
 
-    debug("Freezing module '%s' (from '%s').", module_name, filename)
+    if Options.isShowInclusion():
+        inclusion_logger.info(
+            "Freezing module '%s' (from '%s')." % (module_name, filename)
+        )
 
     is_package = os.path.basename(filename) == "__init__.py"
     source_code = Plugins.onFrozenModuleSourceCode(
         module_name=module_name, is_package=is_package, source_code=source_code
     )
 
-    bytecode = compile(source_code, filename, "exec", dont_inherit=True)
+    bytecode = compile(
+        source_code,
+        module_name.replace(".", os.path.sep) + ".py",
+        "exec",
+        dont_inherit=True,
+    )
 
     bytecode = Plugins.onFrozenModuleBytecode(
         module_name=module_name, is_package=is_package, bytecode=bytecode
@@ -211,7 +229,7 @@ def _detectImports(command, user_provided, technical):
     # pylint: disable=too-many-branches,too-many-locals,too-many-statements
 
     # Print statements for stuff to show, the modules loaded.
-    if python_version >= 300:
+    if python_version >= 0x300:
         command += (
             '\nprint("\\n".join(sorted("import " + module.__name__ + " # sourcefile " + '
             'module.__file__ for module in sys.modules.values() if hasattr(module, "__file__") and '
@@ -238,7 +256,7 @@ def _detectImports(command, user_provided, technical):
     tmp_file, tmp_filename = tempfile.mkstemp()
 
     try:
-        if python_version >= 300:
+        if python_version >= 0x300:
             command = command.encode("utf8")
         os.write(tmp_file, command)
         os.close(tmp_file)
@@ -255,15 +273,13 @@ def _detectImports(command, user_provided, technical):
 
     # Don't let errors here go unnoticed.
     if process.returncode != 0:
-        warning("There is a problem with detecting imports, CPython said:")
+        general.warning("There is a problem with detecting imports, CPython said:")
         for line in stderr.split(b"\n"):
-            Tracing.printError(line)
-        sys.exit("Error, please report the issue with above output.")
+            printError(line)
+        general.sysexit("Error, please report the issue with above output.")
 
     result = []
 
-    debug("Detecting imports:")
-
     detections = []
 
     for line in stderr.replace(b"\r", b"").split(b"\n"):
@@ -275,7 +291,7 @@ def _detectImports(command, user_provided, technical):
             module_name = parts[0].split(b" ", 2)[1]
             origin = parts[1].split()[0]
 
-            if python_version >= 300:
+            if python_version >= 0x300:
                 module_name = module_name.decode("utf-8")
 
             module_name = ModuleName(module_name)
@@ -284,7 +300,7 @@ def _detectImports(command, user_provided, technical):
                 # This is a ".pyc" file that was imported, even before we have a
                 # chance to do anything, we need to preserve it.
                 filename = parts[1][len(b"precompiled from ") :]
-                if python_version >= 300:
+                if python_version >= 0x300:
                     filename = filename.decode("utf-8")
 
                 # Do not leave standard library when freezing.
@@ -294,7 +310,7 @@ def _detectImports(command, user_provided, technical):
                 detections.append((module_name, 3, "precompiled", filename))
             elif origin == b"sourcefile":
                 filename = parts[1][len(b"sourcefile ") :]
-                if python_version >= 300:
+                if python_version >= 0x300:
                     filename = filename.decode("utf-8")
 
                 # Do not leave standard library when freezing.
@@ -303,11 +319,13 @@ def _detectImports(command, user_provided, technical):
 
                 if filename.endswith(".py"):
                     detections.append((module_name, 2, "sourcefile", filename))
+                elif filename.endswith(".pyc"):
+                    detections.append((module_name, 3, "precompiled", filename))
                 elif not filename.endswith("<frozen>"):
                     # Python3 started lying in "__name__" for the "_decimal"
                     # calls itself "decimal", which then is wrong and also
                     # clashes with "decimal" proper
-                    if python_version >= 300:
+                    if python_version >= 0x300:
                         if module_name == "decimal":
                             module_name = ModuleName("_decimal")
 
@@ -316,7 +334,7 @@ def _detectImports(command, user_provided, technical):
                 # Shared library in early load, happens on RPM based systems and
                 # or self compiled Python installations.
                 filename = parts[1][len(b"dynamically loaded from ") :]
-                if python_version >= 300:
+                if python_version >= 0x300:
                     filename = filename.decode("utf-8")
 
                 # Do not leave standard library when freezing.
@@ -385,7 +403,7 @@ def scanStandardLibraryPath(stdlib_dir):
             if "ensurepip" in dirs:
                 dirs.remove("ensurepip")
 
-            # Ignore "lib-dynload" and "lib-tk" and alikes.
+            # Ignore "lib-dynload" and "lib-tk" and alike.
             dirs[:] = [
                 dirname
                 for dirname in dirs
@@ -420,13 +438,13 @@ def scanStandardLibraryPath(stdlib_dir):
             if "test_utils.py" in filenames:
                 filenames.remove("test_utils.py")
 
-        if python_version >= 340 and Utils.isWin32Windows():
+        if python_version >= 0x340 and Utils.isWin32Windows():
             if import_path == "multiprocessing":
                 filenames.remove("popen_fork.py")
                 filenames.remove("popen_forkserver.py")
                 filenames.remove("popen_spawn_posix.py")
 
-        if python_version >= 300 and Utils.isPosixWindows():
+        if python_version >= 0x300 and Utils.isPosixWindows():
             if import_path == "curses":
                 filenames.remove("has_key.py")
 
@@ -442,7 +460,7 @@ def scanStandardLibraryPath(stdlib_dir):
                 else:
                     yield import_path + "." + module_name
 
-        if python_version >= 300:
+        if python_version >= 0x300:
             if "__pycache__" in dirs:
                 dirs.remove("__pycache__")
 
@@ -473,7 +491,7 @@ def _detectEarlyImports():
     import_code += ";import locale;"
 
     # For Python3 we patch inspect without knowing if it is used.
-    if python_version >= 300:
+    if python_version >= 0x300:
         import_code += "import inspect;"
 
     result = _detectImports(command=import_code, user_provided=False, technical=True)
@@ -543,8 +561,6 @@ for imp in imports:
             if module.getFullName() not in early_names
         ]
 
-    debug("Finished detecting early imports.")
-
     return result
 
 
@@ -581,7 +597,7 @@ def _detectBinaryPathDLLsPosix(dll_filename):
 
         if _detected_python_rpath:
             _detected_python_rpath = _detected_python_rpath.replace(
-                b"$ORIGIN", os.path.dirname(sys.executable).encode("utf-8")
+                "$ORIGIN", os.path.dirname(sys.executable)
             )
 
     with withEnvironmentPathAdded("LD_LIBRARY_PATH", _detected_python_rpath):
@@ -591,6 +607,10 @@ def _detectBinaryPathDLLsPosix(dll_filename):
 
         stdout, _stderr = process.communicate()
 
+        inclusion_logger.debug(
+            "ldd output for %s is %s and %s" % (dll_filename, stdout, _stderr)
+        )
+
         for line in stdout.split(b"\n"):
             if not line:
                 continue
@@ -608,7 +628,7 @@ def _detectBinaryPathDLLsPosix(dll_filename):
             if not filename:
                 continue
 
-            if python_version >= 300:
+            if python_version >= 0x300:
                 filename = filename.decode("utf-8")
 
             # Sometimes might use stuff not found or supplied by ldd itself.
@@ -704,7 +724,7 @@ def _detectBinaryPathDLLsMacOS(original_dir, binary_filename, keep_unresolved):
                 stop = True
                 break
         if not stop:
-            if python_version >= 300:
+            if python_version >= 0x300:
                 filename = filename.decode("utf-8")
 
             # print("adding", filename)
@@ -760,7 +780,7 @@ def _detectBinaryRPathsMacOS(original_dir, binary_filename):
     for i, o in enumerate(lines):
         if o.endswith(b"cmd LC_RPATH"):
             line = lines[i + 2]
-            if python_version >= 300:
+            if python_version >= 0x300:
                 line = line.decode("utf-8")
 
             line = line.split("path ")[1]
@@ -934,6 +954,8 @@ def getScanDirectories(package_name, original_dir):
         if os.path.basename(scan_dir) == "__pycache__":
             continue
 
+        scan_dir = getDirectoryRealPath(scan_dir)
+
         # No DLLs, no use.
         if not any(entry[1].lower().endswith(".dll") for entry in listDir(scan_dir)):
             continue
@@ -980,7 +1002,7 @@ def detectBinaryPathDLLsWindowsDependencyWalker(
             return result
 
     if Options.isShowProgress():
-        info("Analysing dependencies of '%s'." % binary_filename)
+        general.info("Analysing dependencies of '%s'." % binary_filename)
 
     scan_dirs = getScanDirectories(package_name, original_dir)
 
@@ -1028,7 +1050,7 @@ SxS
     depends_exe_process.wait()
 
     if not os.path.exists(output_filename):
-        sys.exit("Error, depends.exe failed to product output.")
+        inclusion_logger.sysexit("Error, depends.exe failed to product output.")
 
     # Opening the result under lock, so it is not getting locked by new processes.
 
@@ -1040,9 +1062,7 @@ SxS
     deleteFile(dwp_filename, must_exist=True)
 
     if update_cache:
-        with open(cache_filename, "w") as cache_file:
-            for dll_filename in result:
-                print(dll_filename, file=cache_file)
+        putTextFileContents(filename=cache_filename, contents=result)
 
     return result
 
@@ -1081,7 +1101,7 @@ def _parsePEFileOutput(
         extracted = eval(getFileContents(cache_filename))
     else:
         if Options.isShowProgress():
-            info("Analysing dependencies of '%s'." % binary_filename)
+            general.info("Analysing dependencies of '%s'." % binary_filename)
 
         extracted = getPEFileInformation(binary_filename)
 
@@ -1190,10 +1210,10 @@ def detectBinaryDLLs(
     use_cache,
     update_cache,
 ):
-    """ Detect the DLLs used by a binary.
+    """Detect the DLLs used by a binary.
 
-        Using "ldd" (Linux), "pefile" or "depends.exe" (Windows), or
-        "otool" (macOS) the list of used DLLs is retrieved.
+    Using "ldd" (Linux), "pefile" or "depends.exe" (Windows), or
+    "otool" (macOS) the list of used DLLs is retrieved.
     """
 
     if (
@@ -1203,7 +1223,8 @@ def detectBinaryDLLs(
         return _detectBinaryPathDLLsPosix(dll_filename=original_filename)
     elif Utils.isWin32Windows() and Options.getWindowsDependencyTool() == "pefile":
         with TimerReport(
-            "Finding dependencies for %s took %%.2f seconds" % binary_filename
+            message="Finding dependencies for %s took %%.2f seconds" % binary_filename,
+            decider=Options.isShowProgress,
         ):
             return detectBinaryPathDLLsWindowsPE(
                 is_main_executable=is_main_executable,
@@ -1216,7 +1237,8 @@ def detectBinaryDLLs(
             )
     elif Utils.isWin32Windows():
         with TimerReport(
-            "Running depends.exe for %s took %%.2f seconds" % binary_filename
+            message="Running depends.exe for %s took %%.2f seconds" % binary_filename,
+            decider=Options.isShowProgress,
         ):
             return detectBinaryPathDLLsWindowsDependencyWalker(
                 is_main_executable=is_main_executable,
@@ -1256,10 +1278,8 @@ def detectUsedDLLs(source_dir, standalone_entry_points, use_cache, update_cache)
         for dll_filename in sorted(tuple(used_dlls)):
             if not os.path.isfile(dll_filename):
                 if _unfound_dlls:
-                    warning(
-                        """\
-Dependency '%s' could not be found, you might need to copy it
-manually."""
+                    general.warning(
+                        "Dependency '%s' could not be found, you might need to copy it manually."
                         % dll_filename
                     )
 
@@ -1274,17 +1294,15 @@ manually."""
     with ThreadPoolExecutor(max_workers=Utils.getCoreCount() * 3) as worker_pool:
         workers = []
 
-        for count, (original_filename, binary_filename, package_name) in enumerate(
-            standalone_entry_points
-        ):
+        for count, standalone_entry_point in enumerate(standalone_entry_points):
             workers.append(
                 worker_pool.submit(
                     addDLLInfo,
                     count,
                     source_dir,
-                    original_filename,
-                    binary_filename,
-                    package_name,
+                    standalone_entry_point.source_path,
+                    standalone_entry_point.dest_path,
+                    standalone_entry_point.package_name,
                 )
             )
 
@@ -1337,14 +1355,19 @@ def getSharedLibraryRPATH(filename):
     retcode = process.poll()
 
     if retcode != 0:
-        sys.exit(
+        inclusion_logger.sysexit(
             "Error reading shared library path for %s, tool said %r"
             % (filename, stderr)
         )
 
     for line in stdout.split(b"\n"):
         if b"RPATH" in line or b"RUNPATH" in line:
-            return line[line.find(b"[") + 1 : line.rfind(b"]")]
+            result = line[line.find(b"[") + 1 : line.rfind(b"]")]
+
+            if str is not bytes:
+                result = result.decode("utf-8")
+
+            return result
 
     return None
 
@@ -1354,10 +1377,12 @@ def removeSharedLibraryRPATH(filename):
 
     if rpath is not None:
         if Options.isShowInclusion():
-            info("Removing 'RPATH' setting from '%s'.", filename)
+            inclusion_logger.info(
+                "Removing 'RPATH' setting '%s' from '%s'." % (rpath, filename)
+            )
 
         if not Utils.isExecutableCommand("chrpath"):
-            sys.exit(
+            inclusion_logger.sysexit(
                 """\
 Error, needs 'chrpath' on your system, due to 'RPATH' settings in used shared
 libraries that need to be removed."""
@@ -1377,6 +1402,16 @@ libraries that need to be removed."""
         assert retcode == 0, filename
 
 
+# These DLLs are run time DLLs from Microsoft, and packages will depend on different
+# ones, but it will be OK to use the latest one.
+ms_runtime_dlls = (
+    "msvcp140_1.dll",
+    "msvcp140.dll",
+    "vcruntime140_1.dll",
+    "concrt140.dll",
+)
+
+
 def copyUsedDLLs(source_dir, dist_dir, standalone_entry_points):
     # This is terribly complex, because we check the list of used DLLs
     # trying to avoid duplicates, and detecting errors with them not
@@ -1394,6 +1429,7 @@ def copyUsedDLLs(source_dir, dist_dir, standalone_entry_points):
     )
 
     removed_dlls = set()
+    warned_about = set()
 
     # Fist make checks and remove some.
     for dll_filename1, sources1 in tuple(iterItems(used_dlls)):
@@ -1421,17 +1457,14 @@ def copyUsedDLLs(source_dir, dist_dir, standalone_entry_points):
             dll_name = os.path.basename(dll_filename1)
 
             if Options.isShowInclusion():
-                info(
+                inclusion_logger.info(
                     """Colliding DLL names for %s, checking identity of \
 '%s' <-> '%s'."""
                     % (dll_name, dll_filename1, dll_filename2)
                 )
 
-            # Check that if a DLL has the same name, if it's identical,
-            # happens at least for OSC and Fedora 20.
-            import filecmp
-
-            if filecmp.cmp(dll_filename1, dll_filename2):
+            # Check that if a DLL has the same name, if it's identical, then it's easy.
+            if haveSameFileContents(dll_filename1, dll_filename2):
                 del used_dlls[dll_filename2]
                 removed_dlls.add(dll_filename2)
 
@@ -1456,15 +1489,20 @@ def copyUsedDLLs(source_dir, dist_dir, standalone_entry_points):
                     solved = False
 
                 if solved:
-                    warning(
-                        "Ignoring conflicting DLLs for '%s' and using newest file version."
-                        % dll_name
-                    )
+                    if dll_name not in warned_about and dll_name not in ms_runtime_dlls:
+                        warned_about.add(dll_name)
+
+                        inclusion_logger.warning(
+                            "Conflicting DLLs for '%s' in your installation, newest file version used, hoping for the best."
+                            % dll_name
+                        )
+
                     continue
 
-            # So we have conflicting DLLs, in which case we do not proceed.
-            warning(
-                """Ignoring non-identical DLLs for '%s'.
+            # So we have conflicting DLLs, in which case we do report the fact.
+            inclusion_logger.warning(
+                """\
+Ignoring non-identical DLLs for '%s'.
 %s used by:
    %s
 different from
@@ -1494,7 +1532,7 @@ different from
         dll_map.append((dll_filename, dll_name))
 
         if Options.isShowInclusion():
-            Tracing.general.info(
+            inclusion_logger.info(
                 "Included used shared library '%s' (used by %s)."
                 % (dll_filename, ", ".join(sources))
             )
@@ -1504,9 +1542,9 @@ different from
         # the relative DLL location in the ".dist" folder.
         for standalone_entry_point in standalone_entry_points:
             fixupBinaryDLLPathsMacOS(
-                binary_filename=standalone_entry_point[1],
+                binary_filename=standalone_entry_point.dest_path,
                 dll_map=dll_map,
-                original_location=standalone_entry_point[0],
+                original_location=standalone_entry_point.source_path,
             )
 
         for original_path, dll_filename in dll_map:
@@ -1520,22 +1558,72 @@ different from
         # For Linux, the "rpath" of libraries may be an issue and must be
         # removed.
         for standalone_entry_point in standalone_entry_points[1:]:
-            removeSharedLibraryRPATH(standalone_entry_point[1])
+            removeSharedLibraryRPATH(standalone_entry_point.dest_path)
 
         for _original_path, dll_filename in dll_map:
             removeSharedLibraryRPATH(os.path.join(dist_dir, dll_filename))
     elif Utils.isWin32Windows():
-        if python_version < 300:
+        if python_version < 0x300:
             # For Win32, we might have to remove SXS paths
             for standalone_entry_point in standalone_entry_points[1:]:
-                removeSxsFromDLL(standalone_entry_point[1])
+                removeSxsFromDLL(standalone_entry_point.dest_path)
 
             for _original_path, dll_filename in dll_map:
                 removeSxsFromDLL(os.path.join(dist_dir, dll_filename))
 
 
+def _handleDataFile(dist_dir, tracer, included_datafile):
+    """Handle a data file."""
+    if isinstance(included_datafile, IncludedDataFile):
+        if included_datafile.kind == "empty_dirs":
+            tracer.info(
+                "Included empty directories %s due to %s."
+                % (
+                    ",".join(included_datafile.dest_path),
+                    included_datafile.reason,
+                )
+            )
+
+            for sub_dir in included_datafile.dest_path:
+                makePath(os.path.join(dist_dir, sub_dir))
+        elif included_datafile.kind == "datafile":
+            dest_path = os.path.join(dist_dir, included_datafile.dest_path)
+
+            tracer.info(
+                "Included data file %r due to %s."
+                % (
+                    included_datafile.dest_path,
+                    included_datafile.reason,
+                )
+            )
+
+            makePath(os.path.dirname(dest_path))
+            shutil.copyfile(included_datafile.source_path, dest_path)
+        else:
+            assert False, included_datafile
+    else:
+        # TODO: Goal is have this unused.
+        source_desc, target_filename = included_datafile
+
+        if not isPathBelow(dist_dir, target_filename):
+            target_filename = os.path.join(dist_dir, target_filename)
+
+        makePath(os.path.dirname(target_filename))
+
+        if inspect.isfunction(source_desc):
+            content = source_desc(target_filename)
+
+            if content is not None:  # support creation of empty directories
+                with open(
+                    target_filename, "wb" if type(content) is bytes else "w"
+                ) as output:
+                    output.write(content)
+        else:
+            shutil.copy2(source_desc, target_filename)
+
+
 def copyDataFiles(dist_dir):
-    """ Copy the data files needed for standalone distribution.
+    """Copy the data files needed for standalone distribution.
 
     Args:
         dist_dir: The distribution folder under creation
@@ -1545,25 +1633,71 @@ def copyDataFiles(dist_dir):
         necessary handling if provided like this.
     """
 
+    # Many details to deal with, pylint: disable=too-many-locals
+
+    for pattern, dest, arg in Options.getShallIncludeDataFiles():
+        filenames = resolveShellPatternToFilenames(pattern)
+
+        if not filenames:
+            inclusion_logger.warning("No match data file to be included: %r" % pattern)
+
+        for filename in filenames:
+            file_reason = "specified data file %r on command line" % arg
+
+            rel_path = dest
+
+            if rel_path.endswith(("/", os.path.sep)):
+                rel_path = os.path.join(rel_path, os.path.basename(filename))
+
+            _handleDataFile(
+                dist_dir,
+                inclusion_logger,
+                makeIncludedDataFile(filename, rel_path, file_reason),
+            )
+
     # Cyclic dependency
     from nuitka import ModuleRegistry
 
     for module in ModuleRegistry.getDoneModules():
-        for _plugin_name, (source_desc, target_filename) in Plugins.considerDataFiles(
-            module
-        ):
-            target_filename = os.path.join(dist_dir, target_filename)
-            assert isPathBelow(dist_dir, target_filename)
+        for plugin, included_datafile in Plugins.considerDataFiles(module):
+            _handleDataFile(
+                dist_dir=dist_dir, tracer=plugin, included_datafile=included_datafile
+            )
 
-            makePath(os.path.dirname(target_filename))
+    for module in ModuleRegistry.getDoneModules():
+        if module.isCompiledPythonPackage() or module.isUncompiledPythonPackage():
+            package_name = module.getFullName()
 
-            if inspect.isfunction(source_desc):
-                content = source_desc(target_filename)
+            match, reason = package_name.matchesToShellPatterns(
+                patterns=Options.getShallIncludePackageData()
+            )
 
-                if content is not None:  # support creation of empty directories
-                    with open(
-                        target_filename, "wb" if type(content) is bytes else "w"
-                    ) as output:
-                        output.write(content)
-            else:
-                shutil.copy2(source_desc, target_filename)
+            if match:
+                package_directory = module.getCompileTimeDirectory()
+
+                pkg_filenames = getFileList(
+                    package_directory,
+                    ignore_dirs=("__pycache__",),
+                    ignore_suffixes=(".py", ".pyw", ".pyc", ".pyo", ".dll")
+                    + getSharedLibrarySuffixes(),
+                )
+
+                if pkg_filenames:
+                    file_reason = "package '%s' %s" % (package_name, reason)
+
+                    for pkg_filename in pkg_filenames:
+                        rel_path = os.path.join(
+                            package_name.asPath(),
+                            os.path.relpath(pkg_filename, package_directory),
+                        )
+
+                        _handleDataFile(
+                            dist_dir,
+                            inclusion_logger,
+                            makeIncludedDataFile(pkg_filename, rel_path, file_reason),
+                        )
+
+                # assert False, (module.getCompileTimeDirectory(), pkg_files)
+
+
+from .IncludedDataFiles import makeIncludedDataFile
