@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -17,10 +17,10 @@
 #
 """ Locating modules and package source on disk.
 
-The actual import of a module would already execute code that changes
-things. Imagine a module that does ``os.system()``, it would be done during
-compilation. People often connect to databases, and these kind of things,
-at import time.
+The actual import of a module would already execute code that changes things.
+Imagine a module that does ``os.system()``, it would be done during
+compilation. People often connect to databases, and these kind of things, at
+import time.
 
 Therefore CPython exhibits the interfaces in an ``imp`` module in standard
 library, which one can use those to know ahead of time, what file import would
@@ -30,27 +30,30 @@ exactly like CPython does, so we implement here a multi step search process
 that is compatible.
 
 This approach is much safer of course and there is no loss. To determine if
-it's from the standard library, one can abuse the attribute ``__file__`` of
-the ``os`` module like it's done in ``isStandardLibraryPath`` of this module.
+it's from the standard library, one can abuse the attribute ``__file__`` of the
+``os`` module like it's done in ``isStandardLibraryPath`` of this module.
 
 """
 
 from __future__ import print_function
 
+import collections
 import hashlib
 import imp
 import os
 import sys
 import zipfile
-from logging import warning
 
 from nuitka import Options
 from nuitka.containers.oset import OrderedSet
 from nuitka.importing import StandardLibrary
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
+from nuitka.Tracing import recursion_logger
 from nuitka.utils.AppDirs import getCacheDir
 from nuitka.utils.FileOperations import listDir
+from nuitka.utils.Importing import getSharedLibrarySuffixes
+from nuitka.utils.ModuleNames import ModuleName
 
 from .PreloadedPackages import getPreloadedPackagePath, isPreloadedPackagePath
 from .Whitelisting import isWhiteListedNotExistingModule
@@ -62,10 +65,11 @@ warned_about = set()
 # Directory where the main script lives. Should attempt to import from there.
 main_path = None
 
-def setMainScriptDirectory(main_dir):
-    """ Initialize the main script directory.
 
-        We use this as part of the search path for modules.
+def setMainScriptDirectory(main_dir):
+    """Initialize the main script directory.
+
+    We use this as part of the search path for modules.
     """
     # We need to set this from the outside, pylint: disable=global-statement
 
@@ -74,56 +78,49 @@ def setMainScriptDirectory(main_dir):
 
 
 def isPackageDir(dirname):
-    """ Decide if a directory is a package.
+    """Decide if a directory is a package.
 
-        Before Python3.3 it's required to have a "__init__.py" file, but then
-        it became impossible to decide, and for extra fun, there is also the
-        extra packages provided via "*.pth" file tricks by "site.py" loading.
+    Before Python3.3 it's required to have a "__init__.py" file, but then
+    it became impossible to decide, and for extra fun, there is also the
+    extra packages provided via "*.pth" file tricks by "site.py" loading.
     """
 
-    return '.' not in os.path.basename(dirname) and \
-           os.path.isdir(dirname) and \
-           (
-               python_version >= 300 or
-               os.path.isfile(os.path.join(dirname, "__init__.py")) or
-               isPreloadedPackagePath(dirname)
-           )
-
-
-def getPackageNameFromFullName(full_name):
-    if '.' in full_name:
-        return full_name[:full_name.rfind('.')]
-    else:
-        return None
-
-
-def getModuleFullNameFromPackageAndName(package, name):
-    if not package:
-        return name
-    else:
-        return package + '.' + name
-
-
-def getExtensionModuleSuffixes():
-    for suffix, _mode, kind in imp.get_suffixes():
-        if kind == imp.C_EXTENSION:
-            yield suffix
+    return (
+        "." not in os.path.basename(dirname)
+        and os.path.isdir(dirname)
+        and (
+            python_version >= 0x300
+            or os.path.isfile(os.path.join(dirname, "__init__.py"))
+            or isPreloadedPackagePath(dirname)
+        )
+    )
 
 
 def getModuleNameAndKindFromFilename(module_filename):
+    """Given a filename, decide the module name and kind.
+
+    Args:
+        module_name - file path of the module
+    Returns:
+        Tuple with the name of the module basename, and the kind of the
+        module derived from the file suffix. Can be None, None if is is not a
+        known file suffix.
+    Notes:
+        This doesn't concern itself with packages, that needs to be tracked
+        by the using code. It cannot be decided from the filename at all.
+    """
+
+    # TODO: This does not handle ".pyw" files it seems.
     if os.path.isdir(module_filename):
-        module_name = os.path.basename(module_filename)
+        module_name = ModuleName(os.path.basename(module_filename))
         module_kind = "py"
     elif module_filename.endswith(".py"):
         module_name = os.path.basename(module_filename)[:-3]
         module_kind = "py"
     else:
-        for suffix, _mode, kind in imp.get_suffixes():
-            if kind != imp.C_EXTENSION:
-                continue
-
+        for suffix in getSharedLibrarySuffixes():
             if module_filename.endswith(suffix):
-                module_name = os.path.basename(module_filename)[:-len(suffix)]
+                module_name = os.path.basename(module_filename)[: -len(suffix)]
                 module_kind = "shlib"
 
                 break
@@ -145,8 +142,9 @@ def warnAbout(importing, module_name, parent_package, level, tried_names):
     if module_name == "":
         return
 
-    if not isWhiteListedImport(importing) and \
-       not isWhiteListedNotExistingModule(module_name):
+    if not isWhiteListedImport(importing) and not isWhiteListedNotExistingModule(
+        module_name
+    ):
         key = module_name, parent_package, level
 
         if key not in warned_about:
@@ -171,20 +169,24 @@ def warnAbout(importing, module_name, parent_package, level, tried_names):
 
             if _debug_module_finding:
                 if parent_package is not None:
-                    warning(
-                        "%s: Cannot find '%s' in package '%s' %s (tried %s).",
-                        importing.getSourceReference().getAsString(),
-                        module_name,
-                        parent_package,
-                        level_desc,
-                        ','.join(tried_names)
+                    recursion_logger.warning(
+                        "%s: Cannot find '%s' in package '%s' %s (tried %s)."
+                        % (
+                            importing.getSourceReference().getAsString(),
+                            module_name,
+                            parent_package,
+                            level_desc,
+                            ",".join(tried_names),
+                        )
                     )
                 else:
-                    warning(
-                        "%s: Cannot find '%s' %s.",
-                        importing.getSourceReference().getAsString(),
-                        module_name,
-                        level_desc
+                    recursion_logger.warning(
+                        "%s: Cannot find '%s' %s."
+                        % (
+                            importing.getSourceReference().getAsString(),
+                            module_name,
+                            level_desc,
+                        )
                     )
 
 
@@ -193,36 +195,36 @@ def normalizePackageName(module_name):
     # platform, we either cannot look into it, or we require that we resolve it
     # here correctly.
     if module_name == "os.path":
-        module_name = os.path.basename(os.path.__name__)
+        module_name = ModuleName(os.path.basename(os.path.__name__))
 
     return module_name
 
 
 def findModule(importing, module_name, parent_package, level, warn):
-    """ Find a module with given package name as parent.
+    """Find a module with given package name as parent.
 
-        The package name can be None of course. Level is the same
-        as with "__import__" built-in. Warnings are optional.
+    The package name can be None of course. Level is the same
+    as with "__import__" built-in. Warnings are optional.
 
+    Returns:
         Returns a triple of package name the module is in, filename of
         it, which can be a directory for packages, and the location
-        method.
+        method used.
     """
-
     # We have many branches here, because there are a lot of cases to try.
     # pylint: disable=too-many-branches
+
+    assert type(module_name) is ModuleName, module_name
+
     if _debug_module_finding:
         print(
-            "findModule: Enter to search %r in package %r level %s." % (
-                module_name,
-                parent_package,
-                level
-            )
+            "findModule: Enter to search %r in package %r level %s."
+            % (module_name, parent_package, level)
         )
 
     # Do not allow star imports to get here. We just won't find modules with
     # that name, but it would be wasteful.
-    assert module_name != '*'
+    assert module_name != "*"
 
     tried_names = []
 
@@ -230,7 +232,10 @@ def findModule(importing, module_name, parent_package, level, warn):
         # TODO: Should give a warning and return not found if the levels
         # exceed the package name.
         if parent_package is not None:
-            parent_package = '.'.join(parent_package.split('.')[:-level+1])
+            # TODO: This should be done with the API instead.
+            parent_package = ModuleName(
+                ".".join(parent_package.asString().split(".")[: -level + 1])
+            )
 
             if parent_package == "":
                 parent_package = None
@@ -239,73 +244,64 @@ def findModule(importing, module_name, parent_package, level, warn):
 
     # Try relative imports first if we have a parent package.
     if level != 0 and parent_package is not None:
-        full_name = normalizePackageName(parent_package + '.' + module_name)
-
-        if full_name.endswith('.'):
-            full_name = full_name[:-1]
+        if module_name:
+            full_name = ModuleName(parent_package + "." + module_name)
+        else:
+            full_name = ModuleName(parent_package)
 
+        full_name = normalizePackageName(full_name)
         tried_names.append(full_name)
 
         try:
-            module_filename = _findModule(
-                module_name = full_name,
-            )
+            module_filename = _findModule(module_name=full_name)
         except ImportError:
             # For relative import, that is OK, we will still try absolute.
             pass
         else:
             if _debug_module_finding:
                 print(
-                    "findModule: Relative imported module '%s' as '%s' in filename '%s':" % (
-                        module_name,
-                        full_name,
-                        module_filename
-                    )
+                    "findModule: Relative imported module '%s' as '%s' in filename '%s':"
+                    % (module_name, full_name, module_filename)
                 )
 
-            return getPackageNameFromFullName(full_name), module_filename, "relative"
+            return full_name.getPackageName(), module_filename, "relative"
 
     if level <= 1 and module_name != "":
         module_name = normalizePackageName(module_name)
         tried_names.append(module_name)
 
-        package_name = getPackageNameFromFullName(module_name)
+        package_name = module_name.getPackageName()
 
         # Built-in module names must not be searched any further.
         if module_name in sys.builtin_module_names:
             if _debug_module_finding:
                 print(
-                    "findModule: Absolute imported module '%s' in as built-in':" % (
-                        module_name,
-                    )
+                    "findModule: Absolute imported module '%s' in as built-in':"
+                    % (module_name,)
                 )
             return package_name, None, "built-in"
 
         try:
-            module_filename = _findModule(
-                module_name = module_name,
-            )
+            module_filename = _findModule(module_name=module_name)
         except ImportError:
             # For relative import, that is OK, we will still try absolute.
             pass
         else:
             if _debug_module_finding:
                 print(
-                    "findModule: Found absolute imported module '%s' in filename '%s':" % (
-                        module_name,
-                        module_filename
-                    )
+                    "findModule: Found absolute imported module '%s' in filename '%s':"
+                    % (module_name, module_filename)
                 )
 
             return package_name, module_filename, "absolute"
 
     if warn:
         warnAbout(
-            importing      = importing,
-            module_name    = module_name,
-            parent_package = parent_package,
-            tried_names    = tried_names,
-            level          = level
+            importing=importing,
+            module_name=module_name,
+            parent_package=parent_package,
+            tried_names=tried_names,
+            level=level,
         )
 
     return None, None, "not-found"
@@ -314,12 +310,36 @@ def findModule(importing, module_name, parent_package, level, warn):
 # Some platforms are case insensitive.
 case_sensitive = not sys.platform.startswith(("win", "cygwin", "darwin"))
 
-def _findModuleInPath2(module_name, search_path):
-    """ This is out own module finding low level implementation.
+ImportScanFinding = collections.namedtuple(
+    "ImportScanFinding", ("found_in", "priority", "full_path", "search_order")
+)
 
-        Just the full module name and search path are given. This is then
-        tasked to raise "ImportError" or return a path if it finds it, or
-        None, if it is a built-in.
+
+def _reportCandidates(module_name, candidate, candidates):
+    if (
+        candidate.priority == 1
+        and Options.shallPreferSourcecodeOverExtensionModules() is None
+    ):
+        for c in candidates:
+            # Don't compare to itself and don't consider unused bytecode a problem.
+            if c is candidate or c.priority == 3:
+                continue
+
+            if c.search_order == candidate.search_order:
+                recursion_logger.info(
+                    """\
+Should decide '--prefer-source-code' vs. '--no-prefer-source-code', using \
+existing '%s' extension module by default. Candidates were: %s <-> %s."""
+                    % (module_name, candidate, c)
+                )
+
+
+def _findModuleInPath2(module_name, search_path):
+    """This is out own module finding low level implementation.
+
+    Just the full module name and search path are given. This is then
+    tasked to raise "ImportError" or return a path if it finds it, or
+    None, if it is a built-in.
     """
     # We have many branches here, because there are a lot of cases to try.
     # pylint: disable=too-many-branches,too-many-locals
@@ -330,18 +350,27 @@ def _findModuleInPath2(module_name, search_path):
 
     considered = set()
 
-    for entry in search_path:
+    # Higher values are lower priority.
+    priority_map = {
+        imp.PY_COMPILED: 3,
+        imp.PY_SOURCE: 0 if Options.shallPreferSourcecodeOverExtensionModules() else 2,
+        imp.C_EXTENSION: 1,
+    }
+
+    for count, entry in enumerate(search_path):
         # Don't try again, just with an entry of different casing or complete
         # duplicate.
         if os.path.normcase(entry) in considered:
             continue
         considered.add(os.path.normcase(entry))
 
-        package_directory = os.path.join(entry, module_name)
+        package_directory = os.path.join(entry, module_name.asPath())
 
         # First, check for a package with an init file, that would be the
         # first choice.
         if os.path.isdir(package_directory):
+            found = False
+
             for suffix, _mode, mtype in imp.get_suffixes():
                 if mtype == imp.C_EXTENSION:
                     continue
@@ -352,46 +381,62 @@ def _findModuleInPath2(module_name, search_path):
 
                 if os.path.isfile(file_path):
                     candidates.add(
-                        (entry, 1, package_directory)
-                    )
-                    break
-            else:
-                if python_version >= 300:
-                    candidates.add(
-                        (entry, 2, package_directory)
+                        ImportScanFinding(
+                            found_in=entry,
+                            priority=priority_map[mtype],
+                            full_path=package_directory,
+                            search_order=count,
+                        )
                     )
+                    found = True
 
-        # Then, check out suffixes of all kinds.
-        for suffix, _mode, _type in imp.get_suffixes():
-            file_path = os.path.join(entry, module_name + suffix)
-            if os.path.isfile(file_path):
+            if not found and python_version >= 0x300:
                 candidates.add(
-                    (entry, 1, file_path)
+                    ImportScanFinding(
+                        found_in=entry,
+                        priority=10,
+                        full_path=package_directory,
+                        search_order=count + len(search_path),
+                    )
                 )
-                break
+
+        # Then, check out suffixes of all kinds, but only for one directory.
+        last_mtype = 0
+        for suffix, _mode, mtype in imp.get_suffixes():
+            # Use first match per kind only.
+            if mtype == last_mtype:
+                continue
+
+            full_path = os.path.join(entry, module_name + suffix)
+
+            if os.path.isfile(full_path):
+                candidates.add(
+                    ImportScanFinding(
+                        found_in=entry,
+                        priority=priority_map[mtype],
+                        full_path=full_path,
+                        search_order=count,
+                    )
+                )
+                last_mtype = mtype
 
     if _debug_module_finding:
-        print("Candidates", candidates)
+        print("Candidates:", candidates)
 
     if candidates:
-        # Ignore lower priority matches from package directories without
-        # "__init__.py" file.
-        min_prio = min(candidate[1] for candidate in candidates)
-        candidates = [
-            candidate
-            for candidate in
-            candidates
-            if candidate[1] == min_prio
-        ]
+        # Sort by priority, with entries from same path element coming first, then desired type.
+        candidates = sorted(candidates, key=lambda c: (c.search_order, c.priority))
 
         # On case sensitive systems, no resolution needed.
         if case_sensitive:
-            return candidates[0][2]
+            _reportCandidates(module_name, candidates[0], candidates)
+            return candidates[0].full_path
         else:
             for candidate in candidates:
                 for fullname, _filename in listDir(candidate[0]):
-                    if fullname == candidate[2]:
-                        return candidate[2]
+                    if fullname == candidate.full_path:
+                        _reportCandidates(module_name, candidate, candidates)
+                        return candidate.full_path
 
             # Only exact case matches matter, all candidates were ignored,
             # lets just fall through to raising the import error.
@@ -399,23 +444,22 @@ def _findModuleInPath2(module_name, search_path):
     # Nothing found.
     raise ImportError
 
+
 _egg_files = {}
 
+
 def _unpackPathElement(path_entry):
     if not path_entry:
-        return '.' # empty means current directory
+        return "."  # empty means current directory
 
     if os.path.isfile(path_entry) and path_entry.lower().endswith(".egg"):
         if path_entry not in _egg_files:
-            checksum = hashlib.md5(open(path_entry, "rb").read()).hexdigest()
+            with open(path_entry, "rb") as f:
+                checksum = hashlib.md5(f.read()).hexdigest()
 
-            target_dir = os.path.join(
-                getCacheDir(),
-                "egg-content",
-                checksum
-            )
+            target_dir = os.path.join(getCacheDir(), "egg-content", checksum)
 
-            zip_ref = zipfile.ZipFile(path_entry, 'r')
+            zip_ref = zipfile.ZipFile(path_entry, "r")
             zip_ref.extractall(target_dir)
             zip_ref.close()
 
@@ -425,24 +469,20 @@ def _unpackPathElement(path_entry):
 
     return path_entry
 
+
 def getPackageSearchPath(package_name):
     assert main_path is not None
 
     if package_name is None:
         return [os.getcwd(), main_path] + [
-            _unpackPathElement(path_element)
-            for path_element in
-            sys.path
+            _unpackPathElement(path_element) for path_element in sys.path
         ]
-    elif '.' in package_name:
-        parent_package_name, child_package_name = package_name.rsplit('.', 1)
+    elif "." in package_name:
+        parent_package_name, child_package_name = package_name.splitModuleBasename()
 
         result = []
         for element in getPackageSearchPath(parent_package_name):
-            package_dir = os.path.join(
-                element,
-                child_package_name
-            )
+            package_dir = os.path.join(element, child_package_name.asPath())
 
             if isPackageDir(package_dir):
                 result.append(package_dir)
@@ -460,7 +500,7 @@ def getPackageSearchPath(package_name):
             return preloaded_path
 
         def getPackageDirCandidates(element):
-            yield os.path.join(element, package_name), False
+            yield os.path.join(element, package_name.asPath()), False
 
             # Hack for PyWin32. TODO: Move this "__path__" extensions to be
             # plug-in decisions.
@@ -476,7 +516,9 @@ def getPackageSearchPath(package_name):
         return result
 
 
-def _findModuleInPath(module_name, package_name):
+def _findModuleInPath(module_name):
+    package_name, module_name = module_name.splitModuleBasename()
+
     if _debug_module_finding:
         print("_findModuleInPath: Enter", module_name, "in", package_name)
 
@@ -500,16 +542,13 @@ def _findModuleInPath(module_name, package_name):
 
     try:
         module_filename = _findModuleInPath2(
-            module_name = module_name,
-            search_path = search_path
+            module_name=module_name, search_path=search_path
         )
     except SyntaxError:
         # Warn user, as this is kind of unusual.
-        warning(
+        recursion_logger.warning(
             "%s: Module cannot be imported due to syntax errors.",
-            module_name
-              if package_name is None else
-            package_name + '.' + module_name,
+            module_name if package_name is None else package_name + "." + module_name,
         )
 
         return None
@@ -519,17 +558,15 @@ def _findModuleInPath(module_name, package_name):
 
     return module_filename
 
+
 module_search_cache = {}
 
+
 def _findModule(module_name):
     if _debug_module_finding:
-        print(
-            "_findModule: Enter to search '%s'." % (
-                module_name,
-            )
-        )
+        print("_findModule: Enter to search '%s'." % (module_name,))
 
-    assert not module_name.endswith('.'), module_name
+    assert module_name.getBasename(), module_name
 
     key = module_name
 
@@ -541,8 +578,8 @@ def _findModule(module_name):
 
         if result is ImportError:
             raise ImportError
-        else:
-            return result
+
+        return result
 
     try:
         module_search_cache[key] = _findModule2(module_name)
@@ -552,8 +589,8 @@ def _findModule(module_name):
         if new_module_name is None:
             module_search_cache[key] = ImportError
             raise
-        else:
-            module_search_cache[key] = _findModule(new_module_name)
+
+        module_search_cache[key] = _findModule(new_module_name)
 
     return module_search_cache[key]
 
@@ -567,13 +604,4 @@ def _findModule2(module_name):
     if preloaded_path is not None:
         return preloaded_path[0]
 
-    if '.' in module_name:
-        package_part = module_name[ : module_name.rfind('.') ]
-        module_name = module_name[ module_name.rfind('.') + 1 : ]
-    else:
-        package_part = None
-
-    return _findModuleInPath(
-        module_name  = module_name,
-        package_name = package_part
-    )
+    return _findModuleInPath(module_name=module_name)
