@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -31,18 +31,19 @@ the traces.
 """
 
 from nuitka.ModuleRegistry import getOwnerFromCodeName
+from nuitka.Options import isExperimental
 
 from .NodeBases import StatementBase, StatementChildHavingBase
 from .NodeMakingHelpers import (
+    makeRaiseExceptionReplacementStatement,
     makeStatementExpressionOnlyReplacementNode,
-    makeStatementsSequenceReplacementNode
+    makeStatementsSequenceReplacementNode,
 )
+from .shapes.StandardShapes import tshape_unknown
 
 
 class StatementAssignmentVariableName(StatementChildHavingBase):
-    """ Precursor of StatementAssignmentVariable used during tree building phase
-
-    """
+    """Precursor of StatementAssignmentVariable used during tree building phase"""
 
     kind = "STATEMENT_ASSIGNMENT_VARIABLE_NAME"
 
@@ -54,11 +55,7 @@ class StatementAssignmentVariableName(StatementChildHavingBase):
     def __init__(self, provider, variable_name, source, source_ref):
         assert source is not None, source_ref
 
-        StatementChildHavingBase.__init__(
-            self,
-            value      = source,
-            source_ref = source_ref
-        )
+        StatementChildHavingBase.__init__(self, value=source, source_ref=source_ref)
 
         self.variable_name = variable_name
         self.provider = provider
@@ -66,10 +63,7 @@ class StatementAssignmentVariableName(StatementChildHavingBase):
         assert not provider.isExpressionOutlineBody(), source_ref
 
     def getDetails(self):
-        return {
-            "variable_name" : self.variable_name,
-            "provider" : self.provider
-        }
+        return {"variable_name": self.variable_name, "provider": self.provider}
 
     def getVariableName(self):
         return self.variable_name
@@ -81,26 +75,20 @@ class StatementAssignmentVariableName(StatementChildHavingBase):
         # tree building.
         assert False
 
-    def getStatementNiceName(self):
+    @staticmethod
+    def getStatementNiceName():
         return "variable assignment statement"
 
-    getAssignSource = StatementChildHavingBase.childGetter("source")
-
 
 class StatementDelVariableName(StatementBase):
-    """ Precursor of StatementDelVariable used during tree building phase
-
-    """
+    """Precursor of StatementDelVariable used during tree building phase"""
 
     kind = "STATEMENT_DEL_VARIABLE_NAME"
 
     __slots__ = "variable_name", "provider", "tolerant"
 
     def __init__(self, provider, variable_name, tolerant, source_ref):
-        StatementBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        StatementBase.__init__(self, source_ref=source_ref)
 
         self.variable_name = variable_name
         self.provider = provider
@@ -113,9 +101,9 @@ class StatementDelVariableName(StatementBase):
 
     def getDetails(self):
         return {
-            "variable_name" : self.variable_name,
-            "provider" : self.provider,
-            "tolerant" : self.tolerant
+            "variable_name": self.variable_name,
+            "provider": self.provider,
+            "tolerant": self.tolerant,
         }
 
     def getVariableName(self):
@@ -130,17 +118,17 @@ class StatementDelVariableName(StatementBase):
 
 
 class StatementAssignmentVariable(StatementChildHavingBase):
-    """ Assignment to a variable from an expression.
+    """Assignment to a variable from an expression.
 
-        All assignment forms that are not to attributes, slices, subscripts
-        use this.
+    All assignment forms that are not to attributes, slices, subscripts
+    use this.
 
-        The source might be a complex expression. The target can be any kind
-        of variable, temporary, local, global, etc.
+    The source might be a complex expression. The target can be any kind
+    of variable, temporary, local, global, etc.
 
-        Assigning a variable is something we trace in a new version, this is
-        hidden behind target variable reference, which has this version once
-        it can be determined.
+    Assigning a variable is something we trace in a new version, this is
+    hidden behind target variable reference, which has this version once
+    it can be determined.
     """
 
     kind = "STATEMENT_ASSIGNMENT_VARIABLE"
@@ -148,9 +136,15 @@ class StatementAssignmentVariable(StatementChildHavingBase):
     named_child = "source"
     nice_child = "assignment source"
 
-    __slots__ = ("variable", "variable_version", "variable_trace", "inplace_suspect")
+    __slots__ = (
+        "subnode_source",
+        "variable",
+        "variable_version",
+        "variable_trace",
+        "inplace_suspect",
+    )
 
-    def __init__(self, source, variable, source_ref, version = None):
+    def __init__(self, source, variable, source_ref, version=None):
         assert source is not None, source_ref
 
         if variable is not None:
@@ -160,31 +154,26 @@ class StatementAssignmentVariable(StatementChildHavingBase):
         self.variable = variable
         self.variable_version = version
 
-        StatementChildHavingBase.__init__(
-            self,
-            value      = source,
-            source_ref = source_ref
-        )
+        StatementChildHavingBase.__init__(self, value=source, source_ref=source_ref)
 
         self.variable_trace = None
         self.inplace_suspect = None
 
-    def getDetail(self):
-        if self.variable is not None:
-            return "to variable %s" % self.variable
-        else:
-            return "to variable %s" % self.variable_name
+    def finalize(self):
+        StatementChildHavingBase.finalize(self)
+
+        del self.variable
+        del self.variable_trace
 
     def getDetails(self):
-        return {
-            "variable"      : self.variable,
-        }
+        return {"variable": self.variable}
 
     def getDetailsForDisplay(self):
         return {
-            "variable_name" : self.getVariableName(),
-            "is_temp"       : self.variable.isTempVariable(),
-            "owner"         : self.variable.getOwner().getCodeName(),
+            "variable_name": self.getVariableName(),
+            "is_temp": self.variable.isTempVariable(),
+            "var_type": self.variable.getVariableType(),
+            "owner": self.variable.getOwner().getCodeName(),
         }
 
     @classmethod
@@ -194,21 +183,19 @@ class StatementAssignmentVariable(StatementChildHavingBase):
         owner = getOwnerFromCodeName(args["owner"])
 
         if args["is_temp"] == "True":
-            variable = owner.createTempVariable(args["variable_name"])
+            variable = owner.createTempVariable(
+                args["variable_name"], temp_type=["var_type"]
+            )
         else:
             variable = owner.getProvidedVariable(args["variable_name"])
 
         del args["is_temp"]
+        del args["var_type"]
         del args["owner"]
 
         version = variable.allocateTargetNumber()
 
-        return cls(
-            variable   = variable,
-            version    = version,
-            source_ref = source_ref,
-            **args
-        )
+        return cls(variable=variable, version=version, source_ref=source_ref, **args)
 
     def makeClone(self):
         if self.variable is not None:
@@ -217,19 +204,12 @@ class StatementAssignmentVariable(StatementChildHavingBase):
             version = None
 
         return StatementAssignmentVariable(
-            source     = self.getAssignSource().makeClone(),
-            variable   = self.variable,
-            version    = version,
-            source_ref = self.source_ref
+            source=self.subnode_source.makeClone(),
+            variable=self.variable,
+            version=version,
+            source_ref=self.source_ref,
         )
 
-    getAssignSource = StatementChildHavingBase.childGetter(
-        "source"
-    )
-    setAssignSource = StatementChildHavingBase.childSetter(
-        "source"
-    )
-
     def getVariableName(self):
         return self.variable.getName()
 
@@ -249,26 +229,25 @@ class StatementAssignmentVariable(StatementChildHavingBase):
     def isInplaceSuspect(self):
         return self.inplace_suspect
 
+    def unmarkAsInplaceSuspect(self):
+        self.inplace_suspect = False
+
     def mayRaiseException(self, exception_type):
-        return self.getAssignSource().mayRaiseException(exception_type)
+        return self.subnode_source.mayRaiseException(exception_type)
 
     def computeStatement(self, trace_collection):
         # This is very complex stuff, pylint: disable=too-many-branches,too-many-return-statements
 
         # TODO: Way too ugly to have global trace kinds just here, and needs to
         # be abstracted somehow. But for now we let it live here.
-        source = self.getAssignSource()
+        source = self.subnode_source
 
         if source.isExpressionSideEffects():
             # If the assignment source has side effects, we can put them into a
             # sequence and compute that instead.
             statements = [
-                makeStatementExpressionOnlyReplacementNode(
-                    side_effect,
-                    self
-                )
-                for side_effect in
-                source.getSideEffects()
+                makeStatementExpressionOnlyReplacementNode(side_effect, self)
+                for side_effect in source.subnode_side_effects
             ]
 
             statements.append(self)
@@ -278,37 +257,38 @@ class StatementAssignmentVariable(StatementChildHavingBase):
 
             # Need to update ourselves to no longer reference the side effects,
             # but go to the wrapped thing.
-            self.setAssignSource(source.getExpression())
+            self.setChild("source", source.subnode_expression)
 
             result = makeStatementsSequenceReplacementNode(
-                statements = statements,
-                node       = self,
+                statements=statements, node=self
             )
             result.parent = parent
 
             return (
                 result.computeStatementsSequence(trace_collection),
                 "new_statements",
-"""\
-Side effects of assignments promoted to statements."""
+                """\
+Side effects of assignments promoted to statements.""",
             )
 
         # Let assignment source may re-compute first.
-        trace_collection.onExpression(self.getAssignSource())
-        source = self.getAssignSource()
+        source = trace_collection.onExpression(self.subnode_source)
 
         # No assignment will occur, if the assignment source raises, so give up
         # on this, and return it as the only side effect.
         if source.willRaiseException(BaseException):
             result = makeStatementExpressionOnlyReplacementNode(
-                expression = source,
-                node       = self
+                expression=source, node=self
             )
 
             del self.parent
 
-            return result, "new_raise", """\
-Assignment raises exception in assigned value, removed assignment."""
+            return (
+                result,
+                "new_raise",
+                """\
+Assignment raises exception in assigned value, removed assignment.""",
+            )
 
         variable = self.variable
 
@@ -323,32 +303,40 @@ Assignment raises exception in assigned value, removed assignment."""
         # are to be a feature of the trace. Assigning from known assigned is
         # supposed to be possible to eliminate. If we get that wrong, we are
         # doing it wrong.
-        if not variable.isModuleVariable() and \
-           source.isExpressionVariableRef() and \
-           source.getVariable() is variable:
+        if (
+            not variable.isModuleVariable()
+            and source.isExpressionVariableRef()
+            and source.getVariable() is variable
+        ):
 
             # A variable access that has a side effect, must be preserved,
             # so it can e.g. raise an exception, otherwise we can be fully
             # removed.
             if source.mayHaveSideEffects():
                 result = makeStatementExpressionOnlyReplacementNode(
-                    expression = source,
-                    node       = self
+                    expression=source, node=self
                 )
 
-                return result, "new_statements", """\
-Reduced assignment of %s from itself to mere access of it.""" % variable.getDescription()
+                return (
+                    result,
+                    "new_statements",
+                    """\
+Lowered assignment of %s from itself to mere access of it."""
+                    % variable.getDescription(),
+                )
             else:
-                return None, "new_statements", """\
-Removed assignment of %s from itself which is known to be defined.""" % variable.getDescription()
-
+                return (
+                    None,
+                    "new_statements",
+                    """\
+Removed assignment of %s from itself which is known to be defined."""
+                    % variable.getDescription(),
+                )
 
         # Set-up the trace to the trace collection, so future references will
         # find this assignment.
         self.variable_trace = trace_collection.onVariableSet(
-            variable    = variable,
-            version     = self.variable_version,
-            assign_node = self
+            variable=variable, version=self.variable_version, assign_node=self
         )
 
         provider = trace_collection.getOwner()
@@ -356,68 +344,56 @@ Removed assignment of %s from itself which is known to be defined.""" % variable
         if variable.hasAccessesOutsideOf(provider) is False:
             last_trace = variable.getMatchingAssignTrace(self)
 
-            if last_trace is not None:
-                if source.isCompileTimeConstant() and \
-                   not last_trace.hasLoopUsages():
-                    if not variable.isModuleVariable():
+            if last_trace is not None and not last_trace.getMergeOrNameUsageCount():
+                if source.isCompileTimeConstant():
+                    if variable.isModuleVariable():
+                        # TODO: We do not trust these yet a lot, but more might be
+                        pass
+                    else:
+                        # Unused constants can be eliminated in any case.
+                        if not last_trace.getUsageCount():
+                            if not last_trace.getPrevious().isUnassignedTrace():
+                                result = StatementDelVariable(
+                                    variable=self.variable,
+                                    version=self.variable_version,
+                                    tolerant=True,
+                                    source_ref=self.source_ref,
+                                )
+                            else:
+                                result = None
+
+                            return (
+                                result,
+                                "new_statements",
+                                "Dropped dead assignment statement to '%s'."
+                                % (self.getVariableName()),
+                            )
 
                         # Can safely forward propagate only non-mutable constants.
-                        if source.isMutable():
-                            # Something might be possible still, lets check for
-                            # ununused.
-                            if not last_trace.hasPotentialUsages() and \
-                               not last_trace.hasDefiniteUsages() and \
-                               not last_trace.getNameUsageCount():
-                                if not last_trace.getPrevious().isUninitTrace():
-                                    # TODO: We could well decide, if that's even necessary, but for now
-                                    # the "StatementDelVariable" is tasked with that.
-                                    result = StatementDelVariable(
-                                        variable   = self.variable,
-                                        version    = self.variable_version,
-                                        tolerant   = True,
-                                        source_ref = self.getSourceReference()
-                                    )
-                                else:
-                                    result = None
+                        if not source.isMutable():
+                            self.variable_trace.setReplacementNode(
+                                lambda _usage: source.makeClone()
+                            )
 
-                                return (
-                                    result,
-                                    "new_statements",
-                                    "Dropped dead assignment statement to '%s'." % (
-                                       self.getVariableName()
-                                    )
+                            if not last_trace.getPrevious().isUnassignedTrace():
+                                result = StatementDelVariable(
+                                    variable=self.variable,
+                                    version=self.variable_version,
+                                    tolerant=True,
+                                    source_ref=self.source_ref,
                                 )
-                        else:
-                            if not last_trace.getNameUsageCount():
-                                self.variable_trace.setReplacementNode(
-                                    lambda _usage : source.makeClone()
-                                )
-
-                                propagated = True
                             else:
-                                propagated = False
+                                result = None
 
-                            if not last_trace.hasPotentialUsages() and not last_trace.getNameUsageCount():
-                                if not last_trace.getPrevious().isUninitTrace():
-                                    # TODO: We could well decide, if that's even necessary, but for now
-                                    # the "StatementDelVariable" is tasked with that.
-                                    result = StatementDelVariable(
-                                        variable   = self.variable,
-                                        version    = self.variable_version,
-                                        tolerant   = True,
-                                        source_ref = self.getSourceReference()
-                                    )
-                                else:
-                                    result = None
-
-                                return (
-                                    result,
-                                    "new_statements",
-                                    "Dropped %s assignment statement to '%s'." % (
-                                       "propagated" if propagated else "dead",
-                                       self.getVariableName()
-                                    )
-                                )
+                            return (
+                                result,
+                                "new_statements",
+                                "Dropped propagated assignment statement to '%s'."
+                                % self.getVariableName(),
+                            )
+                elif source.isExpressionFunctionCreation():
+                    # TODO: Prepare for inlining.
+                    pass
                 else:
                     # More cases thinkable.
                     pass
@@ -434,30 +410,47 @@ Removed assignment of %s from itself which is known to be defined.""" % variable
         else:
             return None
 
-    def getStatementNiceName(self):
+    @staticmethod
+    def getStatementNiceName():
         return "variable assignment statement"
 
+    def getTypeShape(self):
+        # Might be finalized, e.g. due to being dead code.
+        try:
+            source = self.subnode_source
+        except AttributeError:
+            return tshape_unknown
+
+        return source.getTypeShape()
+
 
 class StatementDelVariable(StatementBase):
-    """ Deleting a variable.
+    """Deleting a variable.
 
-        All del forms that are not to attributes, slices, subscripts
-        use this.
+    All del forms that are not to attributes, slices, subscripts
+    use this.
 
-        The target can be any kind of variable, temporary, local, global, etc.
+    The target can be any kind of variable, temporary, local, global, etc.
 
-        Deleting a variable is something we trace in a new version, this is
-        hidden behind target variable reference, which has this version once
-        it can be determined.
+    Deleting a variable is something we trace in a new version, this is
+    hidden behind target variable reference, which has this version once
+    it can be determined.
 
-        Tolerance means that the value might be unset. That can happen with
-        re-formulation of ours, and Python3 exception variables.
+    Tolerance means that the value might be unset. That can happen with
+    re-formulation of ours, and Python3 exception variables.
     """
+
     kind = "STATEMENT_DEL_VARIABLE"
 
-    __slots__ = "variable", "variable_version", "variable_trace", "previous_trace", "tolerant"
+    __slots__ = (
+        "variable",
+        "variable_version",
+        "variable_trace",
+        "previous_trace",
+        "tolerant",
+    )
 
-    def __init__(self, tolerant, source_ref, variable, version = None):
+    def __init__(self, tolerant, source_ref, variable, version=None):
         if type(tolerant) is str:
             tolerant = tolerant == "True"
         assert tolerant is True or tolerant is False, repr(tolerant)
@@ -466,10 +459,7 @@ class StatementDelVariable(StatementBase):
             if version is None:
                 version = variable.allocateTargetNumber()
 
-        StatementBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        StatementBase.__init__(self, source_ref=source_ref)
 
         self.variable = variable
         self.variable_version = version
@@ -485,25 +475,20 @@ class StatementDelVariable(StatementBase):
         del self.variable_trace
         del self.previous_trace
 
-    def getDetail(self):
-        if self.variable is not None:
-            return "to variable %s" % self.variable
-        else:
-            return "to variable %s" % self.variable_name
-
     def getDetails(self):
         return {
-            "variable" : self.variable,
-            "version"  : self.variable_version,
-            "tolerant" : self.tolerant
+            "variable": self.variable,
+            "version": self.variable_version,
+            "tolerant": self.tolerant,
         }
 
     def getDetailsForDisplay(self):
         return {
-            "variable_name" : self.getVariableName(),
-            "is_temp"       : self.variable.isTempVariable(),
-            "owner"         : self.variable.getOwner().getCodeName(),
-            "tolerant"      : self.tolerant
+            "variable_name": self.getVariableName(),
+            "is_temp": self.variable.isTempVariable(),
+            "var_type": self.variable.getVariableType(),
+            "owner": self.variable.getOwner().getCodeName(),
+            "tolerant": self.tolerant,
         }
 
     @classmethod
@@ -513,21 +498,20 @@ class StatementDelVariable(StatementBase):
         owner = getOwnerFromCodeName(args["owner"])
 
         if args["is_temp"] == "True":
-            variable = owner.createTempVariable(args["variable_name"])
+            variable = owner.createTempVariable(
+                args["variable_name"], temp_type=args["var_type"]
+            )
         else:
             variable = owner.getProvidedVariable(args["variable_name"])
 
         del args["is_temp"]
+        del args["var_type"]
         del args["owner"]
 
         version = variable.allocateTargetNumber()
         variable.version_number = max(variable.version_number, version)
 
-        return cls(
-            variable   = variable,
-            source_ref = source_ref,
-            **args
-        )
+        return cls(variable=variable, source_ref=source_ref, **args)
 
     def makeClone(self):
         if self.variable is not None:
@@ -536,10 +520,10 @@ class StatementDelVariable(StatementBase):
             version = None
 
         return StatementDelVariable(
-            variable   = self.variable,
-            version    = version,
-            tolerant   = self.tolerant,
-            source_ref = self.source_ref
+            variable=self.variable,
+            version=version,
+            tolerant=self.tolerant,
+            source_ref=self.source_ref,
         )
 
     # TODO: Value propagation needs to make a difference based on this.
@@ -565,21 +549,69 @@ class StatementDelVariable(StatementBase):
     def computeStatement(self, trace_collection):
         variable = self.variable
 
-        self.previous_trace = trace_collection.getVariableCurrentTrace(variable)
-
-        # First eliminate us entirely if we can.
-        if self.tolerant and self.previous_trace.isUninitTrace():
+        # Special case, boolean temp variables need no "del".
+        # TODO: Later, these might not exist, if we forward propagate them not as "del"
+        # at all
+        if variable.isTempVariableBool():
             return (
                 None,
                 "new_statements",
-                "Removed tolerant 'del' statement of '%s' without effect." % (
-                    self.getVariableName(),
-                )
+                "Removed 'del' statement of boolean '%s' without effect."
+                % (self.getVariableName(),),
             )
 
-        # The "del" is a potential use of a value. TODO: This could be made more
-        # beautiful indication, as it's not any kind of usage.
-        self.previous_trace.addPotentialUsage()
+        self.previous_trace = trace_collection.getVariableCurrentTrace(variable)
+
+        # First eliminate us entirely if we can.
+        if self.previous_trace.mustNotHaveValue():
+            if self.tolerant:
+                return (
+                    None,
+                    "new_statements",
+                    "Removed tolerant 'del' statement of '%s' without effect."
+                    % (self.getVariableName(),),
+                )
+            else:
+                assert self.variable.isLocalVariable(), self.variable
+
+                result = makeRaiseExceptionReplacementStatement(
+                    statement=self,
+                    exception_type="UnboundLocalError",
+                    exception_value="""local variable '%s' referenced before assignment"""
+                    % variable.getName(),
+                )
+
+                return trace_collection.computedStatementResult(
+                    result,
+                    "new_raise",
+                    "Variable del of not initialized variable '%s'"
+                    % variable.getName(),
+                )
+
+        if not self.tolerant:
+            self.previous_trace.addNameUsage()
+
+        # TODO: Why doesn't this module variable check not follow from other checks done here, e.g. name usages.
+        # TODO: This currently cannot be done as releases do not create successor traces yet, although they
+        # probably should.
+        if isExperimental("del_optimization") and not variable.isModuleVariable():
+            provider = trace_collection.getOwner()
+
+            if variable.hasAccessesOutsideOf(provider) is False:
+                last_trace = variable.getMatchingDelTrace(self)
+
+                if last_trace is not None and not last_trace.getMergeOrNameUsageCount():
+                    if not last_trace.getUsageCount():
+                        result = StatementReleaseVariable(
+                            variable=variable, source_ref=self.source_ref
+                        )
+
+                        return trace_collection.computedStatementResult(
+                            result,
+                            "new_statements",
+                            "Changed del to release for variable '%s' not used afterwards."
+                            % variable.getName(),
+                        )
 
         # If not tolerant, we may exception exit now during the __del__
         if not self.tolerant and not self.previous_trace.mustHaveValue():
@@ -587,8 +619,7 @@ class StatementDelVariable(StatementBase):
 
         # Record the deletion, needs to start a new version then.
         self.variable_trace = trace_collection.onVariableDel(
-            variable = variable,
-            version  = self.variable_version
+            variable=variable, version=self.variable_version, del_node=self
         )
 
         trace_collection.onVariableContentEscapes(variable)
@@ -613,20 +644,22 @@ class StatementDelVariable(StatementBase):
                     return False
 
                 # If SSA knows, that's fine.
-                if self.previous_trace is not None and \
-                   self.previous_trace.mustHaveValue():
+                if (
+                    self.previous_trace is not None
+                    and self.previous_trace.mustHaveValue()
+                ):
                     return False
 
             return True
 
 
 class StatementReleaseVariable(StatementBase):
-    """ Releasing a variable.
+    """Releasing a variable.
 
-        Just release the value, which of course is not to be used afterwards.
+    Just release the value, which of course is not to be used afterwards.
 
-        Typical code: Function exit, try/finally release of temporary
-        variables.
+    Typical code: Function exit, try/finally release of temporary
+    variables.
     """
 
     kind = "STATEMENT_RELEASE_VARIABLE"
@@ -636,10 +669,7 @@ class StatementReleaseVariable(StatementBase):
     def __init__(self, variable, source_ref):
         assert variable is not None, source_ref
 
-        StatementBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        StatementBase.__init__(self, source_ref=source_ref)
 
         self.variable = variable
 
@@ -650,18 +680,13 @@ class StatementReleaseVariable(StatementBase):
         del self.variable_trace
         del self.parent
 
-    def getDetail(self):
-        return "of variable %s" % self.variable
-
     def getDetails(self):
-        return {
-            "variable" : self.variable
-        }
+        return {"variable": self.variable}
 
     def getDetailsForDisplay(self):
         return {
-            "variable_name" : self.variable.getName(),
-            "owner"    : self.variable.getOwner().getCodeName()
+            "variable_name": self.variable.getName(),
+            "owner": self.variable.getOwner().getCodeName(),
         }
 
     @classmethod
@@ -673,10 +698,7 @@ class StatementReleaseVariable(StatementBase):
 
         variable = owner.getProvidedVariable(args["variable_name"])
 
-        return cls(
-            variable   = variable,
-            source_ref = source_ref
-        )
+        return cls(variable=variable, source_ref=source_ref)
 
     def getVariable(self):
         return self.variable
@@ -688,15 +710,22 @@ class StatementReleaseVariable(StatementBase):
         self.variable = variable
 
     def computeStatement(self, trace_collection):
+        if self.variable.isParameterVariable():
+            if self.variable.getOwner().isAutoReleaseVariable(self.variable):
+                return (
+                    None,
+                    "new_statements",
+                    "Original parameter variable value %s is not released."
+                    % (self.variable.getDescription()),
+                )
+
         self.variable_trace = trace_collection.getVariableCurrentTrace(self.variable)
 
-        if self.variable_trace.isUninitTrace():
+        if self.variable_trace.mustNotHaveValue():
             return (
                 None,
                 "new_statements",
-                "Uninitialized %s is not released." % (
-                    self.variable.getDescription()
-                )
+                "Uninitialized %s is not released." % (self.variable.getDescription()),
             )
 
         trace_collection.onVariableContentEscapes(self.variable)
