@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,37 +23,33 @@ for some types, so it is still useful. Also calls to it can be accelerated
 slightly.
 """
 
-from .ExpressionBases import ExpressionChildrenHavingBase
+from .ExpressionBases import ExpressionChildHavingBase
 
 
-class ExpressionBuiltinHash(ExpressionChildrenHavingBase):
+class ExpressionBuiltinHash(ExpressionChildHavingBase):
     kind = "EXPRESSION_BUILTIN_HASH"
 
-    named_children = (
-        "value",
-    )
+    named_child = "value"
 
     def __init__(self, value, source_ref):
-        ExpressionChildrenHavingBase.__init__(
-            self,
-            values     = {
-                "value" : value,
-            },
-            source_ref = source_ref
-        )
-
-    getValue = ExpressionChildrenHavingBase.childGetter(
-        "value"
-    )
+        ExpressionChildHavingBase.__init__(self, value=value, source_ref=source_ref)
 
     def computeExpression(self, trace_collection):
-        value = self.getValue()
+        value = self.subnode_value
 
-        # TODO: Have a computation slot for hashing.
+        # TODO: Have a computation slot for hashing and specialize for known cases.
         if not value.isKnownToBeHashable():
             trace_collection.onExceptionRaiseExit(BaseException)
 
+        # TODO: Static raise if it's known not to be hashable.
+
         return self, None, None
 
     def mayRaiseException(self, exception_type):
-        return not self.getValue().isKnownToBeHashable()
+        return (
+            self.subnode_value.mayRaiseException(exception_type)
+            or not self.subnode_value.isKnownToBeHashable()
+        )
+
+    def mayRaiseExceptionOperation(self):
+        return not self.subnode_value.isKnownToBeHashable()
