@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,20 +24,16 @@ it that could be in-lined sometimes for more static analysis.
 
 from nuitka.specs import BuiltinParameterSpecs
 
-from .ExpressionBases import ExpressionChildrenHavingBase
+from .ExpressionBases import (
+    ExpressionChildHavingBase,
+    ExpressionChildrenHavingBase,
+)
 
 
-class ExpressionBuiltinSumBase(ExpressionChildrenHavingBase):
+class ExpressionBuiltinSumMixin(object):
 
     builtin_spec = BuiltinParameterSpecs.builtin_sum_spec
 
-    def __init__(self, values, source_ref):
-        ExpressionChildrenHavingBase.__init__(
-            self,
-            values     = values,
-            source_ref = source_ref
-        )
-
     def computeBuiltinSpec(self, trace_collection, given_values):
         assert self.builtin_spec is not None, self
 
@@ -49,80 +45,50 @@ class ExpressionBuiltinSumBase(ExpressionChildrenHavingBase):
             return self, None, None
 
         return trace_collection.getCompileTimeComputationResult(
-            node        = self,
-            computation = lambda : self.builtin_spec.simulateCall(
-                given_values
-            ),
-            description = "Built-in call to '%s' computed." % (
-                self.builtin_spec.getName()
-            )
+            node=self,
+            computation=lambda: self.builtin_spec.simulateCall(given_values),
+            description="Built-in call to '%s' computed."
+            % (self.builtin_spec.getName()),
         )
 
 
-class ExpressionBuiltinSum1(ExpressionBuiltinSumBase):
+class ExpressionBuiltinSum1(ExpressionBuiltinSumMixin, ExpressionChildHavingBase):
     kind = "EXPRESSION_BUILTIN_SUM1"
 
-    named_children = (
-        "sequence",
-    )
+    named_child = "sequence"
 
     def __init__(self, sequence, source_ref):
         assert sequence is not None
 
-        ExpressionBuiltinSumBase.__init__(
-            self,
-            values     = {
-                "sequence" : sequence,
-            },
-            source_ref = source_ref
-        )
-
-    getSequence = ExpressionChildrenHavingBase.childGetter("sequence")
+        ExpressionChildHavingBase.__init__(self, value=sequence, source_ref=source_ref)
 
     def computeExpression(self, trace_collection):
-        sequence  = self.getSequence()
+        sequence = self.subnode_sequence
 
         # TODO: Protect against large xrange constants
         return self.computeBuiltinSpec(
-            trace_collection = trace_collection,
-            given_values     = (
-                sequence,
-            )
+            trace_collection=trace_collection, given_values=(sequence,)
         )
 
-class ExpressionBuiltinSum2(ExpressionBuiltinSumBase):
+
+class ExpressionBuiltinSum2(ExpressionBuiltinSumMixin, ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_SUM2"
 
-    named_children = (
-        "sequence",
-        "start"
-    )
+    named_children = ("sequence", "start")
 
     def __init__(self, sequence, start, source_ref):
         assert sequence is not None
         assert start is not None
 
-        ExpressionBuiltinSumBase.__init__(
-            self,
-            values     = {
-                "sequence" : sequence,
-                "start" : start
-            },
-            source_ref = source_ref
+        ExpressionChildrenHavingBase.__init__(
+            self, values={"sequence": sequence, "start": start}, source_ref=source_ref
         )
 
-    getSequence = ExpressionChildrenHavingBase.childGetter("sequence")
-    getStart = ExpressionChildrenHavingBase.childGetter("start")
-
     def computeExpression(self, trace_collection):
-        sequence  = self.getSequence()
-        start = self.getStart()
+        sequence = self.subnode_sequence
+        start = self.subnode_start
 
         # TODO: Protect against large xrange constants
         return self.computeBuiltinSpec(
-            trace_collection = trace_collection,
-            given_values     = (
-                sequence,
-                start
-            )
+            trace_collection=trace_collection, given_values=(sequence, start)
         )
