@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,7 +19,11 @@
 
 """
 
-from .ExpressionBases import ExpressionBase, ExpressionChildrenHavingBase
+from .ExpressionBases import (
+    ExpressionBase,
+    ExpressionChildHavingBase,
+    ExpressionChildrenHavingBase,
+)
 from .NodeBases import StatementBase, StatementChildrenHavingBase
 from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
 
@@ -34,18 +38,26 @@ class StatementRaiseExceptionMixin(object):
         return True
 
 
-class StatementRaiseException(StatementRaiseExceptionMixin, StatementChildrenHavingBase):
+class StatementRaiseException(
+    StatementRaiseExceptionMixin, StatementChildrenHavingBase
+):
     kind = "STATEMENT_RAISE_EXCEPTION"
 
     named_children = (
         "exception_type",
         "exception_value",
         "exception_trace",
-        "exception_cause"
+        "exception_cause",
     )
 
-    def __init__(self, exception_type, exception_value, exception_trace,
-                 exception_cause, source_ref):
+    def __init__(
+        self,
+        exception_type,
+        exception_value,
+        exception_trace,
+        exception_cause,
+        source_ref,
+    ):
         assert exception_type is not None
 
         if exception_type is None:
@@ -56,119 +68,113 @@ class StatementRaiseException(StatementRaiseExceptionMixin, StatementChildrenHav
 
         StatementChildrenHavingBase.__init__(
             self,
-            values     = {
-                "exception_type"  : exception_type,
-                "exception_value" : exception_value,
-                "exception_trace" : exception_trace,
-                "exception_cause" : exception_cause
+            values={
+                "exception_type": exception_type,
+                "exception_value": exception_value,
+                "exception_trace": exception_trace,
+                "exception_cause": exception_cause,
             },
-            source_ref = source_ref
+            source_ref=source_ref,
         )
 
         self.reraise_finally = False
 
-    getExceptionType = StatementChildrenHavingBase.childGetter(
-        "exception_type"
-    )
-    getExceptionValue = StatementChildrenHavingBase.childGetter(
-        "exception_value"
-    )
-    getExceptionTrace = StatementChildrenHavingBase.childGetter(
-        "exception_trace"
-    )
-    getExceptionCause = StatementChildrenHavingBase.childGetter(
-        "exception_cause"
-    )
-
     def computeStatement(self, trace_collection):
-        trace_collection.onExpression(
-            expression = self.getExceptionType(),
-            allow_none = True
+        exception_type = trace_collection.onExpression(
+            expression=self.subnode_exception_type, allow_none=True
         )
-        exception_type = self.getExceptionType()
 
         # TODO: Limit by type.
         trace_collection.onExceptionRaiseExit(BaseException)
 
-        if exception_type is not None and \
-           exception_type.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementExpressionOnlyReplacementNode
+        if exception_type is not None and exception_type.willRaiseException(
+            BaseException
+        ):
+            from .NodeMakingHelpers import (
+                makeStatementExpressionOnlyReplacementNode,
+            )
 
             result = makeStatementExpressionOnlyReplacementNode(
-                expression = exception_type,
-                node       = self
+                expression=exception_type, node=self
             )
 
-            return result, "new_raise", """\
-Explicit raise already raises implicitly building exception type."""
+            return (
+                result,
+                "new_raise",
+                """\
+Explicit raise already raises implicitly building exception type.""",
+            )
 
-        trace_collection.onExpression(
-            expression = self.getExceptionValue(),
-            allow_none = True
+        exception_value = trace_collection.onExpression(
+            expression=self.subnode_exception_value, allow_none=True
         )
-        exception_value = self.getExceptionValue()
 
-        if exception_value is not None and exception_value.willRaiseException(BaseException):
+        if exception_value is not None and exception_value.willRaiseException(
+            BaseException
+        ):
             result = makeStatementOnlyNodesFromExpressions(
-                expressions = (
-                    exception_type,
-                    exception_value
-                )
+                expressions=(exception_type, exception_value)
             )
 
-            return result, "new_raise", """\
-Explicit raise already raises implicitly building exception value."""
+            return (
+                result,
+                "new_raise",
+                """\
+Explicit raise already raises implicitly building exception value.""",
+            )
 
-        trace_collection.onExpression(
-            expression = self.getExceptionTrace(),
-            allow_none = True
+        exception_trace = trace_collection.onExpression(
+            expression=self.subnode_exception_trace, allow_none=True
         )
-        exception_trace = self.getExceptionTrace()
 
-        if exception_trace is not None and \
-           exception_trace.willRaiseException(BaseException):
+        if exception_trace is not None and exception_trace.willRaiseException(
+            BaseException
+        ):
             result = makeStatementOnlyNodesFromExpressions(
-                expressions = (
-                    exception_type,
-                    exception_value,
-                    exception_trace
-                )
+                expressions=(exception_type, exception_value, exception_trace)
             )
 
-            return result, "new_raise", """\
-Explicit raise already raises implicitly building exception traceback."""
+            return (
+                result,
+                "new_raise",
+                """\
+Explicit raise already raises implicitly building exception traceback.""",
+            )
 
-        trace_collection.onExpression(
-            expression = self.getExceptionCause(),
-            allow_none = True
+        exception_cause = trace_collection.onExpression(
+            expression=self.subnode_exception_cause, allow_none=True
         )
-        exception_cause = self.getExceptionCause()
 
-        if exception_cause is not None and \
-           exception_cause.willRaiseException(BaseException):
+        if exception_cause is not None and exception_cause.willRaiseException(
+            BaseException
+        ):
             result = makeStatementOnlyNodesFromExpressions(
-                expressions = (
-                    exception_type,
-                    exception_cause,
-                )
+                expressions=(exception_type, exception_cause)
             )
 
-            return result, "new_raise", """
-Explicit raise already raises implicitly building exception cause."""
+            return (
+                result,
+                "new_raise",
+                """
+Explicit raise already raises implicitly building exception cause.""",
+            )
 
         return self, None, None
 
-    def needsFrame(self):
+    @staticmethod
+    def needsFrame():
         return True
 
-    def getStatementNiceName(self):
+    @staticmethod
+    def getStatementNiceName():
         return "exception raise statement"
 
 
 class StatementRaiseExceptionImplicit(StatementRaiseException):
     kind = "STATEMENT_RAISE_EXCEPTION_IMPLICIT"
 
-    def getStatementNiceName(self):
+    @staticmethod
+    def getStatementNiceName():
         return "implicit exception raise statement"
 
 
@@ -185,15 +191,17 @@ class StatementReraiseException(StatementRaiseExceptionMixin, StatementBase):
 
     # TODO: Not actually true, leads to wrong frame attached if there is
     # no pending exception.
-    def needsFrame(self):
+    @staticmethod
+    def needsFrame():
         return False
 
-    def getStatementNiceName(self):
+    @staticmethod
+    def getStatementNiceName():
         return "exception re-raise statement"
 
 
 class ExpressionRaiseException(ExpressionChildrenHavingBase):
-    """ This node type is only produced via optimization.
+    """This node type is only produced via optimization.
 
     CPython only knows exception raising as a statement, but often the raising
     of exceptions can be predicted to occur as part of an expression, which it
@@ -202,19 +210,16 @@ class ExpressionRaiseException(ExpressionChildrenHavingBase):
 
     kind = "EXPRESSION_RAISE_EXCEPTION"
 
-    named_children = (
-        "exception_type",
-        "exception_value"
-    )
+    named_children = ("exception_type", "exception_value")
 
     def __init__(self, exception_type, exception_value, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
-            values     = {
-                "exception_type"  : exception_type,
-                "exception_value" : exception_value
+            values={
+                "exception_type": exception_type,
+                "exception_value": exception_value,
             },
-            source_ref = source_ref
+            source_ref=source_ref,
         )
 
     def willRaiseException(self, exception_type):
@@ -222,13 +227,6 @@ class ExpressionRaiseException(ExpressionChildrenHavingBase):
         # closely if it is predictable.
         return exception_type is BaseException
 
-    getExceptionType = ExpressionChildrenHavingBase.childGetter(
-        "exception_type"
-    )
-    getExceptionValue = ExpressionChildrenHavingBase.childGetter(
-        "exception_value"
-    )
-
     def computeExpression(self, trace_collection):
         return self, None, None
 
@@ -237,52 +235,83 @@ class ExpressionRaiseException(ExpressionChildrenHavingBase):
 
         del self.parent
 
-        return result, "new_raise", """\
-Propagated implicit raise expression to raise statement."""
+        return (
+            result,
+            "new_raise",
+            """\
+Propagated implicit raise expression to raise statement.""",
+        )
 
     def asStatement(self):
         return StatementRaiseExceptionImplicit(
-            exception_type  = self.getExceptionType(),
-            exception_value = self.getExceptionValue(),
-            exception_trace = None,
-            exception_cause = None,
-            source_ref      = self.getSourceReference()
+            exception_type=self.subnode_exception_type,
+            exception_value=self.subnode_exception_value,
+            exception_trace=None,
+            exception_cause=None,
+            source_ref=self.source_ref,
         )
 
 
-class ExpressionBuiltinMakeException(ExpressionChildrenHavingBase):
+class ExpressionBuiltinMakeException(ExpressionChildHavingBase):
     kind = "EXPRESSION_BUILTIN_MAKE_EXCEPTION"
 
-    named_children = (
-        "args",
-    )
+    named_child = "args"
+
+    __slots__ = ("exception_name",)
 
     def __init__(self, exception_name, args, source_ref):
-        ExpressionChildrenHavingBase.__init__(
-            self,
-            values     = {
-                "args" : tuple(args),
-            },
-            source_ref = source_ref
+        ExpressionChildHavingBase.__init__(
+            self, value=tuple(args), source_ref=source_ref
         )
 
         self.exception_name = exception_name
 
     def getDetails(self):
-        return {
-            "exception_name" : self.exception_name
-        }
+        return {"exception_name": self.exception_name}
 
     def getExceptionName(self):
         return self.exception_name
 
-    getArgs = ExpressionChildrenHavingBase.childGetter("args")
-
     def computeExpression(self, trace_collection):
         return self, None, None
 
     def mayRaiseException(self, exception_type):
-        for arg in self.getArgs():
+        for arg in self.subnode_args:
+            if arg.mayRaiseException(exception_type):
+                return True
+
+        return False
+
+
+class ExpressionBuiltinMakeExceptionImportError(ExpressionChildrenHavingBase):
+    """ Python3 ImportError dedicated node with extra arguments. """
+
+    kind = "EXPRESSION_BUILTIN_MAKE_EXCEPTION_IMPORT_ERROR"
+
+    named_children = ("args", "name", "path")
+
+    __slots__ = ("exception_name",)
+
+    def __init__(self, exception_name, args, name, path, source_ref):
+        ExpressionChildrenHavingBase.__init__(
+            self,
+            values={"args": tuple(args), "name": name, "path": path},
+            source_ref=source_ref,
+        )
+
+        self.exception_name = exception_name
+
+    def getDetails(self):
+        return {"exception_name": self.exception_name}
+
+    def getExceptionName(self):
+        return self.exception_name
+
+    def computeExpression(self, trace_collection):
+        return self, None, None
+
+    def mayRaiseException(self, exception_type):
+        for arg in self.subnode_args:
             if arg.mayRaiseException(exception_type):
                 return True
 
@@ -293,10 +322,7 @@ class ExpressionCaughtExceptionTypeRef(ExpressionBase):
     kind = "EXPRESSION_CAUGHT_EXCEPTION_TYPE_REF"
 
     def __init__(self, source_ref):
-        ExpressionBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        ExpressionBase.__init__(self, source_ref=source_ref)
 
     def finalize(self):
         del self.parent
@@ -314,10 +340,7 @@ class ExpressionCaughtExceptionValueRef(ExpressionBase):
     kind = "EXPRESSION_CAUGHT_EXCEPTION_VALUE_REF"
 
     def __init__(self, source_ref):
-        ExpressionBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        ExpressionBase.__init__(self, source_ref=source_ref)
 
     def finalize(self):
         del self.parent
@@ -335,10 +358,7 @@ class ExpressionCaughtExceptionTracebackRef(ExpressionBase):
     kind = "EXPRESSION_CAUGHT_EXCEPTION_TRACEBACK_REF"
 
     def __init__(self, source_ref):
-        ExpressionBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        ExpressionBase.__init__(self, source_ref=source_ref)
 
     def finalize(self):
         del self.parent
