@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -27,30 +27,48 @@ from nuitka.PythonVersions import python_version
 from nuitka.utils.InstanceCounters import counted_del, counted_init
 
 # These defaults have changed with Python versions.
-_future_division_default = python_version >= 300
-_future_absolute_import_default = python_version >= 300
-_future_generator_stop_default = python_version >= 370
+_future_division_default = python_version >= 0x300
+_future_absolute_import_default = python_version >= 0x300
+_future_generator_stop_default = python_version >= 0x370
+_future_annotations_default = python_version >= 0x400
+
+
+class FutureSpec(object):
+    __slots__ = (
+        "future_division",
+        "unicode_literals",
+        "absolute_import",
+        "future_print",
+        "barry_bdfl",
+        "generator_stop",
+        "future_annotations",
+    )
 
-class FutureSpec:
     @counted_init
     def __init__(self):
-        self.future_division  = _future_division_default
+        self.future_division = _future_division_default
         self.unicode_literals = False
-        self.absolute_import  = _future_absolute_import_default
-        self.future_print     = False
-        self.barry_bdfl       = False
-        self.generator_stop   = _future_generator_stop_default
+        self.absolute_import = _future_absolute_import_default
+        self.future_print = False
+        self.barry_bdfl = False
+        self.generator_stop = _future_generator_stop_default
+        self.future_annotations = _future_annotations_default
 
     __del__ = counted_del()
 
+    def __repr__(self):
+        return "<FutureSpec %s>" % ",".join(self.asFlags())
+
     def clone(self):
         result = FutureSpec()
 
-        result.future_division   = self.future_division
-        result.unicode_literals  = self.unicode_literals
-        result.absolute_import   = self.absolute_import
-        result.future_print      = self.future_print
-        result.barry_bdfl        = self.barry_bdfl
+        result.future_division = self.future_division
+        result.unicode_literals = self.unicode_literals
+        result.absolute_import = self.absolute_import
+        result.future_print = self.future_print
+        result.barry_bdfl = self.barry_bdfl
+        result.generator_stop = self.generator_stop
+        result.future_annotations = result.future_annotations
 
         return result
 
@@ -84,38 +102,49 @@ class FutureSpec:
     def isGeneratorStop(self):
         return self.generator_stop
 
-    def asFlags(self):
-        """ Create a list of C identifiers to represent the flag values.
+    def enableFutureAnnotations(self):
+        self.future_annotations = True
 
-            This is for use in code generation and to restore from
-            saved modules.
+    def isFutureAnnotations(self):
+        return self.future_annotations
+
+    def asFlags(self):
+        """Create a list of C identifiers to represent the flag values.
+
+        This is for use in code generation and to restore from
+        saved modules.
         """
 
         result = []
 
-        if python_version < 300 and self.future_division:
+        if python_version < 0x300 and self.future_division:
             result.append("CO_FUTURE_DIVISION")
 
         if self.unicode_literals:
             result.append("CO_FUTURE_UNICODE_LITERALS")
 
-        if python_version < 300 and self.absolute_import:
+        if python_version < 0x300 and self.absolute_import:
             result.append("CO_FUTURE_ABSOLUTE_IMPORT")
 
-        if python_version < 300 and self.future_print:
+        if python_version < 0x300 and self.future_print:
             result.append("CO_FUTURE_PRINT_FUNCTION")
 
-        if python_version >= 300 and self.barry_bdfl:
+        if python_version >= 0x300 and self.barry_bdfl:
             result.append("CO_FUTURE_BARRY_AS_BDFL")
 
-        if python_version >= 350 and self.generator_stop:
+        if 0x350 <= python_version < 0x370 and self.generator_stop:
             result.append("CO_FUTURE_GENERATOR_STOP")
 
+        if python_version >= 0x370 and self.future_annotations:
+            result.append("CO_FUTURE_ANNOTATIONS")
+
         return tuple(result)
 
 
 def fromFlags(flags):
-    flags = flags.split(',')
+    flags = flags.split(",")
+    if "" in flags:
+        flags.remove("")
 
     result = FutureSpec()
 
@@ -137,4 +166,7 @@ def fromFlags(flags):
     if "CO_FUTURE_GENERATOR_STOP" in flags:
         result.enableGeneratorStop()
 
+    # Check if we are going to give similar results than what we got.
+    assert tuple(result.asFlags()) == tuple(flags), (result, result.asFlags(), flags)
+
     return result
