@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -15,7 +15,7 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-""" Globals/locals/singe arg dir nodes
+""" Globals/locals/single arg dir nodes
 
 These nodes give access to variables, highly problematic, because using them,
 the code may change or access anything about them, so nothing can be trusted
@@ -25,7 +25,7 @@ The "dir()" call without arguments is reformulated to locals or globals calls.
 """
 
 from .ConstantRefNodes import makeConstantRefNode
-from .DictionaryNodes import ExpressionKeyValuePair, ExpressionMakeDict
+from .DictionaryNodes import ExpressionKeyValuePair, makeExpressionMakeDict
 from .ExpressionBases import ExpressionBase, ExpressionBuiltinSingleArgBase
 from .VariableRefNodes import ExpressionTempVariableRef, ExpressionVariableRef
 
@@ -34,10 +34,7 @@ class ExpressionBuiltinGlobals(ExpressionBase):
     kind = "EXPRESSION_BUILTIN_GLOBALS"
 
     def __init__(self, source_ref):
-        ExpressionBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        ExpressionBase.__init__(self, source_ref=source_ref)
 
     def finalize(self):
         del self.parent
@@ -45,10 +42,12 @@ class ExpressionBuiltinGlobals(ExpressionBase):
     def computeExpressionRaw(self, trace_collection):
         return self, None, None
 
-    def mayHaveSideEffects(self):
+    @staticmethod
+    def mayHaveSideEffects():
         return False
 
-    def mayRaiseException(self, exception_type):
+    @staticmethod
+    def mayRaiseException(exception_type):
         return False
 
 
@@ -58,10 +57,7 @@ class ExpressionBuiltinLocalsBase(ExpressionBase):
     __slots__ = ("variable_traces", "locals_scope")
 
     def __init__(self, locals_scope, source_ref):
-        ExpressionBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        ExpressionBase.__init__(self, source_ref=source_ref)
 
         self.variable_traces = None
         self.locals_scope = locals_scope
@@ -79,25 +75,23 @@ class ExpressionBuiltinLocalsBase(ExpressionBase):
     def getVariableTraces(self):
         return self.variable_traces
 
+    def getLocalsScope(self):
+        return self.locals_scope
+
 
 class ExpressionBuiltinLocalsUpdated(ExpressionBuiltinLocalsBase):
     kind = "EXPRESSION_BUILTIN_LOCALS_UPDATED"
 
     def __init__(self, locals_scope, source_ref):
         ExpressionBuiltinLocalsBase.__init__(
-            self,
-            locals_scope = locals_scope,
-            source_ref   = source_ref
+            self, locals_scope=locals_scope, source_ref=source_ref
         )
 
         assert locals_scope is not None
 
-    def getLocalsScope(self):
-        return self.locals_scope
-
     def computeExpressionRaw(self, trace_collection):
         # Just inform the collection that all escaped.
-        self.variable_traces = trace_collection.onLocalsUsage(self.getParentVariableProvider())
+        self.variable_traces = trace_collection.onLocalsUsage(self.locals_scope)
 
         trace_collection.onLocalsDictEscaped(self.locals_scope)
 
@@ -109,9 +103,7 @@ class ExpressionBuiltinLocalsRef(ExpressionBuiltinLocalsBase):
 
     def __init__(self, locals_scope, source_ref):
         ExpressionBuiltinLocalsBase.__init__(
-            self,
-            locals_scope = locals_scope,
-            source_ref   = source_ref
+            self, locals_scope=locals_scope, source_ref=source_ref
         )
 
     def getLocalsScope(self):
@@ -119,23 +111,20 @@ class ExpressionBuiltinLocalsRef(ExpressionBuiltinLocalsBase):
 
     def computeExpressionRaw(self, trace_collection):
         if self.locals_scope.isMarkedForPropagation():
-            result = ExpressionMakeDict(
-                pairs      = (
+            result = makeExpressionMakeDict(
+                pairs=(
                     ExpressionKeyValuePair(
-                        key        = makeConstantRefNode(
-                            constant   = variable_name,
-                            source_ref = self.source_ref
+                        key=makeConstantRefNode(
+                            constant=variable_name, source_ref=self.source_ref
                         ),
-                        value      = ExpressionTempVariableRef(
-                            variable   = variable,
-                            source_ref = self.source_ref
+                        value=ExpressionTempVariableRef(
+                            variable=variable, source_ref=self.source_ref
                         ),
-                        source_ref = self.source_ref
+                        source_ref=self.source_ref,
                     )
-                    for variable_name, variable in
-                    self.locals_scope.getPropagationVariables().items()
+                    for variable_name, variable in self.locals_scope.getPropagationVariables().items()
                 ),
-                source_ref = self.source_ref
+                source_ref=self.source_ref,
             )
 
             new_result = result.computeExpressionRaw(trace_collection)
@@ -146,10 +135,9 @@ class ExpressionBuiltinLocalsRef(ExpressionBuiltinLocalsBase):
 
             return result, "new_expression", "Propagated locals dictionary reference."
 
-
         # Just inform the collection that all escaped unless it is abortative.
         if not self.getParent().isStatementReturn():
-            trace_collection.onLocalsUsage(self.getParentVariableProvider())
+            trace_collection.onLocalsUsage(locals_scope=self.locals_scope)
 
         return self, None, None
 
@@ -160,10 +148,15 @@ class ExpressionBuiltinLocalsCopy(ExpressionBuiltinLocalsBase):
     def computeExpressionRaw(self, trace_collection):
         # Just inform the collection that all escaped.
 
-        self.variable_traces = trace_collection.onLocalsUsage(self.getParentVariableProvider())
+        self.variable_traces = trace_collection.onLocalsUsage(
+            locals_scope=self.locals_scope
+        )
 
         for variable, variable_trace in self.variable_traces:
-            if not variable_trace.mustHaveValue() and not variable_trace.mustNotHaveValue():
+            if (
+                not variable_trace.mustHaveValue()
+                and not variable_trace.mustNotHaveValue()
+            ):
                 return self, None, None
 
             # Other locals elsewhere.
@@ -175,33 +168,34 @@ class ExpressionBuiltinLocalsCopy(ExpressionBuiltinLocalsBase):
             if variable_trace.mustHaveValue():
                 pairs.append(
                     ExpressionKeyValuePair(
-                        key        = makeConstantRefNode(
-                            constant      = variable.getName(),
-                            user_provided = True,
-                            source_ref    = self.source_ref
+                        key=makeConstantRefNode(
+                            constant=variable.getName(),
+                            user_provided=True,
+                            source_ref=self.source_ref,
                         ),
-                        value      = ExpressionVariableRef(
-                            variable   = variable,
-                            source_ref = self.source_ref
+                        value=ExpressionVariableRef(
+                            variable=variable, source_ref=self.source_ref
                         ),
-                        source_ref = self.source_ref
+                        source_ref=self.source_ref,
                     )
                 )
 
         # Locals is sorted of course.
         def _sorted(pairs):
+            # TODO: Should use a locals scope for this.
             names = self.getParentVariableProvider().getLocalVariableNames()
 
-            return sorted(
-                pairs,
-                key = lambda pair: names.index(
-                    pair.getKey().getCompileTimeConstant()
-                ),
+            return tuple(
+                sorted(
+                    pairs,
+                    key=lambda pair: names.index(
+                        pair.subnode_key.getCompileTimeConstant()
+                    ),
+                )
             )
 
-        result = ExpressionMakeDict(
-            pairs      = _sorted(pairs),
-            source_ref = self.source_ref
+        result = makeExpressionMakeDict(
+            pairs=_sorted(pairs), source_ref=self.source_ref
         )
 
         return result, "new_expression", "Statically predicted locals dictionary."
