@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -95,12 +95,12 @@ class PythonModuleBase(NodeBase):
                 module_name=package_name,
                 parent_package=None,
                 level=1,
-                warn=python_version < 300,
+                warn=python_version < 0x300,
             )
 
             # TODO: Temporary, if we can't find the package for Python3.3 that
             # is semi-OK, maybe.
-            if python_version >= 300 and not package_filename:
+            if python_version >= 0x300 and not package_filename:
                 return ()
 
             if package_name == "uniconvertor.app.modules":
@@ -156,7 +156,7 @@ class PythonModuleBase(NodeBase):
             all.
 
         """
-        return os.path.abspath(self.getSourceReference().getFilename())
+        return os.path.abspath(self.source_ref.getFilename())
 
     def getCompileTimeDirectory(self):
         """The compile time directory for the module.
@@ -218,10 +218,6 @@ class CompiledPythonModule(
     kind = "COMPILED_PYTHON_MODULE"
 
     named_children = ("body", "functions")
-    getBody = ChildrenHavingMixin.childGetter("body")
-    setBody = ChildrenHavingMixin.childSetter("body")
-    getFunctions = ChildrenHavingMixin.childGetter("functions")
-    setFunctions = ChildrenHavingMixin.childSetter("functions")
 
     checkers = {"body": checkStatementsSequenceOrNone}
 
@@ -246,7 +242,12 @@ class CompiledPythonModule(
 
         self.variables = {}
 
+        # Functions that have been used.
         self.active_functions = OrderedSet()
+
+        # Functions that should be visited again.
+        self.visited_functions = set()
+
         self.cross_used_functions = OrderedSet()
 
         self.used_modules = OrderedSet()
@@ -263,6 +264,10 @@ class CompiledPythonModule(
             self.module_dict_name, "module_dict", self
         )
 
+    @staticmethod
+    def isCompiledPythonModule():
+        return True
+
     def getDetails(self):
         return {
             "filename": self.source_ref.getFilename(),
@@ -350,10 +355,6 @@ class CompiledPythonModule(
 
         return graph
 
-    @staticmethod
-    def isCompiledPythonModule():
-        return True
-
     def getSourceCode(self):
         if self.source_code is not None:
             return self.source_code
@@ -420,15 +421,18 @@ class CompiledPythonModule(
         return encodePythonIdentifierToC(self.getFullName())
 
     def addFunction(self, function_body):
-        functions = self.getFunctions()
+        functions = self.subnode_functions
         assert function_body not in functions
         functions += (function_body,)
-        self.setFunctions(functions)
+        self.setChild("functions", functions)
 
     def startTraversal(self):
         self.used_modules = OrderedSet()
         self.active_functions = OrderedSet()
 
+    def restartTraversal(self):
+        self.visited_functions = set()
+
     def addUsedModule(self, key):
         self.used_modules.add(key)
 
@@ -436,7 +440,7 @@ class CompiledPythonModule(
         return self.used_modules
 
     def addUsedFunction(self, function_body):
-        assert function_body in self.getFunctions()
+        assert function_body in self.subnode_functions, function_body
 
         assert (
             function_body.isExpressionFunctionBody()
@@ -446,9 +450,10 @@ class CompiledPythonModule(
             or function_body.isExpressionAsyncgenObjectBody()
         )
 
-        result = function_body not in self.active_functions
-        if result:
-            self.active_functions.add(function_body)
+        self.active_functions.add(function_body)
+
+        result = function_body not in self.visited_functions
+        self.visited_functions.add(function_body)
 
         return result
 
@@ -456,7 +461,7 @@ class CompiledPythonModule(
         return self.active_functions
 
     def getUnusedFunctions(self):
-        for function in self.getFunctions():
+        for function in self.subnode_functions:
             if function not in self.active_functions:
                 yield function
 
@@ -468,7 +473,7 @@ class CompiledPythonModule(
         return self.cross_used_functions
 
     def getFunctionFromCodeName(self, code_name):
-        for function in self.getFunctions():
+        for function in self.subnode_functions:
             if function.getCodeName() == code_name:
                 return function
 
@@ -487,11 +492,13 @@ class CompiledPythonModule(
         return result.replace(")", "").replace("(", "")
 
     def computeModule(self):
+        self.restartTraversal()
+
         old_collection = self.trace_collection
 
         self.trace_collection = TraceCollectionModule(self)
 
-        module_body = self.getBody()
+        module_body = self.subnode_body
 
         if module_body is not None:
             result = module_body.computeStatementsSequence(
@@ -499,7 +506,7 @@ class CompiledPythonModule(
             )
 
             if result is not module_body:
-                self.setBody(result)
+                self.setChild("body", result)
 
         new_modules = self.attemptRecursion()
 
@@ -787,7 +794,7 @@ class PythonShlibModule(PythonModuleBase):
         del self.used_modules
 
     def getFilename(self):
-        return self.getSourceReference().getFilename()
+        return self.source_ref.getFilename()
 
     def startTraversal(self):
         pass
@@ -804,7 +811,7 @@ class PythonShlibModule(PythonModuleBase):
     def _readPyPIFile(self):
         """ Read the .pyi file if present and scan for dependencies. """
 
-        # Complex stuff, pylint: disable=too-many-branches
+        # Complex stuff, pylint: disable=too-many-branches,too-many-statements
 
         if self.used_modules is None:
             pyi_filename = self.getPyIFilename()
@@ -829,10 +836,18 @@ class PythonShlibModule(PythonModuleBase):
                             assert parts[0] == "from"
                             assert parts[2] == "import"
 
-                            if parts[1] == "typing":
+                            origin_name = parts[1]
+
+                            if origin_name == "typing":
                                 continue
 
-                            pyi_deps.add(parts[1])
+                            if origin_name == ".":
+                                origin_name = self.getFullName()
+
+                            # TODO: Might want to add full relative import handling.
+
+                            if origin_name != self.getFullName():
+                                pyi_deps.add(origin_name)
 
                             imported = parts[3]
                             if imported.startswith("("):
@@ -840,7 +855,7 @@ class PythonShlibModule(PythonModuleBase):
                                 if not imported.endswith(")"):
                                     in_import = True
                                     imported = imported[1:]
-                                    in_import_part = parts[1]
+                                    in_import_part = origin_name
                                     assert in_import_part, (
                                         "Multiline part in file %s cannot be empty"
                                         % pyi_filename
@@ -856,7 +871,7 @@ class PythonShlibModule(PythonModuleBase):
                             for name in imported.split(","):
                                 if name:
                                     name = name.strip()
-                                    pyi_deps.add(parts[1] + "." + name)
+                                    pyi_deps.add(origin_name + "." + name)
 
                     else:  # In import
                         imported = line
@@ -879,6 +894,8 @@ class PythonShlibModule(PythonModuleBase):
     def getUsedModules(self):
         self._readPyPIFile()
 
+        assert "." not in self.used_modules, self
+
         return self.used_modules
 
     def getParentModule(self):
