@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -28,11 +28,8 @@ expression only stuff.
 from abc import abstractmethod
 
 from nuitka import Options, Tracing, TreeXML, Variables
-from nuitka.__past__ import (  # pylint: disable=I0021,redefined-builtin
-    intern,
-    iterItems,
-)
-from nuitka.Errors import NuitkaNodeError
+from nuitka.__past__ import iterItems
+from nuitka.Errors import NuitkaNodeDesignError, NuitkaNodeError
 from nuitka.PythonVersions import python_version
 from nuitka.SourceCodeReferences import SourceCodeReference
 from nuitka.utils.InstanceCounters import counted_del, counted_init
@@ -46,7 +43,7 @@ class NodeBase(NodeMetaClassBase):
     __slots__ = "parent", "source_ref"
 
     # Avoid the attribute unless it's really necessary.
-    if Options.isFullCompat():
+    if Options.is_fullcompat:
         __slots__ += ("effective_source_ref",)
 
     # String to identify the node class, to be consistent with its name.
@@ -79,10 +76,10 @@ class NodeBase(NodeMetaClassBase):
         graphical display.
 
         """
-        details = self.getDetails()
+        details = self.getDetailsForDisplay()
 
         if details:
-            return "'%s' with %s" % (self.kind, self.getDetails())
+            return "'%s' with %s" % (self.kind, details)
         else:
             return "'%s'" % self.kind
 
@@ -253,7 +250,7 @@ class NodeBase(NodeMetaClassBase):
         # this first.
         if (
             self.source_ref is not source_ref
-            and Options.isFullCompat()
+            and Options.is_fullcompat
             and self.source_ref != source_ref
         ):
             # An attribute outside of "__init__", so we save one memory for the
@@ -270,7 +267,7 @@ class NodeBase(NodeMetaClassBase):
         return getattr(self, "effective_source_ref", self.source_ref)
 
     def asXml(self):
-        line = self.getSourceReference().getLineNumber()
+        line = self.source_ref.getLineNumber()
 
         result = TreeXML.Element("node", kind=self.__class__.__name__, line="%s" % line)
 
@@ -348,31 +345,28 @@ class NodeBase(NodeMetaClassBase):
     def isExpressionOperationInplace():
         return False
 
-    def isExpressionSideEffects(self):
-        # Virtual method, pylint: disable=no-self-use
-
-        # We need to provide this, as these node kinds are only imported if
-        # necessary, but we test against them.
+    @staticmethod
+    def isExpressionSideEffects():
         return False
 
-    def isExpressionMakeSequence(self):
-        # Virtual method, pylint: disable=no-self-use
+    @staticmethod
+    def isExpressionMakeSequence():
         return False
 
-    def isNumberConstant(self):
-        # Virtual method, pylint: disable=no-self-use
+    @staticmethod
+    def isNumberConstant():
         return False
 
-    def isExpressionCall(self):
-        # Virtual method, pylint: disable=no-self-use
+    @staticmethod
+    def isExpressionCall():
         return False
 
-    def isExpressionFunctionBodyBase(self):
-        # Virtual method, pylint: disable=no-self-use
+    @staticmethod
+    def isExpressionFunctionBodyBase():
         return False
 
-    def isExpressionOutlineFunctionBase(self):
-        # Virtual method, pylint: disable=no-self-use
+    @staticmethod
+    def isExpressionOutlineFunctionBase():
         return False
 
     def visit(self, context, visitor):
@@ -381,26 +375,29 @@ class NodeBase(NodeMetaClassBase):
         for visitable in self.getVisitableNodes():
             visitable.visit(context, visitor)
 
-    def getVisitableNodes(self):
-        # Virtual method, pylint: disable=no-self-use
+    @staticmethod
+    def getVisitableNodes():
+
         return ()
 
-    def getVisitableNodesNamed(self):
+    @staticmethod
+    def getVisitableNodesNamed():
         """Named children dictionary.
 
         For use in debugging and XML output.
         """
 
-        # Virtual method, pylint: disable=no-self-use
         return ()
 
-    def getName(self):
-        # Virtual method, pylint: disable=no-self-use
+    @staticmethod
+    def getName():
+        """ Name of the node if any. """
+
         return None
 
-    def mayHaveSideEffects(self):
+    @staticmethod
+    def mayHaveSideEffects():
         """ Unless we are told otherwise, everything may have a side effect. """
-        # Virtual method, pylint: disable=no-self-use
 
         return True
 
@@ -412,21 +409,24 @@ class NodeBase(NodeMetaClassBase):
 
         return (self,)
 
-    def mayRaiseException(self, exception_type):
+    @staticmethod
+    def mayRaiseException(exception_type):
         """ Unless we are told otherwise, everything may raise everything. """
-        # Virtual method, pylint: disable=no-self-use,unused-argument
+        # Virtual method, pylint: disable=unused-argument
 
         return True
 
-    def mayReturn(self):
-        return "_RETURN" in self.kind
-
-    def mayBreak(self):
-        # For overload, pylint: disable=no-self-use
+    @staticmethod
+    def mayReturn():
+        """ May this node do a return exit, to be overloaded for things that might. """
         return False
 
-    def mayContinue(self):
-        # For overload, pylint: disable=no-self-use
+    @staticmethod
+    def mayBreak():
+        return False
+
+    @staticmethod
+    def mayContinue():
         return False
 
     def needsFrame(self):
@@ -434,20 +434,22 @@ class NodeBase(NodeMetaClassBase):
 
         return self.mayRaiseException(BaseException)
 
-    def willRaiseException(self, exception_type):
+    @staticmethod
+    def willRaiseException(exception_type):
         """ Unless we are told otherwise, nothing may raise anything. """
-        # Virtual method, pylint: disable=no-self-use,unused-argument
-
+        # Virtual method, pylint: disable=unused-argument
         return False
 
-    def isStatementAborting(self):
+    @staticmethod
+    def isStatementAborting():
         """ Is the node aborting, control flow doesn't continue after this node.  """
-        assert self.isStatement(), self.kind
-
         return False
 
 
 class CodeNodeMixin(object):
+    # Mixins are not allow to specify slots, pylint: disable=assigning-non-slot
+    __slots__ = ()
+
     def __init__(self, name, code_prefix):
         assert name is not None
 
@@ -480,7 +482,7 @@ class CodeNodeMixin(object):
             if str is not bytes:
                 name = name.encode("ascii", "c_identifier").decode()
 
-            self.code_name = "%s$$$%s%s" % (parent_name, self.code_prefix, name)
+            self.code_name = "%s$$$%s_%s" % (parent_name, self.code_prefix, name)
 
         return self.code_name
 
@@ -494,19 +496,29 @@ class CodeNodeMixin(object):
 
 
 class ChildrenHavingMixin(object):
+    # Mixins are not allow to specify slots.
+    __slots__ = ()
+
     named_children = ()
 
     checkers = {}
 
     def __init__(self, values):
-        assert type(self.named_children) is tuple and self.named_children
+        assert (
+            type(self.named_children) is tuple and self.named_children
+        ), self.named_children
 
         # TODO: Make this true.
         # assert len(self.named_children) > 1, self.kind
 
         # Check for completeness of given values, everything should be there
         # but of course, might be put to None.
-        assert set(values.keys()) == set(self.named_children)
+        if set(values.keys()) != set(self.named_children):
+            raise NuitkaNodeDesignError(
+                "Must pass named children in value dictionary",
+                set(values.keys()),
+                set(self.named_children),
+            )
 
         for name, value in values.items():
             if name in self.checkers:
@@ -556,26 +568,26 @@ class ChildrenHavingMixin(object):
 
         setattr(self, attr_name, value)
 
+    def clearChild(self, name):
+        # Only accept legal child names
+        assert name in self.named_children, name
+
+        if name in self.checkers:
+            self.checkers[name](None)
+
+        attr_name = "subnode_" + name
+
+        # Determine old value, and inform it about losing its parent.
+        old_value = getattr(self, attr_name)
+
+        assert old_value is not None
+
+        setattr(self, attr_name, None)
+
     def getChild(self, name):
         attr_name = "subnode_" + name
         return getattr(self, attr_name)
 
-    @staticmethod
-    def childGetter(name):
-        attr_name = intern("subnode_" + name)
-
-        def getter(self):
-            return getattr(self, attr_name)
-
-        return getter
-
-    @staticmethod
-    def childSetter(name):
-        def setter(self, value):
-            self.setChild(name, value)
-
-        return setter
-
     def getVisitableNodes(self):
         # TODO: Consider if a generator would be faster.
         result = []
@@ -676,7 +688,10 @@ class ChildrenHavingMixin(object):
 
 
 class ClosureGiverNodeMixin(CodeNodeMixin):
-    """ Blass class for nodes that provide variables for closure takers. """
+    """Base class for nodes that provide variables for closure takers. """
+
+    # Mixins are not allow to specify slots, pylint: disable=assigning-non-slot
+    __slots__ = ()
 
     def __init__(self, name, code_prefix):
         CodeNodeMixin.__init__(self, name=name, code_prefix=code_prefix)
@@ -758,7 +773,7 @@ class ClosureGiverNodeMixin(CodeNodeMixin):
         del self.temp_variables[variable.getName()]
 
     def allocatePreserverId(self):
-        if python_version >= 300:
+        if python_version >= 0x300:
             self.preserver_id += 1
 
         return self.preserver_id
@@ -767,6 +782,9 @@ class ClosureGiverNodeMixin(CodeNodeMixin):
 class ClosureTakerMixin(object):
     """ Mixin for nodes that accept variables from closure givers. """
 
+    # Mixins are not allow to specify slots, pylint: disable=assigning-non-slot
+    __slots__ = ()
+
     def __init__(self, provider):
         self.provider = provider
 
@@ -828,8 +846,8 @@ class StatementBase(NodeBase):
 
     # TODO: Have them all.
     # @abstractmethod
-    def getStatementNiceName(self):
-        # Virtual method, pylint: disable=no-self-use
+    @staticmethod
+    def getStatementNiceName():
         return "undescribed statement"
 
     def computeStatementSubExpressions(self, trace_collection):
@@ -936,22 +954,6 @@ class StatementChildHavingBase(StatementBase):
         attr_name = "subnode_" + name
         return getattr(self, attr_name)
 
-    @staticmethod
-    def childGetter(name):
-        attr_name = "subnode_" + name
-
-        def getter(self):
-            return getattr(self, attr_name)
-
-        return getter
-
-    @staticmethod
-    def childSetter(name):
-        def setter(self, value):
-            self.setChild(name, value)
-
-        return setter
-
     def getVisitableNodes(self):
         # TODO: Consider if a generator would be faster.
         attr_name = "subnode_" + self.named_child
@@ -1046,6 +1048,9 @@ class StatementChildHavingBase(StatementBase):
 
 
 class SideEffectsFromChildrenMixin(object):
+    # Mixins are not allow to specify slots.
+    __slots__ = ()
+
     def mayHaveSideEffects(self):
         for child in self.getVisitableNodes():
             if child.mayHaveSideEffects():
