@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,10 +25,7 @@ of checks, and add methods automatically.
 from abc import ABCMeta
 
 from nuitka.__past__ import intern  # pylint: disable=I0021,redefined-builtin
-
-
-class NuitkaNodeDesignError(Exception):
-    pass
+from nuitka.Errors import NuitkaNodeDesignError
 
 
 def _checkBases(name, bases):
@@ -43,7 +40,7 @@ def _checkBases(name, bases):
 
         if is_mixin and last_mixin is False:
             raise NuitkaNodeDesignError(
-                "Mixins must come first in base classes.", bases
+                name, "Mixins must come first in base classes.", bases
             )
 
         last_mixin = is_mixin
@@ -65,12 +62,18 @@ class NodeCheckMetaClass(ABCMeta):
         if "named_child" in dictionary:
             dictionary["__slots__"] += (intern("subnode_" + dictionary["named_child"]),)
 
-        if "named_children" in dictionary and not name.endswith("Base"):
+        if "named_children" in dictionary:
             if len(dictionary["named_children"]) <= 1:
                 raise NuitkaNodeDesignError(
-                    "Use ExpressionChildHaving for one child node classes", name
+                    name, "Use ExpressionChildHaving for one child node classes"
                 )
 
+            assert type(dictionary["named_children"]) is tuple
+            dictionary["__slots__"] += tuple(
+                intern("subnode_" + named_child)
+                for named_child in dictionary["named_children"]
+            )
+
         # Not a method:
         if "checker" in dictionary:
             dictionary["checker"] = staticmethod(dictionary["checker"])
@@ -81,7 +84,9 @@ class NodeCheckMetaClass(ABCMeta):
     def __init__(cls, name, bases, dictionary):  # @NoSelf
 
         if not name.endswith("Base"):
-            assert "kind" in dictionary, name
+            if "kind" not in dictionary:
+                raise NuitkaNodeDesignError(name, "Must provide class variable 'kind'")
+
             kind = dictionary["kind"]
 
             assert type(kind) is str, name
