@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,6 +22,7 @@ when it returns, break, continues, or raises an exception. See Developer
 Manual for how this maps to try/finally and try/except as in Python.
 """
 
+from nuitka.Errors import NuitkaOptimizationError
 from nuitka.optimizations.TraceCollections import TraceCollectionBranch
 
 from .Checkers import checkStatementsSequence, checkStatementsSequenceOrNone
@@ -37,96 +38,64 @@ class StatementTry(StatementChildrenHavingBase):
         "except_handler",
         "break_handler",
         "continue_handler",
-        "return_handler"
+        "return_handler",
     )
 
     checkers = {
-        "tried"            : checkStatementsSequence,
-        "except_handler"   : checkStatementsSequenceOrNone,
-        "break_handler"    : checkStatementsSequenceOrNone,
-        "continue_handler" : checkStatementsSequenceOrNone,
-        "return_handler"   : checkStatementsSequenceOrNone
+        "tried": checkStatementsSequence,
+        "except_handler": checkStatementsSequenceOrNone,
+        "break_handler": checkStatementsSequenceOrNone,
+        "continue_handler": checkStatementsSequenceOrNone,
+        "return_handler": checkStatementsSequenceOrNone,
     }
 
-    def __init__(self, tried, except_handler, break_handler, continue_handler,
-                 return_handler, source_ref):
+    def __init__(
+        self,
+        tried,
+        except_handler,
+        break_handler,
+        continue_handler,
+        return_handler,
+        source_ref,
+    ):
         StatementChildrenHavingBase.__init__(
             self,
-            values     = {
-                "tried"            : tried,
-                "except_handler"   : except_handler,
-                "break_handler"    : break_handler,
-                "continue_handler" : continue_handler,
-                "return_handler"   : return_handler
+            values={
+                "tried": tried,
+                "except_handler": except_handler,
+                "break_handler": break_handler,
+                "continue_handler": continue_handler,
+                "return_handler": return_handler,
             },
-            source_ref = source_ref
+            source_ref=source_ref,
         )
 
-
-    getBlockTry = StatementChildrenHavingBase.childGetter(
-        "tried"
-    )
-    setBlockTry = StatementChildrenHavingBase.childSetter(
-        "tried"
-    )
-
-    getBlockExceptHandler = StatementChildrenHavingBase.childGetter(
-        "except_handler"
-    )
-    setBlockExceptHandler = StatementChildrenHavingBase.childSetter(
-        "except_handler"
-    )
-
-    getBlockBreakHandler = StatementChildrenHavingBase.childGetter(
-        "break_handler"
-    )
-    setBlockBreakHandler = StatementChildrenHavingBase.childSetter(
-        "break_handler"
-    )
-
-    getBlockContinueHandler = StatementChildrenHavingBase.childGetter(
-        "continue_handler"
-    )
-    setBlockContinueHandler = StatementChildrenHavingBase.childSetter(
-        "continue_handler"
-    )
-
-    getBlockReturnHandler = StatementChildrenHavingBase.childGetter(
-        "return_handler"
-    )
-    setBlockReturnHandler = StatementChildrenHavingBase.childSetter(
-        "return_handler"
-    )
-
     def computeStatement(self, trace_collection):
-        # This node has many children to handle, pylint: disable=too-many-branches,too-many-locals
-        tried = self.getBlockTry()
+        # This node has many children to handle, pylint: disable=I0021,too-many-branches,too-many-locals,too-many-statements
+        tried = self.subnode_tried
 
-        except_handler = self.getBlockExceptHandler()
-        break_handler = self.getBlockBreakHandler()
-        continue_handler = self.getBlockContinueHandler()
-        return_handler = self.getBlockReturnHandler()
+        except_handler = self.subnode_except_handler
+        break_handler = self.subnode_break_handler
+        continue_handler = self.subnode_continue_handler
+        return_handler = self.subnode_return_handler
 
         # The tried block must be considered as a branch, if it is not empty
         # already.
         collection_start = TraceCollectionBranch(
-            parent = trace_collection,
-            name   = "try start"
+            parent=trace_collection, name="try start"
         )
 
         abort_context = trace_collection.makeAbortStackContext(
-            catch_breaks     = break_handler is not None,
-            catch_continues  = continue_handler is not None,
-            catch_returns    = return_handler is not None,
-            catch_exceptions = True,
+            catch_breaks=break_handler is not None,
+            catch_continues=continue_handler is not None,
+            catch_returns=return_handler is not None,
+            catch_exceptions=True,
         )
 
         with abort_context:
             # As a branch point for the many types of handlers.
 
-            result = tried.computeStatementsSequence(
-                trace_collection = trace_collection
-            )
+            result = tried.computeStatementsSequence(trace_collection=trace_collection)
 
             # We might be done entirely already.
             if result is None:
@@ -134,7 +103,7 @@ class StatementTry(StatementChildrenHavingBase):
 
             # Might be changed.
             if result is not tried:
-                self.setBlockTry(result)
+                self.setChild("tried", result)
                 tried = result
 
             break_collections = trace_collection.getLoopBreakCollections()
@@ -144,131 +113,158 @@ class StatementTry(StatementChildrenHavingBase):
 
         tried_may_raise = tried.mayRaiseException(BaseException)
         # Exception handling is useless if no exception is to be raised.
-        # TODO: signal the change.
         if not tried_may_raise:
             if except_handler is not None:
-                self.setBlockExceptHandler(None)
+                except_handler.finalize()
+
+                self.clearChild("except_handler")
+                trace_collection.signalChange(
+                    tags="new_statements",
+                    message="Removed useless exception handler.",
+                    source_ref=except_handler.source_ref,
+                )
+
                 except_handler = None
 
         # If tried may raise, even empty exception handler has a meaning to
         # ignore that exception.
         if tried_may_raise:
             collection_exception_handling = TraceCollectionBranch(
-                parent = collection_start,
-                name   = "except handler"
+                parent=collection_start, name="except handler"
             )
 
+            # When no exception exits are there, this is a problem, we just
+            # found an inconsistency that is a bug.
             if not exception_collections:
-                for statement in tried.getStatements():
+                for statement in tried.subnode_statements:
                     if statement.mayRaiseException(BaseException):
-                        assert False, statement.asXmlText()
+                        raise NuitkaOptimizationError(
+                            "This statement does raise but didn't annotate an exception exit.",
+                            statement,
+                        )
 
-                assert False
+                raise NuitkaOptimizationError(
+                    "Falsely assuming tried block may raise, but no statement says so.",
+                    tried,
+                )
 
             collection_exception_handling.mergeMultipleBranches(exception_collections)
 
             if except_handler is not None:
                 result = except_handler.computeStatementsSequence(
-                    trace_collection = collection_exception_handling
+                    trace_collection=collection_exception_handling
                 )
 
                 # Might be changed.
                 if result is not except_handler:
-                    self.setBlockExceptHandler(result)
+                    self.setChild("except_handler", result)
                     except_handler = result
 
         if break_handler is not None:
             if not tried.mayBreak():
-                self.setBlockBreakHandler(None)
+                break_handler.finalize()
+
+                self.clearChild("break_handler")
                 break_handler = None
 
         if break_handler is not None:
             collection_break = TraceCollectionBranch(
-                parent = collection_start,
-                name   = "break handler"
+                parent=collection_start, name="break handler"
             )
 
             collection_break.mergeMultipleBranches(break_collections)
 
             result = break_handler.computeStatementsSequence(
-                trace_collection = collection_break
+                trace_collection=collection_break
             )
 
             # Might be changed.
             if result is not break_handler:
-                self.setBlockBreakHandler(result)
+                self.setChild("break_handler", result)
                 break_handler = result
 
         if continue_handler is not None:
             if not tried.mayContinue():
-                self.setBlockContinueHandler(None)
+                continue_handler.finalize()
+
+                self.clearChild("continue_handler")
                 continue_handler = None
 
         if continue_handler is not None:
             collection_continue = TraceCollectionBranch(
-                parent = collection_start,
-                name   = "continue handler"
+                parent=collection_start, name="continue handler"
             )
 
             collection_continue.mergeMultipleBranches(continue_collections)
 
             result = continue_handler.computeStatementsSequence(
-                trace_collection = collection_continue
+                trace_collection=collection_continue
             )
 
             # Might be changed.
             if result is not continue_handler:
-                self.setBlockContinueHandler(result)
+                self.setChild("continue_handler", result)
                 continue_handler = result
 
         if return_handler is not None:
             if not tried.mayReturn():
-                self.setBlockReturnHandler(None)
+                return_handler.finalize()
+
+                self.clearChild("return_handler")
                 return_handler = None
 
         if return_handler is not None:
             collection_return = TraceCollectionBranch(
-                parent = collection_start,
-                name   = "return handler"
+                parent=collection_start, name="return handler"
             )
 
             collection_return.mergeMultipleBranches(return_collections)
 
             result = return_handler.computeStatementsSequence(
-                trace_collection = collection_return
+                trace_collection=collection_return
             )
 
             # Might be changed.
             if result is not return_handler:
-                self.setBlockReturnHandler(result)
+                self.setChild("return_handler", result)
                 return_handler = result
 
+        # Check for trivial return handlers that immediately return, they can
+        # just be removed.
         if return_handler is not None:
-            if return_handler.getStatements()[0].isStatementReturn() and \
-               return_handler.getStatements()[0].getExpression().isExpressionReturnedValueRef():
-                self.setBlockReturnHandler(None)
-                return_handler = None
+            if return_handler.subnode_statements[0].isStatementReturnReturnedValue():
+                return_handler.finalize()
 
+                self.clearChild("return_handler")
+                return_handler = None
 
         # Merge exception handler only if it is used. Empty means it is not
         # aborting, as it swallows the exception.
         if tried_may_raise and (
-               except_handler is None or \
-               not except_handler.isStatementAborting()
-            ):
+            except_handler is None or not except_handler.isStatementAborting()
+        ):
             trace_collection.mergeBranches(
-                collection_yes = collection_exception_handling,
-                collection_no  = None
+                collection_yes=collection_exception_handling, collection_no=None
             )
 
         # An empty exception handler means we have to swallow exception.
-        if not tried_may_raise and \
-           break_handler is None and \
-           continue_handler is None and \
-           return_handler is None:
+        if (
+            (
+                not tried_may_raise
+                or (
+                    except_handler is not None
+                    and except_handler.subnode_statements[
+                        0
+                    ].isStatementReraiseException()
+                )
+            )
+            and break_handler is None
+            and continue_handler is None
+            and return_handler is None
+        ):
             return tried, "new_statements", "Removed useless try, all handlers removed."
 
-        tried_statements = tried.getStatements()
+        tried_statements = tried.subnode_statements
 
         pre_statements = []
 
@@ -278,16 +274,13 @@ class StatementTry(StatementChildrenHavingBase):
             if tried_statement.mayRaiseException(BaseException):
                 break
 
-            if break_handler is not None and \
-               tried_statement.mayBreak():
+            if break_handler is not None and tried_statement.mayBreak():
                 break
 
-            if continue_handler is not None and \
-               tried_statement.mayContinue():
+            if continue_handler is not None and tried_statement.mayContinue():
                 break
 
-            if return_handler is not None and \
-               tried_statement.mayReturn():
+            if return_handler is not None and tried_statement.mayReturn():
                 break
 
             pre_statements.append(tried_statement)
@@ -304,16 +297,13 @@ class StatementTry(StatementChildrenHavingBase):
                 if tried_statement.mayRaiseException(BaseException):
                     break
 
-                if break_handler is not None and \
-                   tried_statement.mayBreak():
+                if break_handler is not None and tried_statement.mayBreak():
                     break
 
-                if continue_handler is not None and \
-                   tried_statement.mayContinue():
+                if continue_handler is not None and tried_statement.mayContinue():
                     break
 
-                if return_handler is not None and \
-                   tried_statement.mayReturn():
+                if return_handler is not None and tried_statement.mayReturn():
                     break
 
                 post_statements.insert(0, tried_statement)
@@ -322,13 +312,13 @@ class StatementTry(StatementChildrenHavingBase):
                 del tried_statements[-1]
 
         if pre_statements or post_statements:
-            assert tried_statements # Should be dealt with already
+            assert tried_statements  # Should be dealt with already
 
-            tried.setStatements(tried_statements)
+            tried.setChild("statements", tried_statements)
 
             result = StatementsSequence(
-                statements = pre_statements + [self] + post_statements,
-                source_ref = self.getSourceReference()
+                statements=pre_statements + [self] + post_statements,
+                source_ref=self.source_ref,
             )
 
             def explain():
@@ -337,55 +327,48 @@ class StatementTry(StatementChildrenHavingBase):
 
                 if pre_statements:
                     result += " Leading statements at %s." % (
-                        ','.join(
-                            x.getSourceReference().getAsString() + '/' + str(x)
-                            for x in
-                            pre_statements
+                        ",".join(
+                            x.getSourceReference().getAsString() + "/" + str(x)
+                            for x in pre_statements
                         )
                     )
 
                 if post_statements:
                     result += " Trailing statements at %s." % (
-                        ','.join(
-                            x.getSourceReference().getAsString() + '/' + str(x)
-                            for x in
-                            post_statements
+                        ",".join(
+                            x.getSourceReference().getAsString() + "/" + str(x)
+                            for x in post_statements
                         )
                     )
 
                 return result
 
-
-            return (
-                result,
-                "new_statements",
-                explain
-            )
+            return (result, "new_statements", explain)
 
         return self, None, None
 
     def mayReturn(self):
         # TODO: If we optimized return handler away, this would be not needed
         # or even non-optimal.
-        if self.getBlockTry().mayReturn():
+        if self.subnode_tried.mayReturn():
             return True
 
-        except_handler = self.getBlockExceptHandler()
+        except_handler = self.subnode_except_handler
 
         if except_handler is not None and except_handler.mayReturn():
             return True
 
-        break_handler = self.getBlockBreakHandler()
+        break_handler = self.subnode_break_handler
 
         if break_handler is not None and break_handler.mayReturn():
             return True
 
-        continue_handler = self.getBlockContinueHandler()
+        continue_handler = self.subnode_continue_handler
 
         if continue_handler is not None and continue_handler.mayReturn():
             return True
 
-        return_handler = self.getBlockReturnHandler()
+        return_handler = self.subnode_return_handler
 
         if return_handler is not None and return_handler.mayReturn():
             return True
@@ -395,25 +378,25 @@ class StatementTry(StatementChildrenHavingBase):
     def mayBreak(self):
         # TODO: If we optimized return handler away, this would be not needed
         # or even non-optimal.
-        if self.getBlockTry().mayBreak():
+        if self.subnode_tried.mayBreak():
             return True
 
-        except_handler = self.getBlockExceptHandler()
+        except_handler = self.subnode_except_handler
 
         if except_handler is not None and except_handler.mayBreak():
             return True
 
-        break_handler = self.getBlockBreakHandler()
+        break_handler = self.subnode_break_handler
 
         if break_handler is not None and break_handler.mayBreak():
             return True
 
-        continue_handler = self.getBlockContinueHandler()
+        continue_handler = self.subnode_continue_handler
 
         if continue_handler is not None and continue_handler.mayBreak():
             return True
 
-        return_handler = self.getBlockReturnHandler()
+        return_handler = self.subnode_return_handler
 
         if return_handler is not None and return_handler.mayBreak():
             return True
@@ -423,25 +406,25 @@ class StatementTry(StatementChildrenHavingBase):
     def mayContinue(self):
         # TODO: If we optimized return handler away, this would be not needed
         # or even non-optimal.
-        if self.getBlockTry().mayContinue():
+        if self.subnode_tried.mayContinue():
             return True
 
-        except_handler = self.getBlockExceptHandler()
+        except_handler = self.subnode_except_handler
 
         if except_handler is not None and except_handler.mayContinue():
             return True
 
-        break_handler = self.getBlockBreakHandler()
+        break_handler = self.subnode_break_handler
 
         if break_handler is not None and break_handler.mayContinue():
             return True
 
-        continue_handler = self.getBlockContinueHandler()
+        continue_handler = self.subnode_continue_handler
 
         if continue_handler is not None and continue_handler.mayContinue():
             return True
 
-        return_handler = self.getBlockReturnHandler()
+        return_handler = self.subnode_return_handler
 
         if return_handler is not None and return_handler.mayContinue():
             return True
@@ -449,77 +432,85 @@ class StatementTry(StatementChildrenHavingBase):
         return False
 
     def isStatementAborting(self):
-        except_handler = self.getBlockExceptHandler()
+        except_handler = self.subnode_except_handler
 
         if except_handler is None or not except_handler.isStatementAborting():
             return False
 
-        break_handler = self.getBlockBreakHandler()
+        break_handler = self.subnode_break_handler
 
         if break_handler is not None and not break_handler.isStatementAborting():
             return False
 
-        continue_handler = self.getBlockContinueHandler()
+        continue_handler = self.subnode_continue_handler
 
         if continue_handler is not None and not continue_handler.isStatementAborting():
             return False
 
-        return_handler = self.getBlockReturnHandler()
+        return_handler = self.subnode_return_handler
 
         if return_handler is not None and not return_handler.isStatementAborting():
             return False
 
-        return self.getBlockTry().isStatementAborting()
+        return self.subnode_tried.isStatementAborting()
 
     def mayRaiseException(self, exception_type):
-        tried = self.getBlockTry()
+        tried = self.subnode_tried
 
         if tried.mayRaiseException(exception_type):
-            except_handler = self.getBlockExceptHandler()
+            except_handler = self.subnode_except_handler
 
-            if except_handler is not None and \
-               except_handler.mayRaiseException(exception_type):
+            if except_handler is not None and except_handler.mayRaiseException(
+                exception_type
+            ):
                 return True
 
-        break_handler = self.getBlockBreakHandler()
+        break_handler = self.subnode_break_handler
 
-        if break_handler is not None and \
-           break_handler.mayRaiseException(exception_type):
+        if break_handler is not None and break_handler.mayRaiseException(
+            exception_type
+        ):
             return True
 
-        continue_handler = self.getBlockContinueHandler()
+        continue_handler = self.subnode_continue_handler
 
-        if continue_handler is not None and \
-           continue_handler.mayRaiseException(exception_type):
+        if continue_handler is not None and continue_handler.mayRaiseException(
+            exception_type
+        ):
             return True
 
-        return_handler = self.getBlockReturnHandler()
+        return_handler = self.subnode_return_handler
 
-        if return_handler is not None and \
-           return_handler.mayRaiseException(exception_type):
+        if return_handler is not None and return_handler.mayRaiseException(
+            exception_type
+        ):
             return True
 
         return False
 
     def needsFrame(self):
-        except_handler = self.getBlockExceptHandler()
+        except_handler = self.subnode_except_handler
 
         if except_handler is not None and except_handler.needsFrame():
             return True
 
-        break_handler = self.getBlockBreakHandler()
+        break_handler = self.subnode_break_handler
 
         if break_handler is not None and break_handler.needsFrame():
             return True
 
-        continue_handler = self.getBlockContinueHandler()
+        continue_handler = self.subnode_continue_handler
 
         if continue_handler is not None and continue_handler.needsFrame():
             return True
 
-        return_handler = self.getBlockReturnHandler()
+        return_handler = self.subnode_return_handler
 
         if return_handler is not None and return_handler.needsFrame():
             return True
 
-        return self.getBlockTry().needsFrame()
+        return self.subnode_tried.needsFrame()
+
+    @staticmethod
+    def getStatementNiceName():
+        return "tried block statement"
