@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,14 +24,26 @@ make others possible.
 
 
 import inspect
-from logging import debug, info
+import os
 
 from nuitka import ModuleRegistry, Options, Variables
 from nuitka.importing import ImportCache
 from nuitka.nodes.LocalsScopes import LocalsDictHandle, getLocalsDictHandles
 from nuitka.plugins.Plugins import Plugins
-from nuitka.Tracing import printLine
-from nuitka.utils import MemoryUsage
+from nuitka.Tracing import (
+    general,
+    memory_logger,
+    optimization_logger,
+    progress_logger,
+    recursion_logger,
+    reportProgressBar,
+)
+from nuitka.utils.AppDirs import getCacheDir
+from nuitka.utils.FileOperations import makePath
+from nuitka.utils.MemoryUsage import (
+    MemoryWatch,
+    getHumanReadableProcessMemoryUsage,
+)
 
 from . import Graphs, TraceCollections
 from .BytecodeDemotion import demoteCompiledModuleToBytecode
@@ -40,63 +52,66 @@ from .Tags import TagSet
 _progress = Options.isShowProgress()
 _is_verbose = Options.isVerbose()
 
+
 def _attemptRecursion(module):
     new_modules = module.attemptRecursion()
 
-    for new_module in new_modules:
-        debug(
-            "{source_ref} : {tags} : {message}".format(
-                source_ref = new_module.getSourceReference().getAsString(),
-                tags       = "new_code",
-                message    = "Recursed to module package."
+    if Options.isShowInclusion():
+        for new_module in new_modules:
+            recursion_logger.info(
+                "{source_ref} : {tags} : {message}".format(
+                    source_ref=new_module.getSourceReference().getAsString(),
+                    tags="new_code",
+                    message="Recursed to module package.",
+                )
             )
-        )
 
 
 tag_set = None
 
-def signalChange(tags, source_ref, message):
-    """ Indicate a change to the optimization framework.
 
-    """
+def signalChange(tags, source_ref, message):
+    """Indicate a change to the optimization framework."""
     if message is not None:
         # Try hard to not call a delayed evaluation of node descriptions.
 
         if _is_verbose:
-            debug(
+            optimization_logger.info(
                 "{source_ref} : {tags} : {message}".format(
-                    source_ref = source_ref.getAsString(),
-                    tags       = tags,
-                    message    = message() if inspect.isfunction(message) else message
+                    source_ref=source_ref.getAsString(),
+                    tags=tags,
+                    message=message() if inspect.isfunction(message) else message,
                 )
             )
 
     tag_set.onSignal(tags)
 
+
 # Use this globally from there, without cyclic dependency.
 TraceCollections.signalChange = signalChange
 
 
 def optimizeCompiledPythonModule(module):
     if _progress:
-        info(
+        progress_logger.info(
             "Doing module local optimizations for '{module_name}'.".format(
-                module_name = module.getFullName()
+                module_name=module.getFullName()
             )
         )
 
     touched = False
 
     if _progress and Options.isShowMemory():
-        memory_watch = MemoryUsage.MemoryWatch()
+        memory_watch = MemoryWatch()
 
     while True:
         tag_set.clear()
 
         try:
+            # print("Compute module")
             module.computeModule()
         except BaseException:
-            info("Interrupted while working on '%s'." % module)
+            general.info("Interrupted while working on '%s'." % module)
             raise
 
         Graphs.onModuleOptimizationStep(module)
@@ -108,64 +123,64 @@ def optimizeCompiledPythonModule(module):
 
             break
         else:
+            if _progress:
+                progress_logger.info("Finished with the module.")
             break
 
+        if _progress:
+            if "new_code" in tag_set:
+                tag_set.remove("new_code")
+
+            progress_logger.info(
+                "Not finished with the module due to following change kinds: %s"
+                % ",".join(sorted(tag_set))
+            )
+
         # Otherwise we did stuff, so note that for return value.
         touched = True
 
     if _progress and Options.isShowMemory():
         memory_watch.finish()
 
-        info(
-            "Memory usage changed during optimization of '%s': %s" % (
-                module.getFullName(),
-                memory_watch.asStr()
-            )
+        memory_logger.info(
+            "Memory usage changed during optimization of '%s': %s"
+            % (module.getFullName(), memory_watch.asStr())
         )
 
-    Plugins.considerImplicitImports(
-        module        = module,
-        signal_change = signalChange
-    )
+    Plugins.considerImplicitImports(module=module, signal_change=signalChange)
 
     return touched
 
 
 def optimizeUncompiledPythonModule(module):
+    full_name = module.getFullName()
     if _progress:
-        info(
+        progress_logger.info(
             "Doing module dependency considerations for '{module_name}':".format(
-                module_name = module.getFullName()
+                module_name=full_name
             )
         )
 
     for used_module_name, used_module_path in module.getUsedModules():
         used_module = ImportCache.getImportedModuleByNameAndPath(
-            used_module_name,
-            used_module_path
+            used_module_name, used_module_path
         )
         ModuleRegistry.addUsedModule(used_module)
 
-    package_name = module.getPackage()
+    package_name = full_name.getPackageName()
 
     if package_name is not None:
         used_module = ImportCache.getImportedModuleByName(package_name)
         ModuleRegistry.addUsedModule(used_module)
 
-    Plugins.considerImplicitImports(
-        module        = module,
-        signal_change = signalChange
-    )
+    Plugins.considerImplicitImports(module=module, signal_change=signalChange)
 
 
 def optimizeShlibModule(module):
     # Pick up parent package if any.
     _attemptRecursion(module)
 
-    Plugins.considerImplicitImports(
-        module        = module,
-        signal_change = signalChange
-    )
+    Plugins.considerImplicitImports(module=module, signal_change=signalChange)
 
 
 def optimizeModule(module):
@@ -181,48 +196,70 @@ def optimizeModule(module):
     return changed
 
 
-def areEmptyTraces(variable_traces):
-    empty = True
+def areReadOnlyTraces(variable_traces):
+    """Do these traces contain any writes."""
 
+    # Many cases immediately return, that is how we do it here,
     for variable_trace in variable_traces:
         if variable_trace.isAssignTrace():
-            empty = False
-            break
+            return False
         elif variable_trace.isInitTrace():
-            empty = False
-            break
+            pass
+        elif variable_trace.isDeletedTrace():
+            # A "del" statement can do this, and needs to prevent variable
+            # from being not released.
+
+            return False
         elif variable_trace.isUninitTrace():
-            if variable_trace.getPrevious():
-                # A "del" statement can do this, and needs to prevent variable
-                # from being removed.
-
-                empty = False
-                break
-            elif variable_trace.hasDefiniteUsages():
-                # Checking definite is enough, the merges, we shall see
-                # them as well.
-                empty = False
-                break
+            pass
         elif variable_trace.isUnknownTrace():
-            if variable_trace.hasDefiniteUsages():
-                # Checking definite is enough, the merges, we shall see
-                # them as well.
-                empty = False
-                break
+            return False
         elif variable_trace.isMergeTrace():
-            if variable_trace.hasDefiniteUsages():
-                # Checking definite is enough, the merges, we shall see
-                # them as well.
-                empty = False
-                break
-
+            pass
         elif variable_trace.isLoopTrace():
-            empty = False
-            break
+            pass
         else:
             assert False, variable_trace
 
-    return empty
+    return True
+
+
+def areEmptyTraces(variable_traces):
+    """Do these traces contain any writes or accesses."""
+    # Many cases immediately return, that is how we do it here,
+    # pylint: disable=too-many-return-statements
+
+    for variable_trace in variable_traces:
+        if variable_trace.isAssignTrace():
+            return False
+        elif variable_trace.isInitTrace():
+            return False
+        elif variable_trace.isDeletedTrace():
+            # A "del" statement can do this, and needs to prevent variable
+            # from being removed.
+
+            return False
+        elif variable_trace.isUninitTrace():
+            if variable_trace.getUsageCount():
+                # Checking definite is enough, the merges, we shall see
+                # them as well.
+                return False
+        elif variable_trace.isUnknownTrace():
+            if variable_trace.getUsageCount():
+                # Checking definite is enough, the merges, we shall see
+                # them as well.
+                return False
+        elif variable_trace.isMergeTrace():
+            if variable_trace.getUsageCount():
+                # Checking definite is enough, the merges, we shall see
+                # them as well.
+                return False
+        elif variable_trace.isLoopTrace():
+            return False
+        else:
+            assert False, variable_trace
+
+    return True
 
 
 def optimizeUnusedClosureVariables(function_body):
@@ -232,12 +269,14 @@ def optimizeUnusedClosureVariables(function_body):
         # print "VAR", closure_variable
 
         # Need to take closure of those.
-        if closure_variable.isParameterVariable() and \
-           function_body.isExpressionGeneratorObjectBody():
+        if (
+            closure_variable.isParameterVariable()
+            and function_body.isExpressionGeneratorObjectBody()
+        ):
             continue
 
         variable_traces = function_body.trace_collection.getVariableTraces(
-            variable = closure_variable
+            variable=closure_variable
         )
 
         empty = areEmptyTraces(variable_traces)
@@ -247,7 +286,8 @@ def optimizeUnusedClosureVariables(function_body):
             signalChange(
                 "var_usage",
                 function_body.getSourceReference(),
-                message = "Remove unused closure variable '%s'." % closure_variable.getName()
+                message="Remove unused closure variable '%s'."
+                % closure_variable.getName(),
             )
 
             function_body.removeClosureVariable(closure_variable)
@@ -255,6 +295,31 @@ def optimizeUnusedClosureVariables(function_body):
     return changed
 
 
+def optimizeVariableReleases(function_body):
+    changed = False
+
+    for parameter_variable in function_body.getParameterVariablesWithManualRelease():
+
+        variable_traces = function_body.trace_collection.getVariableTraces(
+            variable=parameter_variable
+        )
+
+        read_only = areReadOnlyTraces(variable_traces)
+        if read_only:
+            changed = True
+
+            signalChange(
+                "var_usage",
+                function_body.getSourceReference(),
+                message="Schedule removal releases of unassigned parameter variable '%s'."
+                % parameter_variable.getName(),
+            )
+
+            function_body.removeVariableReleases(parameter_variable)
+
+    return changed
+
+
 def optimizeLocalsDictsHandles():
     # Lots of cases, pylint: disable=too-many-branches
 
@@ -284,16 +349,19 @@ def optimizeLocalsDictsHandles():
                     # then we can push it down the line. TODO: Once temporary
                     # variables and dictionary building allows for unset values
                     # remove this
-                    if variable_trace.getAssignNode().getAssignSource().mayHaveSideEffects():
-                        propagate = False
-                        break
-                elif variable_trace.isUninitTrace():
-                    if variable_trace.previous is not None:
+                    if (
+                        variable_trace.getAssignNode().subnode_source.mayHaveSideEffects()
+                    ):
                         propagate = False
                         break
+                elif variable_trace.isDeletedTrace():
+                    propagate = False
+                    break
                 elif variable_trace.isMergeTrace():
                     propagate = False
                     break
+                elif variable_trace.isUninitTrace():
+                    pass
                 elif variable_trace.isUnknownTrace():
                     propagate = False
                     break
@@ -309,22 +377,45 @@ def optimizeLocalsDictsHandles():
 def optimizeUnusedUserVariables(function_body):
     changed = False
 
-    for local_variable in function_body.getUserLocalVariables() + function_body.getOutlineLocalVariables():
+    for local_variable in function_body.getUserLocalVariables():
         variable_traces = function_body.trace_collection.getVariableTraces(
-            variable = local_variable
+            variable=local_variable
         )
 
         empty = areEmptyTraces(variable_traces)
         if empty:
+            function_body.removeUserVariable(local_variable)
+
             signalChange(
                 "var_usage",
                 function_body.getSourceReference(),
-                message = "Remove unused local variable '%s'." % local_variable.getName()
+                message="Remove unused local variable '%s'." % local_variable.getName(),
             )
 
-            function_body.removeUserVariable(local_variable)
             changed = True
 
+    outlines = function_body.trace_collection.getOutlineFunctions()
+
+    if outlines is not None:
+        for outline in outlines:
+            for local_variable in outline.getUserLocalVariables():
+                variable_traces = function_body.trace_collection.getVariableTraces(
+                    variable=local_variable
+                )
+
+                empty = areEmptyTraces(variable_traces)
+                if empty:
+                    outline.removeUserVariable(local_variable)
+
+                    signalChange(
+                        "var_usage",
+                        outline.getSourceReference(),
+                        message="Remove unused local variable '%s'."
+                        % local_variable.getName(),
+                    )
+
+                    changed = True
+
     return changed
 
 
@@ -333,7 +424,7 @@ def optimizeUnusedTempVariables(provider):
 
     for temp_variable in provider.getTempVariables():
         variable_traces = provider.trace_collection.getVariableTraces(
-            variable = temp_variable
+            variable=temp_variable
         )
 
         empty = areEmptyTraces(variable_traces)
@@ -373,11 +464,14 @@ def optimizeVariables(module):
                     if optimizeUnusedClosureVariables(function_body):
                         changed = True
 
+                    if optimizeVariableReleases(function_body):
+                        changed = True
+
                 if optimizeUnusedTempVariables(function_body):
                     changed = True
         except Exception:
             print("Problem with", function_body)
-            raise#
+            raise  #
 
         if optimizeUnusedUserVariables(module):
             changed = True
@@ -385,10 +479,10 @@ def optimizeVariables(module):
         if optimizeUnusedTempVariables(module):
             changed = True
 
-#        TODO: Global optimizations could go here maybe, so far we can do all
-#        the things in assign nodes themselves based on last trace.
-#        if optimizeUnusedAssignments(module):
-#            changed = True
+    #        TODO: Global optimizations could go here maybe, so far we can do all
+    #        the things in assign nodes themselves based on last trace.
+    #        if optimizeUnusedAssignments(module):
+    #            changed = True
     except Exception:
         print("Problem with", module)
         raise
@@ -397,58 +491,58 @@ def optimizeVariables(module):
 
 
 def _traceProgress(current_module):
-    output = """\
+    if _progress:
+        output = """\
 Optimizing module '{module_name}', {remaining:d} more modules to go \
 after that.""".format(
-            module_name = current_module.getFullName(),
-            remaining   = ModuleRegistry.remainingCount(),
+            module_name=current_module.getFullName(),
+            remaining=ModuleRegistry.getRemainingModulesCount(),
+        )
+        progress_logger.info(output)
+
+    reportProgressBar(
+        stage="Optimization",
+        unit=" modules",
+        item=current_module.getFullName(),
+        total=ModuleRegistry.getRemainingModulesCount()
+        + ModuleRegistry.getDoneModulesCount(),
     )
 
-    if Options.isShowMemory():
-        output += "Memory usage {memory}:".format(
-            memory = MemoryUsage.getHumanReadableProcessMemoryUsage()
+    if _progress and Options.isShowMemory():
+        output = "Memory usage {memory}:".format(
+            memory=getHumanReadableProcessMemoryUsage()
         )
 
-    info(output)
+        memory_logger.info(output)
 
 
 def restoreFromXML(text):
-    from nuitka.TreeXML import fromString
     from nuitka.nodes.NodeBases import fromXML
+    from nuitka.TreeXML import fromString
+
     xml = fromString(text)
 
-    module = fromXML(
-        provider = None,
-        xml      = xml
-    )
+    module = fromXML(provider=None, xml=xml)
 
     return module
 
 
-def makeOptimizationPass(initial_pass):
-    """ Make a single pass for optimization, indication potential completion.
+def makeOptimizationPass():
+    """Make a single pass for optimization, indication potential completion."""
 
-    """
-    # Controls complex optimization, pylint: disable=too-many-branches
+    # Controls complex optimization
 
     finished = True
 
     ModuleRegistry.startTraversal()
 
-    if _progress:
-        if initial_pass:
-            info("Initial optimization pass.")
-        else:
-            info("Next global optimization pass.")
-
     while True:
         current_module = ModuleRegistry.nextModule()
 
         if current_module is None:
             break
 
-        if _progress:
-            _traceProgress(current_module)
+        _traceProgress(current_module)
 
         # The tag set is global, so it can react to changes without context.
         # pylint: disable=global-statement
@@ -464,13 +558,14 @@ def makeOptimizationPass(initial_pass):
     # collections of functions no longer used.
     for current_module in ModuleRegistry.getDoneModules():
         if current_module.isCompiledPythonModule():
-            for function in current_module.getUnusedFunctions():
+            for unused_function in current_module.getUnusedFunctions():
                 Variables.updateVariablesFromCollection(
-                    old_collection = function.trace_collection,
-                    new_collection = None
+                    old_collection=unused_function.trace_collection,
+                    new_collection=None,
+                    source_ref=unused_function.getSourceReference(),
                 )
 
-                function.trace_collection = None
+                unused_function.trace_collection = None
 
     for current_module in ModuleRegistry.getDoneModules():
         if current_module.isCompiledPythonModule():
@@ -484,12 +579,11 @@ def makeOptimizationPass(initial_pass):
 
             used_functions = tuple(
                 function
-                for function in
-                current_module.getFunctions()
+                for function in current_module.subnode_functions
                 if function in used_functions
             )
 
-            current_module.setFunctions(used_functions)
+            current_module.setChild("functions", used_functions)
 
     if Variables.complete:
         if optimizeLocalsDictsHandles():
@@ -498,41 +592,21 @@ def makeOptimizationPass(initial_pass):
     return finished
 
 
+def _getCacheFilename(module):
+    module_cache_dir = os.path.join(getCacheDir(), "module-cache")
+    makePath(module_cache_dir)
+
+    return os.path.join(module_cache_dir, module.getFullName().asString() + ".xml")
+
+
 def _checkXMLPersistence():
-    new_roots = ModuleRegistry.root_modules.__class__()  # @UndefinedVariable
-
-    for module in tuple(ModuleRegistry.getDoneModules()):
-        ModuleRegistry.root_modules.remove(module)
-
-        if module.isPythonShlibModule():
+    for module in ModuleRegistry.getDoneModules():
+        if not module.isCompiledPythonModule():
             continue
 
         text = module.asXmlText()
-        open("out.xml", 'w').write(text)
-        restored = restoreFromXML(text)
-        retext = restored.asXmlText()
-        open("out2.xml", 'w').write(retext)
-
-        assert module.getOutputFilename() == restored.getOutputFilename(), \
-           (module.getOutputFilename(),restored.getOutputFilename())
-
-        # The variable versions give diffs.
-        if True: # To manually enable, pylint: disable=W0125
-            import difflib
-            diff = difflib.unified_diff(
-                text.splitlines(),
-                retext.splitlines(),
-                "xml orig",
-                "xml reloaded"
-            )
-            for line in diff:
-                printLine(line)
-
-        new_roots.add(restored)
-
-    ModuleRegistry.root_modules = new_roots
-    ModuleRegistry.startTraversal()
-
+        with open(_getCacheFilename(module), "w") as f:
+            f.write(text)
 
 
 def optimize(output_filename):
@@ -540,28 +614,36 @@ def optimize(output_filename):
 
     # First pass.
     if _progress:
-        info("PASS 1:")
+        progress_logger.info("PASS 1:")
 
-    makeOptimizationPass(initial_pass = True)
+    makeOptimizationPass()
     Variables.complete = True
 
-    finished = makeOptimizationPass(initial_pass = False)
+    if _progress:
+        progress_logger.info("PASS 2:")
+
+    finished = makeOptimizationPass()
 
     if Options.isExperimental("check_xml_persistence"):
         _checkXMLPersistence()
 
     # Demote compiled modules to bytecode, now that imports had a chance to be resolved, and
     # dependencies were handled.
-    for module in ModuleRegistry.getDoneUserModules():
-        if module.isCompiledPythonModule() and \
-           module.mode == "bytecode":
+    for module in ModuleRegistry.getDoneModules():
+        if (
+            module.isCompiledPythonModule()
+            and module.getCompilationMode() == "bytecode"
+        ):
             demoteCompiledModuleToBytecode(module)
 
-    if _progress:
-        info("PASS 2 ... :")
-
+    pass_count = 2
     # Second, "endless" pass.
     while not finished:
-        finished = makeOptimizationPass(initial_pass = False)
+        pass_count += 1
+
+        if _progress:
+            progress_logger.info("PASS %d:" % pass_count)
+
+        finished = makeOptimizationPass()
 
     Graphs.endGraph(output_filename)
