@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,10 +21,7 @@ For built-in name references, we check if it's one of the supported built-in
 types, and then specialize for the ones, where it makes sense.
 """
 
-from logging import warning
-
 from nuitka.__past__ import xrange  # pylint: disable=I0021,redefined-builtin
-from nuitka.Builtins import calledWithBuiltinArgumentNamesDecorator
 from nuitka.Errors import NuitkaAssumptionError
 from nuitka.nodes.AssignNodes import (
     StatementAssignmentVariable,
@@ -112,6 +109,7 @@ from nuitka.nodes.ConditionalNodes import (
     makeStatementConditional,
 )
 from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
+from nuitka.nodes.ContainerMakingNodes import makeExpressionMakeTupleOrConstant
 from nuitka.nodes.ExecEvalNodes import (
     ExpressionBuiltinCompile,
     ExpressionBuiltinEval,
@@ -135,8 +133,8 @@ from nuitka.nodes.OperatorNodes import (
     ExpressionOperationUnary,
 )
 from nuitka.nodes.OutlineNodes import ExpressionOutlineBody
-from nuitka.nodes.ReturnNodes import StatementReturn
-from nuitka.nodes.SliceNodes import ExpressionBuiltinSlice
+from nuitka.nodes.ReturnNodes import makeStatementReturn
+from nuitka.nodes.SliceNodes import makeExpressionBuiltinSlice
 from nuitka.nodes.TypeNodes import (
     ExpressionBuiltinIsinstance,
     ExpressionBuiltinSuper,
@@ -148,13 +146,13 @@ from nuitka.nodes.VariableRefNodes import (
 )
 from nuitka.PythonVersions import python_version
 from nuitka.specs import BuiltinParameterSpecs
+from nuitka.Tracing import optimization_logger
 from nuitka.tree.ReformulationExecStatements import wrapEvalGlobalsAndLocals
 from nuitka.tree.ReformulationTryFinallyStatements import (
     makeTryFinallyStatement,
 )
 from nuitka.tree.TreeHelpers import (
     makeCallNode,
-    makeSequenceCreationOrConstant,
     makeStatementsSequence,
     makeStatementsSequenceFromStatement,
 )
@@ -177,7 +175,7 @@ def dir_extractor(node):
 
         # For Python3, keys doesn't really return values, but instead a handle
         # only, but we want it to be a list.
-        if python_version >= 300:
+        if python_version >= 0x300:
             result = ExpressionBuiltinList(value=result, source_ref=source_ref)
 
         return result
@@ -217,7 +215,7 @@ def import_extractor(node):
 
 
 def type_extractor(node):
-    args = node.getCallArgs()
+    args = node.subnode_args
 
     if args is None:
         iter_length = 0
@@ -243,7 +241,6 @@ def type_extractor(node):
 
 
 def iter_extractor(node):
-    @calledWithBuiltinArgumentNamesDecorator
     def wrapIterCreation(callable_arg, sentinel, source_ref):
         if sentinel is None:
             return ExpressionBuiltinIter1(value=callable_arg, source_ref=source_ref)
@@ -294,7 +291,7 @@ def sum_extractor(node):
             expression=node,
             exception=TypeError(
                 "sum expected at least 1 arguments, got 0"
-                if python_version < 380
+                if python_version < 0x380
                 else "sum() takes at least 1 positional argument (0 given)"
             ),
         )
@@ -414,7 +411,7 @@ def repr_extractor(node):
     )
 
 
-if python_version >= 300:
+if python_version >= 0x300:
 
     def ascii_extractor(node):
         return BuiltinParameterSpecs.extractBuiltinArgs(
@@ -570,7 +567,9 @@ def float_extractor(node):
     def makeFloat0(source_ref):
         # pylint: disable=unused-argument
 
-        return makeConstantReplacementNode(constant=float(), node=node)
+        return makeConstantReplacementNode(
+            constant=float(), node=node, user_provided=False
+        )
 
     return BuiltinParameterSpecs.extractBuiltinArgs(
         node=node,
@@ -584,7 +583,9 @@ def complex_extractor(node):
     def makeComplex0(source_ref):
         # pylint: disable=unused-argument
 
-        return makeConstantReplacementNode(constant=complex(), node=node)
+        return makeConstantReplacementNode(
+            constant=complex(), node=node, user_provided=False
+        )
 
     def selectComplexBuiltin(real, imag, source_ref):
         if imag is None:
@@ -612,7 +613,7 @@ def str_extractor(node):
     )
 
 
-if python_version < 300:
+if python_version < 0x300:
 
     def unicode_extractor(node):
         return BuiltinParameterSpecs.extractBuiltinArgs(
@@ -632,7 +633,9 @@ else:
         def makeBytes0(source_ref):
             # pylint: disable=unused-argument
 
-            return makeConstantReplacementNode(constant=bytes(), node=node)
+            return makeConstantReplacementNode(
+                constant=bytes(), node=node, user_provided=False
+            )
 
         def selectBytesBuiltin(string, encoding, errors, source_ref):
             if encoding is None and errors is None:
@@ -654,10 +657,18 @@ else:
 
 
 def bool_extractor(node):
+    def makeBool0(source_ref):
+        # pylint: disable=unused-argument
+
+        return makeConstantReplacementNode(
+            constant=bool(), node=node, user_provided=False
+        )
+
     return BuiltinParameterSpecs.extractBuiltinArgs(
         node=node,
         builtin_class=ExpressionBuiltinBool,
         builtin_spec=BuiltinParameterSpecs.builtin_bool_spec,
+        empty_special_class=makeBool0,
     )
 
 
@@ -665,7 +676,9 @@ def int_extractor(node):
     def makeInt0(source_ref):
         # pylint: disable=unused-argument
 
-        return makeConstantReplacementNode(constant=int(), node=node)
+        return makeConstantReplacementNode(
+            constant=int(), node=node, user_provided=False
+        )
 
     def selectIntBuiltin(value, base, source_ref):
         if base is None:
@@ -681,7 +694,7 @@ def int_extractor(node):
     )
 
 
-if python_version < 300:
+if python_version < 0x300:
     from nuitka.nodes.BuiltinIntegerNodes import (
         ExpressionBuiltinLong1,
         ExpressionBuiltinLong2,
@@ -691,7 +704,9 @@ if python_version < 300:
         def makeLong0(source_ref):
             # pylint: disable=unused-argument
 
-            return makeConstantReplacementNode(constant=int(), node=node)
+            return makeConstantReplacementNode(
+                constant=int(), node=node, user_provided=False
+            )
 
         def selectIntBuiltin(value, base, source_ref):
             if base is None:
@@ -733,11 +748,10 @@ def locals_extractor(node):
     )
 
 
-if python_version < 300:
+if python_version < 0x300:
     from nuitka.nodes.ExecEvalNodes import ExpressionBuiltinExecfile
 
     def execfile_extractor(node):
-        @calledWithBuiltinArgumentNamesDecorator
         def wrapExpressionBuiltinExecfileCreation(
             filename, globals_arg, locals_arg, source_ref
         ):
@@ -758,7 +772,7 @@ if python_version < 300:
             tried = makeStatementsSequence(
                 statements=(
                     tried,
-                    StatementReturn(
+                    makeStatementReturn(
                         expression=ExpressionBuiltinExecfile(
                             source_code=makeCallNode(
                                 ExpressionAttributeLookup(
@@ -786,7 +800,8 @@ if python_version < 300:
                 source_ref=source_ref,
             )
 
-            outline_body.setBody(
+            outline_body.setChild(
+                "body",
                 makeStatementsSequenceFromStatement(
                     statement=makeTryFinallyStatement(
                         provider=outline_body,
@@ -794,7 +809,7 @@ if python_version < 300:
                         final=final,
                         source_ref=source_ref,
                     )
-                )
+                ),
             )
 
             return outline_body
@@ -807,7 +822,6 @@ if python_version < 300:
 
 
 def eval_extractor(node):
-    @calledWithBuiltinArgumentNamesDecorator
     def wrapEvalBuiltin(source, globals_arg, locals_arg, source_ref):
         provider = node.getParentVariableProvider()
 
@@ -839,18 +853,19 @@ def eval_extractor(node):
             temp_scope=None, name="source"
         )
 
-        final.setStatements(
-            final.getStatements()
+        final.setChild(
+            "statements",
+            final.subnode_statements
             + (
                 StatementDelVariable(
                     variable=source_variable, tolerant=True, source_ref=source_ref
                 ),
-            )
+            ),
         )
 
         strip_choice = makeConstantRefNode(constant=(" \t",), source_ref=source_ref)
 
-        if python_version >= 300:
+        if python_version >= 0x300:
             strip_choice = ExpressionConditional(
                 condition=ExpressionComparisonIs(
                     left=ExpressionBuiltinType1(
@@ -894,7 +909,7 @@ def eval_extractor(node):
             ExpressionBuiltinAnonymousRef(builtin_name="code", source_ref=source_ref)
         ]
 
-        if python_version >= 270:
+        if python_version >= 0x270:
             acceptable_builtin_types.append(
                 makeExpressionBuiltinTypeRef(
                     builtin_name="memoryview", source_ref=source_ref
@@ -911,9 +926,9 @@ def eval_extractor(node):
                         instance=ExpressionTempVariableRef(
                             variable=source_variable, source_ref=source_ref
                         ),
-                        classes=makeSequenceCreationOrConstant(
-                            sequence_kind="tuple",
+                        classes=makeExpressionMakeTupleOrConstant(
                             elements=acceptable_builtin_types,
+                            user_provided=True,
                             source_ref=source_ref,
                         ),
                         source_ref=source_ref,
@@ -924,7 +939,7 @@ def eval_extractor(node):
                 no_branch=None,
                 source_ref=source_ref,
             ),
-            StatementReturn(
+            makeStatementReturn(
                 expression=ExpressionBuiltinEval(
                     source_code=ExpressionTempVariableRef(
                         variable=source_variable, source_ref=source_ref
@@ -941,7 +956,8 @@ def eval_extractor(node):
             statements=(tried,) + statements, allow_none=False, source_ref=source_ref
         )
 
-        outline_body.setBody(
+        outline_body.setChild(
+            "body",
             makeStatementsSequenceFromStatement(
                 statement=makeTryFinallyStatement(
                     provider=outline_body,
@@ -949,7 +965,7 @@ def eval_extractor(node):
                     final=final,
                     source_ref=source_ref,
                 )
-            )
+            ),
         )
 
         return outline_body
@@ -961,11 +977,10 @@ def eval_extractor(node):
     )
 
 
-if python_version >= 300:
+if python_version >= 0x300:
     from nuitka.nodes.ExecEvalNodes import ExpressionBuiltinExec
 
     def exec_extractor(node):
-        @calledWithBuiltinArgumentNamesDecorator
         def wrapExpressionBuiltinExecCreation(
             source, globals_arg, locals_arg, source_ref
         ):
@@ -986,7 +1001,7 @@ if python_version >= 300:
             tried = makeStatementsSequence(
                 statements=(
                     tried,
-                    StatementReturn(
+                    makeStatementReturn(
                         expression=ExpressionBuiltinExec(
                             source_code=source,
                             globals_arg=globals_ref,
@@ -1003,7 +1018,8 @@ if python_version >= 300:
             # Hack: Allow some APIs to work already
             tried.parent = outline_body
 
-            outline_body.setBody(
+            outline_body.setChild(
+                "body",
                 makeStatementsSequenceFromStatement(
                     statement=makeTryFinallyStatement(
                         provider=provider,
@@ -1011,7 +1027,7 @@ if python_version >= 300:
                         final=final,
                         source_ref=source_ref,
                     )
-                )
+                ),
             )
 
             return outline_body
@@ -1065,9 +1081,8 @@ def open_extractor(node):
 
 
 def super_extractor(node):
-    @calledWithBuiltinArgumentNamesDecorator
     def wrapSuperBuiltin(type_arg, object_arg, source_ref):
-        if type_arg is None and python_version >= 300:
+        if type_arg is None and python_version >= 0x300:
             if provider.isCompiledPythonModule():
                 return makeRaiseExceptionReplacementExpression(
                     expression=node,
@@ -1099,7 +1114,7 @@ def super_extractor(node):
                 return makeRaiseExceptionReplacementExpression(
                     expression=node,
                     exception_type="SystemError"
-                    if python_version < 331
+                    if python_version < 0x331
                     else "RuntimeError",
                     exception_value="super(): __class__ cell not found",
                 )
@@ -1133,7 +1148,7 @@ def super_extractor(node):
                         return makeRaiseExceptionReplacementExpression(
                             expression=node,
                             exception_type="SystemError"
-                            if python_version < 300
+                            if python_version < 0x300
                             else "RuntimeError",
                             exception_value="super(): __class__ cell not found",
                         )
@@ -1238,7 +1253,7 @@ def slice_extractor(node):
             stop = start
             start = None
 
-        return ExpressionBuiltinSlice(
+        return makeExpressionBuiltinSlice(
             start=start, stop=stop, step=step, source_ref=source_ref
         )
 
@@ -1364,7 +1379,7 @@ _dispatch_dict = {
     "divmod": divmod_extractor,
 }
 
-if python_version < 300:
+if python_version < 0x300:
     # These are not in Python3
     _dispatch_dict["long"] = long_extractor
     _dispatch_dict["unicode"] = unicode_extractor
@@ -1413,8 +1428,9 @@ _builtin_white_list = (
 def _describeNewNode(builtin_name, inspect_node):
     """Describe the change for better understanding."""
 
+    # Don't mention side effects, that's not what we care about.
     if inspect_node.isExpressionSideEffects():
-        inspect_node = inspect_node.getExpression()
+        inspect_node = inspect_node.subnode_expression
 
     if inspect_node.isExpressionBuiltinImport():
         tags = "new_import"
@@ -1488,6 +1504,8 @@ def computeBuiltinCall(builtin_name, call_node):
         return new_node, tags, message
     else:
         if False and builtin_name not in _builtin_white_list:
-            warning("Not handling built-in '%s', consider support." % builtin_name)
+            optimization_logger.warning(
+                "Not handling built-in '%s', consider support." % builtin_name
+            )
 
         return call_node, None, None
