@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,7 +24,6 @@ This is about collecting these constraints and to manage them.
 """
 
 import contextlib
-from logging import debug
 
 from nuitka import Tracing, Variables
 from nuitka.__past__ import iterItems  # Python3 compatibility.
@@ -32,135 +31,37 @@ from nuitka.containers.oset import OrderedSet
 from nuitka.importing.ImportCache import getImportedModuleByNameAndPath
 from nuitka.ModuleRegistry import addUsedModule
 from nuitka.nodes.NodeMakingHelpers import getComputationResult
-from nuitka.nodes.shapes.BuiltinTypeShapes import (
-    ShapeTypeDict,
-    ShapeTypeInt,
-    ShapeTypeIntOrLong,
-    ShapeTypeLong
-)
-from nuitka.PythonVersions import python_version
+from nuitka.nodes.shapes.BuiltinTypeShapes import tshape_dict
+from nuitka.nodes.shapes.StandardShapes import tshape_uninit
 from nuitka.tree.SourceReading import readSourceLine
 from nuitka.utils.FileOperations import relpath
 from nuitka.utils.InstanceCounters import counted_del, counted_init
+from nuitka.utils.ModuleNames import ModuleName
 
 from .ValueTraces import (
     ValueTraceAssign,
+    ValueTraceDeleted,
+    ValueTraceEscaped,
     ValueTraceInit,
     ValueTraceLoopComplete,
-    ValueTraceLoopInitial,
+    ValueTraceLoopIncomplete,
     ValueTraceMerge,
     ValueTraceUninit,
-    ValueTraceUnknown
+    ValueTraceUnknown,
 )
 
 signalChange = None
 
 
-class CollectionTracingMixin(object):
-    """ This contains for logic for maintaining active traces.
+class CollectionStartpointMixin(object):
+    """Mixin to use in start points of collections.
 
-        They are kept for "variable" and versions.
+    These are modules, functions, etc. typically entry points.
     """
 
-    def __init__(self):
-        # Currently active values in the tracing.
-        self.variable_actives = {}
+    # Mixins are not allow to specify slots, pylint: disable=assigning-non-slot
+    __slots__ = ()
 
-    def getVariableCurrentTrace(self, variable):
-        """ Get the current value trace associated to this variable
-
-            It is also created on the fly if necessary. We create them
-            lazy so to keep the tracing branches minimal where possible.
-        """
-
-        return self.getVariableTrace(
-            variable = variable,
-            version  = self._getCurrentVariableVersion(variable)
-        )
-
-    def markCurrentVariableTrace(self, variable, version):
-        self.variable_actives[variable] = version
-
-    def _getCurrentVariableVersion(self, variable):
-        try:
-            return self.variable_actives[variable]
-        except KeyError:
-            # Initialize variables on the fly.
-            if not self.hasVariableTrace(variable, 0):
-                self.initVariable(variable)
-
-            self.markCurrentVariableTrace(variable, 0)
-
-            return self.variable_actives[variable]
-
-    def getActiveVariables(self):
-        return self.variable_actives.keys()
-
-    def markActiveVariableAsUnknown(self, variable):
-        current = self.getVariableCurrentTrace(
-            variable = variable,
-        )
-
-        if not current.isUnknownTrace():
-            version = variable.allocateTargetNumber()
-
-            self.addVariableTrace(
-                variable = variable,
-                version  = version,
-                trace    = ValueTraceUnknown(
-                    owner    = self.owner,
-                    previous = current
-                )
-            )
-
-            self.markCurrentVariableTrace(variable, version)
-
-    def markActiveVariableAsLoopMerge(self, variable, shapes, initial):
-        current = self.getVariableCurrentTrace(
-            variable = variable,
-        )
-
-        version = variable.allocateTargetNumber()
-
-        current_shape = current.getTypeShape()
-
-        if current_shape not in shapes:
-            shapes = set(shapes)
-            shapes.add(current_shape)
-
-        if python_version < 300:
-            if ShapeTypeIntOrLong in shapes:
-                if ShapeTypeInt in shapes:
-                    shapes.discard(ShapeTypeInt)
-                if ShapeTypeLong in shapes:
-                    shapes.discard(ShapeTypeLong)
-
-        # print(initial, shapes)
-
-        if initial:
-            result = ValueTraceLoopInitial(current, shapes)
-        else:
-            result = ValueTraceLoopComplete(current, shapes)
-
-        self.addVariableTrace(
-            variable = variable,
-            version  = version,
-            trace    = result
-        )
-
-        self.markCurrentVariableTrace(variable, version)
-
-        return result
-
-    def markActiveVariablesAsUnknown(self):
-        for variable in self.getActiveVariables():
-            if variable.isTempVariable():
-                continue
-
-            self.markActiveVariableAsUnknown(variable)
-
-
-class CollectionStartpointMixin(object):
     # Many things are traced
 
     def __init__(self):
@@ -182,44 +83,55 @@ class CollectionStartpointMixin(object):
     def getLoopBreakCollections(self):
         return self.break_collections
 
-    def onLoopBreak(self, collection = None):
+    def onLoopBreak(self, collection=None):
         if collection is None:
             collection = self
 
         self.break_collections.append(
-            TraceCollectionBranch(
-                parent = collection,
-                name   = "loop break"
-            )
+            TraceCollectionBranch(parent=collection, name="loop break")
         )
 
     def getLoopContinueCollections(self):
         return self.continue_collections
 
-    def onLoopContinue(self, collection = None):
+    def onLoopContinue(self, collection=None):
         if collection is None:
             collection = self
 
         self.continue_collections.append(
-            TraceCollectionBranch(
-                parent = collection,
-                name   = "loop continue"
-            )
+            TraceCollectionBranch(parent=collection, name="loop continue")
         )
 
-    def onFunctionReturn(self, collection = None):
+    def onFunctionReturn(self, collection=None):
         if collection is None:
             collection = self
 
         if self.return_collections is not None:
             self.return_collections.append(
-                TraceCollectionBranch(
-                    parent = collection,
-                    name   = "return"
-                )
+                TraceCollectionBranch(parent=collection, name="return")
             )
 
-    def onExceptionRaiseExit(self, raisable_exceptions, collection = None):
+    def onExceptionRaiseExit(self, raisable_exceptions, collection=None):
+        """Indicate to the trace collection what exceptions may have occurred.
+
+        Args:
+            raisable_exception: Currently ignored, one or more exceptions that
+            could occur, e.g. "BaseException".
+            collection: To pass the collection that will be the parent
+        Notes:
+            Currently this is unused. Passing "collection" as an argument, so
+            we know the original collection to attach the branch to, is maybe
+            not the most clever way to do this
+
+            We also might want to specialize functions for specific exceptions,
+            there is little point in providing BaseException as an argument in
+            so many places.
+
+            The actual storage of the exceptions that can occur is currently
+            missing entirely. We just use this to detect "any exception" by
+            not being empty.
+        """
+
         # TODO: We might want to track per exception, pylint: disable=unused-argument
 
         if collection is None:
@@ -227,10 +139,7 @@ class CollectionStartpointMixin(object):
 
         if self.exception_collections is not None:
             self.exception_collections.append(
-                TraceCollectionBranch(
-                    parent = collection,
-                    name   = "exception"
-                )
+                TraceCollectionBranch(parent=collection, name="exception")
             )
 
     def getFunctionReturnCollections(self):
@@ -273,68 +182,35 @@ class CollectionStartpointMixin(object):
         self.addVariableTrace(variable, version, trace_merge)
 
         return version
-#         return version, trace_merge
-
-    def dumpTraces(self):
-        debug("Constraint collection state: %s", self)
-        for _variable_desc, variable_trace in sorted(iterItems(self.variable_traces)):
-
-            # debug( "%r: %r", variable_trace )
-            variable_trace.dump()
-
-    def dumpActiveTraces(self):
-        Tracing.printSeparator()
-        Tracing.printLine("Active are:")
-        for variable, _version in sorted(self.variable_actives.iteritems()):
-            self.getVariableCurrentTrace(variable).dump()
-
-        Tracing.printSeparator()
 
     def initVariableUnknown(self, variable):
-        trace = ValueTraceUnknown(
-            owner    = self.owner,
-            previous = None
-        )
+        trace = ValueTraceUnknown(owner=self.owner, previous=None)
 
-        self.addVariableTrace(
-            variable = variable,
-            version  = 0,
-            trace    = trace
-        )
+        self.addVariableTrace(variable=variable, version=0, trace=trace)
 
         return trace
 
     def _initVariableInit(self, variable):
         trace = ValueTraceInit(self.owner)
 
-        self.addVariableTrace(
-            variable = variable,
-            version  = 0,
-            trace    = trace
-        )
+        self.addVariableTrace(variable=variable, version=0, trace=trace)
 
         return trace
 
     def _initVariableUninit(self, variable):
-        trace = ValueTraceUninit(
-            owner    = self.owner,
-            previous = None
-        )
+        trace = ValueTraceUninit(owner=self.owner, previous=None)
 
-        self.addVariableTrace(
-            variable = variable,
-            version  = 0,
-            trace    = trace
-        )
+        self.addVariableTrace(variable=variable, version=0, trace=trace)
 
         return trace
 
-    def updateVariablesFromCollection(self, old_collection):
-        Variables.updateVariablesFromCollection(old_collection, self)
+    def updateVariablesFromCollection(self, old_collection, source_ref):
+        Variables.updateVariablesFromCollection(old_collection, self, source_ref)
 
     @contextlib.contextmanager
-    def makeAbortStackContext(self, catch_breaks, catch_continues,
-                              catch_returns, catch_exceptions):
+    def makeAbortStackContext(
+        self, catch_breaks, catch_continues, catch_returns, catch_exceptions
+    ):
         if catch_breaks:
             old_break_collections = self.break_collections
             self.break_collections = []
@@ -369,7 +245,7 @@ class CollectionStartpointMixin(object):
         elif variable.isTempVariable():
             result = self._initVariableUninit(variable)
         elif variable.isLocalsDictVariable():
-            if variable.getOwner().getTypeShape() is ShapeTypeDict:
+            if variable.getOwner().getTypeShape() is tshape_dict:
                 result = self._initVariableUninit(variable)
             else:
                 result = self.initVariableUnknown(variable)
@@ -388,27 +264,30 @@ class CollectionStartpointMixin(object):
         return self.outline_functions
 
     def onLocalsDictEscaped(self, locals_scope):
-        # TODO: Limit to the scope.
         if locals_scope is not None:
             for variable in locals_scope.variables.values():
-                self.markActiveVariableAsUnknown(variable)
-
+                self.markActiveVariableAsEscaped(variable)
 
+        # TODO: Limit to the scope.
         for variable in self.getActiveVariables():
             if variable.isTempVariable() or variable.isModuleVariable():
                 continue
 
-            self.markActiveVariableAsUnknown(variable)
+            self.markActiveVariableAsEscaped(variable)
 
 
+class TraceCollectionBase(object):
+    """This contains for logic for maintaining active traces.
+
+    They are kept for "variable" and versions.
+    """
+
+    __slots__ = ("owner", "parent", "name", "value_states", "variable_actives")
 
-class TraceCollectionBase(CollectionTracingMixin):
     __del__ = counted_del()
 
     @counted_init
     def __init__(self, owner, name, parent):
-        CollectionTracingMixin.__init__(self)
-
         self.owner = owner
         self.parent = parent
         self.name = name
@@ -416,19 +295,90 @@ class TraceCollectionBase(CollectionTracingMixin):
         # Value state extra information per node.
         self.value_states = {}
 
+        # Currently active values in the tracing.
+        self.variable_actives = {}
+
     def __repr__(self):
-        return "<%s for %s at 0x%x>" % (
-            self.__class__.__name__,
-            self.name,
-            id(self)
-        )
+        return "<%s for %s at 0x%x>" % (self.__class__.__name__, self.name, id(self))
 
     def getOwner(self):
         return self.owner
 
+    def getVariableCurrentTrace(self, variable):
+        """Get the current value trace associated to this variable
+
+        It is also created on the fly if necessary. We create them
+        lazy so to keep the tracing branches minimal where possible.
+        """
+
+        return self.getVariableTrace(
+            variable=variable, version=self._getCurrentVariableVersion(variable)
+        )
+
+    def markCurrentVariableTrace(self, variable, version):
+        self.variable_actives[variable] = version
+
+    def _getCurrentVariableVersion(self, variable):
+        try:
+            return self.variable_actives[variable]
+        except KeyError:
+            # Initialize variables on the fly.
+            if not self.hasVariableTrace(variable, 0):
+                self.initVariable(variable)
+
+            self.markCurrentVariableTrace(variable, 0)
+
+            return self.variable_actives[variable]
+
+    def getActiveVariables(self):
+        return self.variable_actives.keys()
+
+    def markActiveVariableAsEscaped(self, variable):
+        current = self.getVariableCurrentTrace(variable=variable)
+
+        if not current.isUnknownTrace():
+            version = variable.allocateTargetNumber()
+
+            self.addVariableTrace(
+                variable=variable,
+                version=version,
+                trace=ValueTraceEscaped(owner=self.owner, previous=current),
+            )
+
+            self.markCurrentVariableTrace(variable, version)
+
+    def markActiveVariableAsLoopMerge(
+        self, loop_node, current, variable, shapes, incomplete
+    ):
+        if incomplete:
+            result = ValueTraceLoopIncomplete(loop_node, current, shapes)
+        else:
+            # TODO: Empty is a missing optimization somewhere, but it also happens that
+            # a variable is getting released in a loop.
+            # assert shapes, (variable, current)
+
+            if not shapes:
+                shapes.add(tshape_uninit)
+
+            result = ValueTraceLoopComplete(loop_node, current, shapes)
+
+        version = variable.allocateTargetNumber()
+        self.addVariableTrace(variable=variable, version=version, trace=result)
+
+        self.markCurrentVariableTrace(variable, version)
+
+        return result
+
+    def markActiveVariablesAsUnknown(self):
+        for variable in self.getActiveVariables():
+            if variable.isTempVariable():
+                continue
+
+            self.markActiveVariableAsEscaped(variable)
+
     @staticmethod
     def signalChange(tags, source_ref, message):
-        # This is monkey patched from another module.
+        # This is monkey patched from another module. pylint: disable=I0021,not-callable
         signalChange(tags, source_ref, message)
 
     def onUsedModule(self, module_name, module_relpath):
@@ -450,9 +400,6 @@ class TraceCollectionBase(CollectionTracingMixin):
 
         return False
 
-    def removeKnowledge(self, node):
-        pass
-
     def onControlFlowEscape(self, node):
         # TODO: One day, we should trace which nodes exactly cause a variable
         # to be considered escaped, pylint: disable=unused-argument
@@ -461,11 +408,19 @@ class TraceCollectionBase(CollectionTracingMixin):
             if variable.isModuleVariable():
                 # print variable
 
-                self.markActiveVariableAsUnknown(variable)
+                self.markActiveVariableAsEscaped(variable)
 
             elif variable.isLocalVariable():
-                if python_version >= 300 and variable.hasWritesOutsideOf(self.owner) is not False:
-                    self.markActiveVariableAsUnknown(variable)
+                if variable.hasAccessesOutsideOf(self.owner) is not False:
+                    self.markActiveVariableAsEscaped(variable)
+
+    def removeKnowledge(self, node):
+        if node.isExpressionVariableRef():
+            self.markActiveVariableAsEscaped(node.variable)
+
+    def onValueEscapeStr(self, node):
+        # TODO: We can ignore these for now.
+        pass
 
     def removeAllKnowledge(self):
         self.markActiveVariablesAsUnknown()
@@ -484,77 +439,55 @@ class TraceCollectionBase(CollectionTracingMixin):
 
     def onVariableSet(self, variable, version, assign_node):
         variable_trace = ValueTraceAssign(
-            owner       = self.owner,
-            assign_node = assign_node,
-            previous    = self.getVariableCurrentTrace(
-                variable = variable
-            )
+            owner=self.owner,
+            assign_node=assign_node,
+            previous=self.getVariableCurrentTrace(variable=variable),
         )
 
-        self.addVariableTrace(
-            variable = variable,
-            version  = version,
-            trace    = variable_trace
-        )
+        self.addVariableTrace(variable=variable, version=version, trace=variable_trace)
 
         # Make references point to it.
         self.markCurrentVariableTrace(variable, version)
 
         return variable_trace
 
-    def onVariableDel(self, variable, version):
+    def onVariableDel(self, variable, version, del_node):
         # Add a new trace, allocating a new version for the variable, and
         # remember the delete of the current
         old_trace = self.getVariableCurrentTrace(variable)
 
-        variable_trace = ValueTraceUninit(
-            owner    = self.owner,
-            previous = old_trace
+        variable_trace = ValueTraceDeleted(
+            owner=self.owner, del_node=del_node, previous=old_trace
         )
 
         # Assign to not initialized again.
-        self.addVariableTrace(
-            variable = variable,
-            version  = version,
-            trace    = variable_trace
-        )
+        self.addVariableTrace(variable=variable, version=version, trace=variable_trace)
 
         # Make references point to it.
         self.markCurrentVariableTrace(variable, version)
 
         return variable_trace
 
-    def onLocalsUsage(self, locals_owner):
-        self.onLocalsDictEscaped(locals_owner.getFunctionLocalsScope())
+    def onLocalsUsage(self, locals_scope):
+        self.onLocalsDictEscaped(locals_scope)
 
         result = []
 
-        include_closure = locals_owner.isExpressionFunctionBody() and \
-                          not locals_owner.isUnoptimized()
+        scope_locals_variables = locals_scope.getLocalsRelevantVariables()
 
         for variable in self.getActiveVariables():
-            if variable.isLocalVariable() and \
-               (variable.getOwner() is locals_owner or
-                include_closure and locals_owner.hasClosureVariable(variable)) and \
-               variable.getName() != ".0":
-                variable_trace = self.getVariableCurrentTrace(
-                    variable
-                )
+            if variable.isLocalVariable() and variable in scope_locals_variables:
+                variable_trace = self.getVariableCurrentTrace(variable)
 
                 variable_trace.addNameUsage()
-                result.append(
-                    (
-                        variable,
-                        variable_trace
-                    )
-                )
+                result.append((variable, variable_trace))
 
         return result
 
     def onVariableContentEscapes(self, variable):
-        self.getVariableCurrentTrace(variable).onValueEscape()
+        self.markActiveVariableAsEscaped(variable)
 
-    def onExpression(self, expression, allow_none = False):
+    def onExpression(self, expression, allow_none=False):
         if expression is None and allow_none:
             return None
 
@@ -565,21 +498,15 @@ class TraceCollectionBase(CollectionTracingMixin):
 
         # Now compute this expression, allowing it to replace itself with
         # something else as part of a local peep hole optimization.
-        r = expression.computeExpressionRaw(
-            trace_collection = self
-        )
-        assert type(r) is tuple, expression
+        r = expression.computeExpressionRaw(trace_collection=self)
+        assert type(r) is tuple, (expression, expression.getVisitableNodes(), r)
 
         new_node, change_tags, change_desc = r
 
         if change_tags is not None:
             # This is mostly for tracing and indication that a change occurred
             # and it may be interesting to look again.
-            self.signalChange(
-                change_tags,
-                expression.getSourceReference(),
-                change_desc
-            )
+            self.signalChange(change_tags, expression.getSourceReference(), change_desc)
 
         if new_node is not expression:
             parent.replaceChild(expression, new_node)
@@ -590,48 +517,133 @@ class TraceCollectionBase(CollectionTracingMixin):
         try:
             assert statement.isStatement(), statement
 
-            new_statement, change_tags, change_desc = \
-              statement.computeStatement(self)
+            new_statement, change_tags, change_desc = statement.computeStatement(self)
 
             # print new_statement, change_tags, change_desc
             if new_statement is not statement:
                 self.signalChange(
-                    change_tags,
-                    statement.getSourceReference(),
-                    change_desc
+                    change_tags, statement.getSourceReference(), change_desc
                 )
 
             return new_statement
         except Exception:
             Tracing.printError(
-                "Problem with statement at %s:\n-> %s" % (
+                "Problem with statement at %s:\n-> %s"
+                % (
                     statement.source_ref.getAsString(),
-                    readSourceLine(statement.source_ref)
+                    readSourceLine(statement.source_ref),
                 )
             )
             raise
 
-    def mergeBranches(self, collection_yes, collection_no):
-        """ Merge two alternative branches into this trace.
+    def computedStatementResult(self, statement, change_tags, change_desc):
+        """Make sure the replacement statement is computed.
 
-            This is mostly for merging conditional branches, or other ways
-            of having alternative control flow. This deals with up to two
-            alternative branches to both change this collection.
+        Use this when a replacement expression needs to be seen by the trace
+        collection and be computed, without causing any duplication, but where
+        otherwise there would be loss of annotated effects.
+
+        This may e.g. be true for nodes that need an initial run to know their
+        exception result and type shape.
+        """
+        # Need to compute the replacement still.
+        new_statement = statement.computeStatement(self)
+
+        if new_statement[0] is not statement:
+            # Signal intermediate result as well.
+            self.signalChange(change_tags, statement.getSourceReference(), change_desc)
+
+            return new_statement
+        else:
+            return statement, change_tags, change_desc
+
+    def computedExpressionResult(self, expression, change_tags, change_desc):
+        """Make sure the replacement expression is computed.
+
+        Use this when a replacement expression needs to be seen by the trace
+        collection and be computed, without causing any duplication, but where
+        otherwise there would be loss of annotated effects.
+
+        This may e.g. be true for nodes that need an initial run to know their
+        exception result and type shape.
         """
 
-        # Refuse to do stupid work
-        if collection_yes is None and collection_no is None:
-            return None
-        elif collection_yes is None or collection_no is None:
+        # Need to compute the replacement still.
+        new_expression = expression.computeExpression(self)
+
+        if new_expression[0] is not expression:
+            # Signal intermediate result as well.
+            self.signalChange(change_tags, expression.getSourceReference(), change_desc)
+
+            return new_expression
+        else:
+            return expression, change_tags, change_desc
+
+    def mergeBranches(self, collection_yes, collection_no):
+        """Merge two alternative branches into this trace.
+
+        This is mostly for merging conditional branches, or other ways
+        of having alternative control flow. This deals with up to two
+        alternative branches to both change this collection.
+        """
+
+        # Many branches due to inlining the actual merge and preparing it
+        # pylint: disable=too-many-branches
+
+        if collection_yes is None:
+            if collection_no is not None:
+                # Handle one branch case, we need to merge versions backwards as
+                # they may make themselves obsolete.
+                collection1 = self
+                collection2 = collection_no
+            else:
+                # Refuse to do stupid work
+                return
+        elif collection_no is None:
             # Handle one branch case, we need to merge versions backwards as
             # they may make themselves obsolete.
-            return self.mergeMultipleBranches(
-                collections = (self, collection_yes or collection_no)
-            )
+            collection1 = self
+            collection2 = collection_yes
         else:
-            return self.mergeMultipleBranches(
-                collections = (collection_yes, collection_no)
-            )
+            # Handle two branch case, they may or may not do the same things.
+            collection1 = collection_yes
+            collection2 = collection_no
+
+        variable_versions = {}
+
+        for variable, version in iterItems(collection1.variable_actives):
+            variable_versions[variable] = version
+
+        for variable, version in iterItems(collection2.variable_actives):
+            if variable not in variable_versions:
+                variable_versions[variable] = 0, version
+            else:
+                other = variable_versions[variable]
+
+                if other != version:
+                    variable_versions[variable] = other, version
+                else:
+                    variable_versions[variable] = other
+
+        for variable in variable_versions:
+            if variable not in collection2.variable_actives:
+                variable_versions[variable] = variable_versions[variable], 0
+
+        self.variable_actives = {}
+
+        for variable, versions in iterItems(variable_versions):
+            if type(versions) is tuple:
+                version = self.addVariableMergeMultipleTrace(
+                    variable=variable,
+                    traces=(
+                        self.getVariableTrace(variable, versions[0]),
+                        self.getVariableTrace(variable, versions[1]),
+                    ),
+                )
+            else:
+                version = versions
+
+            self.markCurrentVariableTrace(variable, version)
 
     def mergeMultipleBranches(self, collections):
         assert collections
@@ -647,7 +659,7 @@ class TraceCollectionBase(CollectionTracingMixin):
         for collection in collections:
             for variable, version in iterItems(collection.variable_actives):
                 if variable not in variable_versions:
-                    variable_versions[variable] = set([version])
+                    variable_versions[variable] = OrderedSet((version,))
                 else:
                     variable_versions[variable].add(version)
 
@@ -658,54 +670,42 @@ class TraceCollectionBase(CollectionTracingMixin):
 
         self.variable_actives = {}
 
-#         merge_traces = None
-
         for variable, versions in iterItems(variable_versions):
             if len(versions) == 1:
-                version, = versions
+                (version,) = versions
             else:
                 version = self.addVariableMergeMultipleTrace(
-                    variable = variable,
-                    traces   = [
-                        self.getVariableTrace(variable, version)
-                        for version in
-                        versions
-                    ]
+                    variable=variable,
+                    traces=tuple(
+                        self.getVariableTrace(variable, version) for version in versions
+                    ),
                 )
 
-#                 if merge_traces is None:
-#                     merge_traces = [trace_merge]
-#                 else:
-#                     merge_traces.append(trace_merge)
-
             self.markCurrentVariableTrace(variable, version)
 
-        # Return "None", or turn the list into a tuple for memory savings.
-#         return merge_traces and tuple(merge_traces)
-
     def replaceBranch(self, collection_replace):
         self.variable_actives.update(collection_replace.variable_actives)
         collection_replace.variable_actives = None
 
-    def onLoopBreak(self, collection = None):
+    def onLoopBreak(self, collection=None):
         if collection is None:
             collection = self
 
         return self.parent.onLoopBreak(collection)
 
-    def onLoopContinue(self, collection = None):
+    def onLoopContinue(self, collection=None):
         if collection is None:
             collection = self
 
         return self.parent.onLoopContinue(collection)
 
-    def onFunctionReturn(self, collection = None):
+    def onFunctionReturn(self, collection=None):
         if collection is None:
             collection = self
 
         return self.parent.onFunctionReturn(collection)
 
-    def onExceptionRaiseExit(self, raisable_exceptions, collection = None):
+    def onExceptionRaiseExit(self, raisable_exceptions, collection=None):
         if collection is None:
             collection = self
 
@@ -723,13 +723,14 @@ class TraceCollectionBase(CollectionTracingMixin):
     def getExceptionRaiseCollections(self):
         return self.parent.getExceptionRaiseCollections()
 
-    def makeAbortStackContext(self, catch_breaks, catch_continues,
-                              catch_returns, catch_exceptions):
+    def makeAbortStackContext(
+        self, catch_breaks, catch_continues, catch_returns, catch_exceptions
+    ):
         return self.parent.makeAbortStackContext(
-            catch_breaks     = catch_breaks,
-            catch_continues  = catch_continues,
-            catch_returns    = catch_returns,
-            catch_exceptions = catch_exceptions
+            catch_breaks=catch_breaks,
+            catch_continues=catch_continues,
+            catch_returns=catch_returns,
+            catch_exceptions=catch_exceptions,
         )
 
     def onLocalsDictEscaped(self, locals_scope):
@@ -737,9 +738,10 @@ class TraceCollectionBase(CollectionTracingMixin):
 
     def getCompileTimeComputationResult(self, node, computation, description):
         new_node, change_tags, message = getComputationResult(
-            node        = node,
-            computation = computation,
-            description = description
+            node=node,
+            computation=computation,
+            description=description,
+            user_provided=False,
         )
 
         if change_tags == "new_raise":
@@ -748,7 +750,7 @@ class TraceCollectionBase(CollectionTracingMixin):
         return new_node, change_tags, message
 
     def getIteratorNextCount(self, iter_node):
-        return self.value_states.get(iter_node, None)
+        return self.value_states.get(iter_node)
 
     def initIteratorValue(self, iter_node):
         # TODO: More complex state information will be needed eventually.
@@ -767,28 +769,22 @@ class TraceCollectionBase(CollectionTracingMixin):
 
 
 class TraceCollectionBranch(TraceCollectionBase):
-    def __init__(self, name, parent):
-        TraceCollectionBase.__init__(
-            self,
-            owner  = parent.owner,
-            name   = name,
-            parent = parent
-        )
+    __slots__ = ()
 
+    def __init__(self, name, parent):
+        TraceCollectionBase.__init__(self, owner=parent.owner, name=name, parent=parent)
+
+        # Detach from others
         self.variable_actives = dict(parent.variable_actives)
 
     def computeBranch(self, branch):
         if branch.isStatementsSequence():
-            result = branch.computeStatementsSequence(
-                trace_collection = self
-            )
+            result = branch.computeStatementsSequence(trace_collection=self)
 
             if result is not branch:
                 branch.parent.replaceChild(branch, result)
         else:
-            self.onExpression(
-                expression = branch
-            )
+            self.onExpression(expression=branch)
 
     def initVariable(self, variable):
         variable_trace = self.parent.initVariable(variable)
@@ -814,21 +810,32 @@ class TraceCollectionBranch(TraceCollectionBase):
         Tracing.printSeparator()
 
 
-class TraceCollectionFunction(CollectionStartpointMixin,
-                              TraceCollectionBase):
+class TraceCollectionFunction(CollectionStartpointMixin, TraceCollectionBase):
+    __slots__ = (
+        "variable_versions",
+        "variable_traces",
+        "break_collections",
+        "continue_collections",
+        "return_collections",
+        "exception_collections",
+        "outline_functions",
+    )
+
     def __init__(self, parent, function_body):
-        assert function_body.isExpressionFunctionBody() or \
-               function_body.isExpressionGeneratorObjectBody() or \
-               function_body.isExpressionCoroutineObjectBody() or \
-               function_body.isExpressionAsyncgenObjectBody(), function_body
+        assert (
+            function_body.isExpressionFunctionBody()
+            or function_body.isExpressionGeneratorObjectBody()
+            or function_body.isExpressionCoroutineObjectBody()
+            or function_body.isExpressionAsyncgenObjectBody()
+        ), function_body
 
         CollectionStartpointMixin.__init__(self)
 
         TraceCollectionBase.__init__(
             self,
-            owner  = function_body,
-            name   = "collection_" + function_body.getCodeName(),
-            parent = parent
+            owner=function_body,
+            name="collection_" + function_body.getCodeName(),
+            parent=parent,
         )
 
         if function_body.isExpressionFunctionBody():
@@ -841,7 +848,7 @@ class TraceCollectionFunction(CollectionStartpointMixin,
             self.variable_actives[closure_variable] = 0
 
         # TODO: Have special function type for exec functions stuff.
-        locals_scope = function_body.getFunctionLocalsScope()
+        locals_scope = function_body.getLocalsScope()
 
         if locals_scope is not None:
             if not locals_scope.isMarkedForPropagation():
@@ -851,35 +858,34 @@ class TraceCollectionFunction(CollectionStartpointMixin,
                 function_body.locals_scope = None
 
 
-class TraceCollectionModule(CollectionStartpointMixin,
-                            TraceCollectionBase):
+class TraceCollectionModule(CollectionStartpointMixin, TraceCollectionBase):
+    __slots__ = (
+        "variable_versions",
+        "variable_traces",
+        "break_collections",
+        "continue_collections",
+        "return_collections",
+        "exception_collections",
+        "outline_functions",
+    )
+
     def __init__(self, module):
         assert module.isCompiledPythonModule(), module
 
         CollectionStartpointMixin.__init__(self)
 
         TraceCollectionBase.__init__(
-            self,
-            owner  = module,
-            name   = "module:" + module.getFullName(),
-            parent = None
+            self, owner=module, name="module:" + module.getFullName(), parent=None
         )
 
-        self.used_modules = OrderedSet()
-
     def onUsedModule(self, module_name, module_relpath):
-        assert type(module_name) is str, module_name
+        assert type(module_name) is ModuleName, module_name
 
         # TODO: Make users provide this through a method that has already
         # done this.
         module_relpath = relpath(module_relpath)
 
-        self.used_modules.add(
-            (module_name, module_relpath)
-        )
+        self.owner.addUsedModule((module_name, module_relpath))
 
         module = getImportedModuleByNameAndPath(module_name, module_relpath)
         addUsedModule(module)
-
-    def getUsedModules(self):
-        return self.used_modules
