@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -48,7 +48,10 @@ from nuitka.nodes.DictionaryNodes import (
     StatementDictOperationSet,
     StatementDictOperationSetKeyValue,
 )
-from nuitka.nodes.FrameNodes import StatementsFrameFunction, StatementsFrameGenerator
+from nuitka.nodes.FrameNodes import (
+    StatementsFrameFunction,
+    StatementsFrameGenerator,
+)
 from nuitka.nodes.FunctionNodes import ExpressionFunctionRef
 from nuitka.nodes.GeneratorNodes import (
     ExpressionGeneratorObjectBody,
@@ -57,11 +60,20 @@ from nuitka.nodes.GeneratorNodes import (
 )
 from nuitka.nodes.LoopNodes import StatementLoop, StatementLoopBreak
 from nuitka.nodes.NodeMakingHelpers import makeVariableRefNode
-from nuitka.nodes.OutlineNodes import ExpressionOutlineBody, ExpressionOutlineFunction
+from nuitka.nodes.OutlineNodes import (
+    ExpressionOutlineBody,
+    ExpressionOutlineFunction,
+)
 from nuitka.nodes.ReturnNodes import StatementReturn
-from nuitka.nodes.StatementNodes import StatementExpressionOnly, StatementsSequence
+from nuitka.nodes.StatementNodes import (
+    StatementExpressionOnly,
+    StatementsSequence,
+)
 from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
-from nuitka.nodes.YieldNodes import ExpressionYield, ExpressionYieldFromWaitable
+from nuitka.nodes.YieldNodes import (
+    ExpressionYield,
+    ExpressionYieldFromWaitable,
+)
 from nuitka.PythonVersions import python_version
 
 from .ReformulationAssignmentStatements import buildAssignmentStatements
@@ -86,7 +98,7 @@ def _makeIteratorCreation(provider, qual, for_asyncgen, source_ref):
             source_ref=source_ref,
         )
 
-        if not for_asyncgen or python_version < 370:
+        if not for_asyncgen or python_version < 0x370:
             result = ExpressionYieldFromWaitable(
                 expression=result, source_ref=source_ref
             )
@@ -163,14 +175,16 @@ def _buildPython2ListContraction(provider, node, source_ref):
         source_ref=source_ref.atInternal(),
     )
 
-    function_body.setBody(makeStatementsSequenceFromStatement(statement=statement))
+    function_body.setChild(
+        "body", makeStatementsSequenceFromStatement(statement=statement)
+    )
 
     return function_body
 
 
 def buildListContractionNode(provider, node, source_ref):
     # List contractions are dealt with by general code.
-    if python_version < 300:
+    if python_version < 0x300:
         return _buildPython2ListContraction(
             provider=provider, node=node, source_ref=source_ref
         )
@@ -206,7 +220,7 @@ def buildDictContractionNode(provider, node, source_ref):
         node=node,
         name="<dictcontraction>",
         emit_class=StatementDictOperationSet
-        if python_version < 380
+        if python_version < 0x380
         else StatementDictOperationSetKeyValue,
         start_value={},
         source_ref=source_ref,
@@ -230,6 +244,7 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
         co_name="<genexpr>",
         co_kind="Generator",
         co_varnames=(".0",),
+        co_freevars=(),
         co_argcount=1,
         co_posonlyargcount=0,
         co_kwonlyargcount=0,
@@ -240,10 +255,14 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
         future_spec=parent_module.getFutureSpec(),
     )
 
-    if python_version < 370:
-        is_async = any(getattr(qual, "is_async", 0) for qual in node.generators)
-    else:
-        is_async = detectFunctionBodyKind(nodes=[node])[0] in ("Asyncgen", "Coroutine")
+    is_async = any(getattr(qual, "is_async", 0) for qual in node.generators)
+
+    # Some of the newly allowed stuff in 3.7 fails to set the async flag.
+    if not is_async and python_version >= 0x370:
+        is_async = detectFunctionBodyKind(nodes=node.generators)[0] in (
+            "Asyncgen",
+            "Coroutine",
+        )
 
     if is_async:
         code_body = ExpressionAsyncgenObjectBody(
@@ -268,7 +287,8 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
 
         maker_class = ExpressionMakeGeneratorObject
 
-    function_body.setBody(
+    function_body.setChild(
+        "body",
         makeStatementsSequenceFromStatements(
             StatementAssignmentVariable(
                 variable=iter_tmp,
@@ -296,7 +316,7 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
                 ),
                 source_ref=source_ref,
             ),
-        )
+        ),
     )
 
     statements, release_statements = _buildContractionBodyNode(
@@ -325,14 +345,15 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
         ),
     )
 
-    code_body.setBody(
+    code_body.setChild(
+        "body",
         makeStatementsSequenceFromStatement(
             statement=StatementsFrameGenerator(
                 statements=mergeStatements(statements, False),
                 code_object=code_object,
                 source_ref=source_ref,
             )
-        )
+        ),
     )
 
     return function_body
@@ -383,7 +404,7 @@ def _buildContractionBodyNode(
             )
         )
 
-    if for_asyncgen and python_version >= 370 and node.generators[0].is_async:
+    if for_asyncgen and python_version >= 0x370 and node.generators[0].is_async:
         statements.append(
             StatementAssignmentVariable(
                 variable=iter_tmp,
@@ -460,7 +481,7 @@ def _buildContractionBodyNode(
         if qual is node.generators[0]:
             iterator_ref = makeVariableRefNode(variable=iter_tmp, source_ref=source_ref)
 
-            if for_asyncgen and python_version >= 370:
+            if for_asyncgen and python_version >= 0x370:
                 iterator_ref = ExpressionYieldFromWaitable(
                     expression=iterator_ref, source_ref=source_ref
                 )
@@ -539,7 +560,7 @@ def _buildContractionBodyNode(
 
         nested_statements.append(
             StatementLoop(
-                body=StatementsSequence(
+                loop_body=StatementsSequence(
                     statements=mergeStatements(loop_statements), source_ref=source_ref
                 ),
                 source_ref=source_ref,
@@ -626,7 +647,7 @@ def _buildContractionNode(provider, node, name, emit_class, start_value, source_
         ),
     )
 
-    if python_version < 300:
+    if python_version < 0x300:
         body = makeStatementsSequenceFromStatements(assign_iter_statement, statements)
     else:
         parent_module = provider.getParentModule()
@@ -635,6 +656,7 @@ def _buildContractionNode(provider, node, name, emit_class, start_value, source_
             co_name=name,
             co_kind="Function",
             co_varnames=(),
+            co_freevars=(),
             co_argcount=1,
             co_posonlyargcount=0,
             co_kwonlyargcount=0,
@@ -654,6 +676,6 @@ def _buildContractionNode(provider, node, name, emit_class, start_value, source_
             ),
         )
 
-    function_body.setBody(body)
+    function_body.setChild("body", body)
 
     return function_body
