@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,6 +29,7 @@ from nuitka import SourceCodeReferences
 from nuitka.__past__ import unicode  # pylint: disable=I0021,redefined-builtin
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version, python_version_str
+from nuitka.Tracing import general
 from nuitka.utils.Shebang import getShebangFromSource, parseShebang
 from nuitka.utils.Utils import getOS
 
@@ -39,7 +40,7 @@ def _readSourceCodeFromFilename3(source_filename):
     # Only using this for Python3, for Python2 it's too buggy.
     import tokenize
 
-    with tokenize.open(source_filename) as source_file:   # @UndefinedVariable
+    with tokenize.open(source_filename) as source_file:
         return source_file.read()
 
 
@@ -49,7 +50,8 @@ def _detectEncoding2(source_file):
 
     line1 = source_file.readline()
 
-    if line1.startswith(b'\xef\xbb\xbf'):
+    if line1.startswith(b"\xef\xbb\xbf"):
+        # BOM marker makes it clear.
         encoding = "utf-8"
     else:
         line1_match = re.search(b"coding[:=]\\s*([-\\w.]+)", line1)
@@ -81,7 +83,7 @@ def _readSourceCodeFromFilename2(source_filename):
             try:
                 _source_code = source_code.decode(encoding)
             except UnicodeDecodeError as e:
-                lines = source_code.split('\n')
+                lines = source_code.split("\n")
                 so_far = 0
 
                 for count, line in enumerate(lines):
@@ -94,35 +96,33 @@ def _readSourceCodeFromFilename2(source_filename):
                     count = -1
 
                 wrong_byte = re.search(
-                    "byte 0x([a-f0-9]{2}) in position",
-                    str(e)
+                    "byte 0x([a-f0-9]{2}) in position", str(e)
                 ).group(1)
 
                 raiseSyntaxError(
                     """\
 Non-ASCII character '\\x%s' in file %s on line %d, but no encoding declared; \
-see http://python.org/dev/peps/pep-0263/ for details""" % (
-                        wrong_byte,
-                        source_filename,
-                        count+1,
-                    ),
+see http://python.org/dev/peps/pep-0263/ for details"""
+                    % (wrong_byte, source_filename, count + 1),
                     SourceCodeReferences.fromFilename(source_filename).atLineNumber(
-                        count+1
+                        count + 1
                     ),
-                    display_line = False
+                    display_line=False,
                 )
 
     return source_code
 
 
 def readSourceCodeFromFilename(module_name, source_filename):
-    if python_version < 300:
+    if python_version < 0x300:
         source_code = _readSourceCodeFromFilename2(source_filename)
     else:
         source_code = _readSourceCodeFromFilename3(source_filename)
 
-    # Allow plug-ins to mess with source code.
-    source_code = Plugins.onModuleSourceCode(module_name, source_code)
+    # Allow plug-ins to mess with source code, test framework usages
+    # will pass None for module name.
+    if module_name is not None:
+        source_code = Plugins.onModuleSourceCode(module_name, source_code)
 
     return source_code
 
@@ -139,37 +139,44 @@ def checkPythonVersionFromCode(source_code):
             try:
                 if os.path.samefile(sys.executable, binary):
                     return True
-            except OSError: # Might not exist
+            except OSError:  # Might not exist
                 pass
 
         basename = os.path.basename(binary)
 
         # Not sure if we should do that.
         if basename == "python":
-            result = python_version < 300
+            result = python_version < 0x300
         elif basename == "python3":
-            result = python_version > 300
+            result = python_version > 0x300
         elif basename == "python2":
-            result = python_version < 300
+            result = python_version < 0x300
         elif basename == "python2.7":
-            result = python_version < 300
+            result = python_version < 0x300
         elif basename == "python2.6":
-            result = python_version < 270
+            result = python_version < 0x270
         elif basename == "python3.2":
-            result = 330 > python_version >= 300
+            result = 0x330 > python_version >= 0x300
         elif basename == "python3.3":
-            result = 340 > python_version >= 330
+            result = 0x340 > python_version >= 0x330
         elif basename == "python3.4":
-            result = 350 > python_version >= 340
+            result = 0x350 > python_version >= 0x340
         elif basename == "python3.5":
-            result = 360 > python_version >= 350
+            result = 0x360 > python_version >= 0x350
         elif basename == "python3.6":
-            result = 370 > python_version >= 360
+            result = 0x370 > python_version >= 0x360
+        elif basename == "python3.7":
+            result = 0x380 > python_version >= 0x370
+        elif basename == "python3.8":
+            result = 0x390 > python_version >= 0x380
+        elif basename == "python3.9":
+            result = 0x3A0 > python_version >= 0x390
         else:
             result = None
 
         if result is False:
-            sys.exit("""\
+            general.sysexit(
+                """\
 The program you compiled wants to be run with: %s.
 
 Nuitka is currently running with Python version '%s', which seems to not
@@ -177,13 +184,14 @@ match that. Nuitka cannot guess the Python version of your source code. You
 therefore might want to specify: '%s -m nuitka'.
 
 That will make use the correct Python version for Nuitka.
-""" % (shebang, python_version_str, binary)
+"""
+                % (shebang, python_version_str, binary)
             )
 
 
 def readSourceLine(source_ref):
     import linecache
+
     return linecache.getline(
-        filename = source_ref.getFilename(),
-        lineno   = source_ref.getLineNumber()
+        filename=source_ref.getFilename(), lineno=source_ref.getLineNumber()
     )
