@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,10 +25,10 @@ binaries (needed for exec) and run them capturing outputs.
 import os
 import shutil
 import subprocess
-import sys
 from contextlib import contextmanager
 
 from nuitka.PythonVersions import python_version
+from nuitka.Tracing import general
 
 from .Utils import getArchitecture, getOS, isWin32Windows
 
@@ -67,7 +67,7 @@ def callExec(args):
 def getExecutablePath(filename):
     """ Find an execute in PATH environment. """
 
-    if python_version >= 300:
+    if python_version >= 0x300:
         return shutil.which(filename)
     else:
         # Append ".exe" suffix  on Windows if not already present.
@@ -171,7 +171,9 @@ def check_call(*popenargs, **kwargs):
     try:
         subprocess.check_call(*popenargs, **kwargs)
     except OSError:
-        sys.exit("Error, failed to execute '%s'. Is it installed?" % popenargs[0])
+        general.sysexit(
+            "Error, failed to execute '%s'. Is it installed?" % popenargs[0]
+        )
 
 
 @contextmanager
@@ -201,6 +203,8 @@ def withEnvironmentPathAdded(env_var_name, *paths):
 
 @contextmanager
 def withEnvironmentVarOverriden(env_var_name, value):
+    """ Change an environment and restore it after context. """
+
     if env_var_name in os.environ:
         old_value = os.environ[env_var_name]
     else:
@@ -220,6 +224,34 @@ def withEnvironmentVarOverriden(env_var_name, value):
         os.environ[env_var_name] = old_value
 
 
+@contextmanager
+def withEnvironmentVarsOverriden(mapping):
+    """ Change multiple environment variables and restore them after context. """
+
+    old_values = {}
+
+    for env_var_name, value in mapping.items():
+
+        if env_var_name in os.environ:
+            old_values[env_var_name] = os.environ[env_var_name]
+        else:
+            old_values[env_var_name] = None
+
+        if value is not None:
+            os.environ[env_var_name] = value
+        elif old_values[env_var_name] is not None:
+            del os.environ[env_var_name]
+
+    yield
+
+    for env_var_name, value in mapping.items():
+        if old_values[env_var_name] is None:
+            if value is not None:
+                del os.environ[env_var_name]
+        else:
+            os.environ[env_var_name] = old_values[env_var_name]
+
+
 def wrapCommandForDebuggerForExec(*args):
     """Wrap a command for system debugger to call exec
 
@@ -237,7 +269,7 @@ def wrapCommandForDebuggerForExec(*args):
     lldb_path = getExecutablePath("lldb")
 
     if gdb_path is None and lldb_path is None:
-        sys.exit("Error, no 'gdb' or 'lldb' binary found in path.")
+        general.sysexit("Error, no 'gdb' or 'lldb' binary found in path.")
 
     if gdb_path is not None:
         args = (gdb_path, "gdb", "-ex=run", "-ex=where", "-ex=quit", "--args") + args
@@ -266,3 +298,69 @@ def wrapCommandForDebuggerForSubprocess(*args):
     args = args[0:1] + args[2:]
 
     return args
+
+
+def getPythonInstallPathWindows(supported, decider=lambda x: True):
+    """Find suitable Python on Windows.
+
+    Go over a list of provided, supported versions, and first try a few
+    guesses for their paths, then look into registry for user or system wide
+    installations.
+
+    The decider may reject a Python, then the search is continued, otherwise
+    it's returned.
+    """
+    # Many cases to deal with, due to arches, caching, etc.
+    # pylint: disable=too-many-branches
+
+    seen = set()
+
+    # Shortcuts for the default installation directories, to avoid going to
+    # registry at all unless necessary. Any Python2 will do for Scons, so it
+    # might be avoided entirely.
+    for search in supported:
+        candidate = r"C:\Python%s\python.exe" % search.replace(".", "")
+
+        if os.path.isfile(candidate):
+            install_dir = os.path.normcase(os.path.dirname(candidate))
+
+            if decider(install_dir):
+                return install_dir
+
+            seen.add(install_dir)
+
+    # Windows only code, pylint: disable=I0021,import-error,undefined-variable
+    if python_version < 0x300:
+        import _winreg as winreg  # pylint: disable=I0021,import-error,no-name-in-module
+    else:
+        import winreg  # pylint: disable=I0021,import-error,no-name-in-module
+
+    for search in supported:
+        for hkey_branch in (winreg.HKEY_LOCAL_MACHINE, winreg.HKEY_CURRENT_USER):
+            for arch_key in (0, winreg.KEY_WOW64_32KEY, winreg.KEY_WOW64_64KEY):
+                for suffix in "", "-32":
+                    try:
+                        key = winreg.OpenKey(
+                            hkey_branch,
+                            r"SOFTWARE\Python\PythonCore\%s%s\InstallPath"
+                            % (search, suffix),
+                            0,
+                            winreg.KEY_READ | arch_key,
+                        )
+
+                        install_dir = os.path.normpath(winreg.QueryValue(key, ""))
+                    except WindowsError:
+                        pass
+                    else:
+                        if install_dir not in seen:
+                            if decider(install_dir):
+                                return install_dir
+
+                            seen.add(install_dir)
+
+
+def getNullOutput():
+    try:
+        return subprocess.NULLDEV
+    except AttributeError:
+        return open(os.devnull, "wb")
