@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,9 +25,7 @@ from .Utils import getOS
 
 
 def getOwnProcessMemoryUsage():
-    """ Memory usage of own process in bytes.
-
-    """
+    """Memory usage of own process in bytes."""
 
     if getOS() == "Windows":
         # adapted from http://code.activestate.com/recipes/578513
@@ -60,9 +58,9 @@ def getOwnProcessMemoryUsage():
 
         counters = PROCESS_MEMORY_COUNTERS_EX()
         rv = GetProcessMemoryInfo(
-            ctypes.windll.kernel32.GetCurrentProcess(),  # @UndefinedVariable
+            ctypes.windll.kernel32.GetCurrentProcess(),
             ctypes.byref(counters),
-            ctypes.sizeof(counters)
+            ctypes.sizeof(counters),
         )
 
         if not rv:
@@ -70,11 +68,10 @@ def getOwnProcessMemoryUsage():
 
         return counters.PrivateUsage
     else:
-        # Posix only code, pylint: disable=I0021,import-error
-        import resource  # @UnresolvedImport
+        import resource  # Posix only code, pylint: disable=I0021,import-error
 
         # The value is from "getrusage", which has OS dependent scaling, at least
-        # MacOS and Linux are different. Others maybe too.
+        # macOS and Linux are different. Others maybe too.
         if getOS() == "Darwin":
             factor = 1
         else:
@@ -83,25 +80,16 @@ def getOwnProcessMemoryUsage():
         return resource.getrusage(resource.RUSAGE_SELF).ru_maxrss * factor
 
 
-def getHumanReadableProcessMemoryUsage(value = None):
+def getHumanReadableProcessMemoryUsage(value=None):
     if value is None:
         value = getOwnProcessMemoryUsage()
 
-    if abs(value) < 1024*1014:
-        return "%.2f KB (%d bytes)" % (
-            value / 1024.0,
-            value
-        )
-    elif abs(value) < 1024*1014*1024:
-        return "%.2f MB (%d bytes)" % (
-            value / (1024*1024.0),
-            value
-        )
-    elif abs(value) < 1024*1014*1024*1024:
-        return "%.2f GB (%d bytes)" % (
-            value / (1024*1024*1024.0),
-            value
-        )
+    if abs(value) < 1024 * 1014:
+        return "%.2f KB (%d bytes)" % (value / 1024.0, value)
+    elif abs(value) < 1024 * 1014 * 1024:
+        return "%.2f MB (%d bytes)" % (value / (1024 * 1024.0), value)
+    elif abs(value) < 1024 * 1014 * 1024 * 1024:
+        return "%.2f GB (%d bytes)" % (value / (1024 * 1024 * 1024.0), value)
     else:
         return "%d bytes" % value
 
@@ -120,7 +108,7 @@ class MemoryWatch(object):
 
 def startMemoryTracing():
     try:
-        import tracemalloc  # @UnresolvedImport
+        import tracemalloc
     except ImportError:
         pass
     else:
@@ -129,7 +117,7 @@ def startMemoryTracing():
 
 def showMemoryTrace():
     try:
-        import tracemalloc  # @UnresolvedImport
+        import tracemalloc
     except ImportError:
         pass
     else:
