@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -24,9 +24,11 @@ except ImportError:
 
 # This kind of built-in overload will have to work.
 
-def len(x):  # @ReservedAssignment
+
+def len(x):
     print("Private built-in called with argument", repr(x))
 
     return _len(x)
 
+
 print("Calling built-in len", len(range(9)))
