@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -15,33 +15,46 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
+""" Tests to cover default parameter behaviours.
+
+"""
+
 from __future__ import print_function
 
+# pylint: disable=dangerous-default-value,unused-argument
+
 module_level = 1
 
-def defaultValueTest1(no_default, some_default_constant = 1):
+
+def defaultValueTest1(no_default, some_default_constant=1):
     return some_default_constant
 
-def defaultValueTest2(no_default, some_default_computed = module_level*2):
+
+def defaultValueTest2(no_default, some_default_computed=module_level * 2):
     local_var = no_default
     return local_var, some_default_computed
 
-def defaultValueTest3(no_default, func_defaulted = defaultValueTest1(module_level)):
-    return [ func_defaulted for _i in range(8) ]
 
-def defaultValueTest4(no_default, funced_defaulted = lambda x: x**2):
+def defaultValueTest3(no_default, func_defaulted=defaultValueTest1(module_level)):
+    return [func_defaulted for _i in range(8)]
+
+
+def defaultValueTest4(no_default, funced_defaulted=lambda x: x ** 2):
     c = 1
     d = 1
-    return ( i+c+d for i in range(8) )
+    return (i + c + d for i in range(8))
 
-def defaultValueTest5(no_default, tuple_defaulted = (1,2,3)):
+
+def defaultValueTest5(no_default, tuple_defaulted=(1, 2, 3)):
     return tuple_defaulted
 
-def defaultValueTest6(no_default, list_defaulted = [1,2,3]):
+
+def defaultValueTest6(no_default, list_defaulted=[1, 2, 3]):
     list_defaulted.append(5)
 
     return list_defaulted
 
+
 print(defaultValueTest1("ignored"))
 
 # The change of the default variable doesn't influence the default
@@ -56,12 +69,12 @@ print(list(defaultValueTest4("unused")))
 
 print(defaultValueTest5("unused"))
 
-print(defaultValueTest6("unused"), end = "")
+print(defaultValueTest6("unused"), end="")
 print(defaultValueTest6("unused"))
 
 print(defaultValueTest6.__defaults__)
 
-defaultValueTest6.func_defaults = ([1,2,3],)
+defaultValueTest6.func_defaults = ([1, 2, 3],)
 print(defaultValueTest6.__defaults__)
 
 print(defaultValueTest6(1))
