@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -43,13 +43,10 @@ import types
 
 from nuitka.tools.testing.Common import (
     async_iterate,
-    checkDebugPython,
     executeReferenceChecked,
     run_async,
 )
 
-checkDebugPython()
-
 
 class AwaitException(Exception):
     pass
