@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -15,14 +15,13 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-
 def someFunctionThatReturnsDeletedValueViaAttributeLookup():
     class C:
         def __getattr__(self, attr_name):
             nonlocal a
             del a
-    c = C()
 
+    c = C()
 
     a = 1
     c.something
@@ -42,8 +41,8 @@ def someFunctionThatReturnsDeletedValueViaAttributeSetting():
         def __setattr__(self, attr_name, value):
             nonlocal a
             del a
-    c = C()
 
+    c = C()
 
     a = 1
     c.something = 1
@@ -57,6 +56,7 @@ except UnboundLocalError:
 else:
     print("Ouch.!")
 
+
 def someFunctionThatReturnsDeletedValueViaAttributeDel():
     class C:
         def __delattr__(self, attr_name):
@@ -85,6 +85,7 @@ def someFunctionThatReturnsDeletedValueViaItemLookup():
         def __getitem__(self, attr_name):
             nonlocal a
             del a
+
     c = C()
 
     a = 1
@@ -99,13 +100,14 @@ except UnboundLocalError:
 else:
     print("Ouch.!")
 
+
 def someFunctionThatReturnsDeletedValueViaItemSetting():
     class C:
         def __setitem__(self, attr_name, value):
             nonlocal a
             del a
-    c = C()
 
+    c = C()
 
     a = 1
     c[2] = 3
@@ -125,8 +127,8 @@ def someFunctionThatReturnsDeletedValueViaItemDel():
         def __delitem__(self, attr_name):
             nonlocal a
             del a
-    c = C()
 
+    c = C()
 
     a = 1
     del c[2]
@@ -146,8 +148,8 @@ def someFunctionThatReturnsDeletedValueViaCall():
         def __call__(self):
             nonlocal a
             del a
-    c = C()
 
+    c = C()
 
     a = 1
     c()
@@ -167,8 +169,8 @@ def someFunctionThatReturnsDeletedValueViaAdd():
         def __add__(self, other):
             nonlocal a
             del a
-    c = C()
 
+    c = C()
 
     a = 1
     c + 1
@@ -188,8 +190,8 @@ def someFunctionThatReturnsDeletedValueViaSub():
         def __sub__(self, other):
             nonlocal a
             del a
-    c = C()
 
+    c = C()
 
     a = 1
     c - 1
@@ -211,8 +213,8 @@ def someFunctionThatReturnsDeletedValueViaMul():
             del a
 
             return 7
-    c = C()
 
+    c = C()
 
     a = 1
     c * 1
@@ -234,8 +236,8 @@ def someFunctionThatReturnsDeletedValueViaRemainder():
             del a
 
             return 7
-    c = C()
 
+    c = C()
 
     a = 1
     c % 1
@@ -257,8 +259,8 @@ def someFunctionThatReturnsDeletedValueViaDivmod():
             del a
 
             return 7
-    c = C()
 
+    c = C()
 
     a = 1
     divmod(c, 1)
@@ -280,8 +282,8 @@ def someFunctionThatReturnsDeletedValueViaPower():
             del a
 
             return 7
-    c = C()
 
+    c = C()
 
     a = 1
     c ** 1
@@ -303,8 +305,8 @@ def someFunctionThatReturnsDeletedValueViaUnaryMinus():
             del a
 
             return 7
-    c = C()
 
+    c = C()
 
     a = 1
     -c
@@ -326,8 +328,8 @@ def someFunctionThatReturnsDeletedValueViaUnaryPlus():
             del a
 
             return 7
-    c = C()
 
+    c = C()
 
     a = 1
     +c
@@ -352,7 +354,6 @@ def someFunctionThatReturnsDeletedValueViaNot():
 
     c = C()
 
-
     a = 1
     not c
     return a
@@ -578,6 +579,7 @@ def someFunctionThatReturnsDeletedValueViaInplaceAdd():
         def __iadd__(self, other):
             nonlocal a
             del a
+
     c = C()
 
     a = 1
@@ -598,6 +600,7 @@ def someFunctionThatReturnsDeletedValueViaInplaceSub():
         def __isub__(self, other):
             nonlocal a
             del a
+
     c = C()
 
     a = 1
@@ -618,6 +621,7 @@ def someFunctionThatReturnsDeletedValueViaInplaceMul():
         def __imul__(self, other):
             nonlocal a
             del a
+
     c = C()
 
     a = 1
@@ -638,6 +642,7 @@ def someFunctionThatReturnsDeletedValueViaInplaceRemainder():
         def __imod__(self, other):
             nonlocal a
             del a
+
     c = C()
 
     a = 1
@@ -653,7 +658,188 @@ else:
     print("Ouch.!")
 
 
-# TODO: More in-place operations are missing here.
+def someFunctionThatReturnsDeletedValueViaInplacePower():
+    class C:
+        def __ipow__(self, other):
+            nonlocal a
+            del a
+
+            return 7
+
+    c = C()
+
+    a = 1
+    c **= 1
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaInplacePower()
+except UnboundLocalError:
+    print("OK, object inplace power correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaInplaceAnd():
+    class C:
+        def __iand__(self, other):
+            nonlocal a
+            del a
+
+            return False
+
+    c = C()
+
+    a = 1
+    c &= 1
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaInplaceAnd()
+except UnboundLocalError:
+    print("OK, object inplace and correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaInplaceFloordiv():
+    class C:
+        def __ifloordiv__(self, other):
+            nonlocal a
+            del a
+
+            return 7
+
+    c = C()
+
+    a = 1
+    c //= 1
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaInplaceFloordiv()
+except UnboundLocalError:
+    print("OK, object inplace floordiv correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaInplaceLshift():
+    class C:
+        def __ilshift__(self, other):
+            nonlocal a
+            del a
+
+            return False
+
+    c = C()
+
+    a = 1
+    c <<= 1
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaInplaceLshift()
+except UnboundLocalError:
+    print("OK, object inplace lshift correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaInplaceRshift():
+    class C:
+        def __irshift__(self, other):
+            nonlocal a
+            del a
+
+            return False
+
+    c = C()
+
+    a = 1
+    c >>= 1
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaInplaceRshift()
+except UnboundLocalError:
+    print("OK, object inplace rshift correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaInplaceOr():
+    class C:
+        def __ior__(self, other):
+            nonlocal a
+            del a
+
+            return False
+
+    c = C()
+
+    a = 1
+    c |= 1
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaInplaceOr()
+except UnboundLocalError:
+    print("OK, object inplace or correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaInplaceTrueDiv():
+    class C:
+        def __itruediv__(self, other):
+            nonlocal a
+            del a
+
+            return 7
+
+    c = C()
+
+    a = 1
+    c /= 1
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaInplaceTrueDiv()
+except UnboundLocalError:
+    print("OK, object inplace truediv correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaInplaceXor():
+    class C:
+        def __ixor__(self, other):
+            nonlocal a
+            del a
+
+            return False
+
+    c = C()
+
+    a = 1
+    c ^= 1
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaInplaceXor()
+except UnboundLocalError:
+    print("OK, object inplace xor correctly deleted an item.")
+else:
+    print("Ouch.!")
 
 
 def someFunctionThatReturnsDeletedValueViaIndex():
@@ -702,12 +888,6 @@ else:
     print("Ouch.!")
 
 
-# TODO: The matrix multiply needs a dedicated test too.
-
-
-# TODO: There is a whole lot more operations to cover.
-
-
 def someFunctionThatReturnsDeletedValueViaRepr():
     class C:
         def __repr__(self):
@@ -718,7 +898,6 @@ def someFunctionThatReturnsDeletedValueViaRepr():
 
     c = C()
 
-
     a = 1
     repr(c)
     return a
@@ -742,7 +921,6 @@ def someFunctionThatReturnsDeletedValueViaStr():
 
     c = C()
 
-
     a = 1
     str(c)
     return a
@@ -766,7 +944,6 @@ def someFunctionThatReturnsDeletedValueViaCompare():
 
     c = C()
 
-
     a = 1
     c < None
     return a
@@ -790,7 +967,6 @@ def someFunctionThatReturnsDeletedValueViaDel():
 
     c = C()
 
-
     a = 1
     del c
     return a
@@ -873,4 +1049,263 @@ else:
     print("Ouch.!")
 
 
+def someFunctionThatReturnsDeletedValueViaEq():
+    class C:
+        def __eq__(self, other):
+            nonlocal a
+            del a
+            return False
+
+    c = C()
+
+    a = 1
+    c == 1
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaEq()
+except UnboundLocalError:
+    print("OK, object eq correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaLe():
+    class C:
+        def __le__(self, other):
+            nonlocal a
+            del a
+            return False
+
+    c = C()
+
+    a = 1
+    c <= 1
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaEq()
+except UnboundLocalError:
+    print("OK, object le correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaGt():
+    class C:
+        def __gt__(self, other):
+            nonlocal a
+            del a
+            return False
+
+    c = C()
+
+    a = 1
+    c > 1
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaEq()
+except UnboundLocalError:
+    print("OK, object gt correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaGe():
+    class C:
+        def __ge__(self, other):
+            nonlocal a
+            del a
+            return False
+
+    c = C()
+
+    a = 1
+    c >= 1
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaEq()
+except UnboundLocalError:
+    print("OK, object ge correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaNe():
+    class C:
+        def __ne__(self, other):
+            nonlocal a
+            del a
+            return False
+
+    c = C()
+
+    a = 1
+    c != 1
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaEq()
+except UnboundLocalError:
+    print("OK, object ne correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaContains():
+    class C:
+        def __contains__(self, item):
+            nonlocal a
+            del a
+            return False
+
+    c = C()
+
+    a = 1
+    1 in c
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaContains()
+except UnboundLocalError:
+    print("OK, object contains correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaInit():
+    class C:
+        def __init__(self):
+            nonlocal a
+            del a
+
+    a = 1
+    c = C()
+
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaInit()
+except UnboundLocalError:
+    print("OK, object init correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaNew():
+    class C:
+        def __new__(self):
+            nonlocal a
+            del a
+
+    a = 1
+    c = C()
+
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaNew()
+except UnboundLocalError:
+    print("OK, object new correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaDir():
+    class C:
+        def __dir__(self):
+            nonlocal a
+            del a
+            return []
+
+    c = C()
+
+    a = 1
+    dir(c)
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaDir()
+except UnboundLocalError:
+    print("OK, object dir correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaReversed():
+    class C:
+        def __reversed__(self):
+            nonlocal a
+            del a
+            return None
+
+    a = 1
+    c = C()
+
+    reversed(c)
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaReversed()
+except UnboundLocalError:
+    print("OK, object reversed correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaFormat():
+    class C:
+        def __format__(self, string):
+            nonlocal a
+            del a
+            return "formated string"
+
+    c = C()
+
+    a = 1
+    format(c, "some string")
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaFormat()
+except UnboundLocalError:
+    print("OK, object format correctly deleted an item.")
+else:
+    print("Ouch.!")
+
+
+def someFunctionThatReturnsDeletedValueViaAbs():
+    class C:
+        def __abs__(self):
+            nonlocal a
+            del a
+            return abs(10)
+
+    a = 1
+    c = C()
+
+    abs(c)
+    return a
+
+
+try:
+    someFunctionThatReturnsDeletedValueViaAbs()
+except UnboundLocalError:
+    print("OK, object abs correctly deleted an item.")
+else:
+    print("Ouch.!")
+
 # TODO: There must be way more than these.
