@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -19,9 +19,11 @@
 
 from __future__ import print_function
 
+
 def one():
     return 1
 
+
 def tryScope1(x):
     try:
         try:
@@ -38,10 +40,12 @@ def tryScope1(x):
     else:
         print("No exception occurred")
 
+
 tryScope1(1)
-print('*' * 20)
+print("*" * 20)
 tryScope1([1])
 
+
 def tryScope2(x, someExceptionClass):
     try:
         x += 1
@@ -50,6 +54,7 @@ def tryScope2(x, someExceptionClass):
     else:
         print("No exception occurred")
 
+
 def tryScope3(x):
     if x:
         try:
@@ -60,18 +65,19 @@ def tryScope3(x):
         print("Not taken")
 
 
-print('*' * 20)
+print("*" * 20)
 
 tryScope2(1, TypeError)
 tryScope2([1], TypeError)
 
-print('*' * 20)
+print("*" * 20)
 
 tryScope3(1)
 tryScope3([1])
 tryScope3([])
 
-print('*' * 20)
+print("*" * 20)
+
 
 def tryScope4(x):
     try:
@@ -83,19 +89,22 @@ def tryScope4(x):
     finally:
         print("finally obeyed")
 
+
 tryScope4(1)
 tryScope4([1])
 
+
 def tryScope5():
     import sys
 
     print("Exception info is initially", sys.exc_info())
     try:
         try:
-            undefined_global += 1  # @UndefinedVariable
+            undefined_global += 1
         finally:
             print("Exception info in 'finally' clause is", sys.exc_info())
     except:
         pass
 
+
 tryScope5()
