@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -15,7 +15,10 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
+""" Module that provides a function for import.
+
+"""
 
 
-def foo():
+def someFunction():
     return "hi"
