@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -17,13 +17,13 @@
 #
 print("Main importing nearby package")
 import pyexpat
+from some_package import normal_importing, star_importing
+
 try:
-    print(pyexpat.defined_in_pyexpat )
+    print(pyexpat.defined_in_pyexpat)
 except AttributeError:
     print("Must be Python3, where absolute imports are default.")
-print( "Main importing from package doing star import" )
-from some_package import star_importing
-print( "Main importing from package doing normal import" )
-from some_package import normal_importing
+print("Main importing from package doing star import")
+print("Main importing from package doing normal import")
 
-print( "Done." )
+print("Done.")
