@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -249,8 +249,8 @@ def executePASS1():
         copyTree(scons_inline_copy_path, os.path.join("nuitka", "build", "inline_copy"))
 
     shutil.copyfile(
-        os.path.join(base_dir, "nuitka", "build", "SingleExe.scons"),
-        os.path.join("nuitka", "build", "SingleExe.scons"),
+        os.path.join(base_dir, "nuitka", "build", "Backend.scons"),
+        os.path.join("nuitka", "build", "Backend.scons"),
     )
     copyTree(
         os.path.join(base_dir, "nuitka", "build", "static_src"),
@@ -427,7 +427,7 @@ def executePASS5():
 
 
 def main():
-    _python_version = setup(needs_io_encoding=True)
+    setup(needs_io_encoding=True)
 
     executePASS1()
     executePASS2()
@@ -438,9 +438,6 @@ def main():
 
     executePASS5()
 
-    os.unlink(os.path.join(tmp_dir, "nuitka" + exe_suffix))
-    os.rmdir(tmp_dir)
-
 
 if __name__ == "__main__":
     main()
