@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Software where
@@ -17,7 +17,8 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-import os, ctypes
+import ctypes
+import os
 
 if os.name == "nt":
     # adapted from http://code.activestate.com/recipes/578513
@@ -52,7 +53,7 @@ if os.name == "nt":
     rv = GetProcessMemoryInfo(
         ctypes.windll.kernel32.GetCurrentProcess(),
         ctypes.byref(counters),
-        ctypes.sizeof(counters)
+        ctypes.sizeof(counters),
     )
 
     if not rv:
