@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Software where
@@ -17,6 +17,12 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
+""" pygtk standalone basic test.
+
+"""
+
+from __future__ import print_function
+
 import warnings
 
 warnings.filterwarnings("ignore", "")
