@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Software where
@@ -17,31 +17,36 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-
 # This test is using signals and will only work if PySide properly accepts
 # compiled functions as callables.
 
 from __future__ import print_function
 
-from PySide.QtCore import Slot, Signal, QObject, QMetaObject
+from PySide.QtCore import QMetaObject, QObject, Signal, Slot
+
+# nuitka-skip-unless-imports: PySide.QtCore
+
 
 class Communicate(QObject):
     speak = Signal(int)
-    def __init__(self,name = "",parent = None):
-        QObject.__init__(self,parent)
+
+    def __init__(self, name="", parent=None):
+        QObject.__init__(self, parent)
         self.setObjectName(name)
 
+
 class Speaker(QObject):
     @Slot(int)
     def on_communicator_speak(self, stuff):
         print(stuff)
 
+
 speaker = Speaker()
-someone = Communicate(name = "communicator",parent = speaker)
+someone = Communicate(name="communicator", parent=speaker)
 
 QMetaObject.connectSlotsByName(speaker)
 
-print("The answer is:",end = "")
+print("The answer is:", end="")
 # emit  'speak' signal
 someone.speak.emit(42)
 print("Slot should have made output by now.")
