@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Software where
@@ -17,9 +17,9 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-try:
-    import pyexpat
 
-    print(pyexpat.__doc__)
-except ImportError:
-    print("Skipped, no pyexpat module installed.")
+# nuitka-skip-unless-imports: pyexpat
+
+import pyexpat
+
+print(pyexpat.__doc__)
