@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Software where
@@ -43,78 +43,41 @@ sys.path.insert(
 
 # isort:start
 
-import subprocess
-
+from nuitka.PythonVersions import (
+    getPartiallySupportedPythonVersions,
+    getSupportedPythonVersions,
+)
 from nuitka.tools.testing.Common import (
+    checkRequirements,
     compareWithCPython,
     createSearchMode,
     decideFilenameVersionSkip,
+    displayFileContents,
+    displayFolderContents,
     getPythonVendor,
     getRuntimeTraceOfLoadedFiles,
-    hasModule,
-    my_print,
     reportSkip,
     setup,
+    test_logger,
 )
-from nuitka.tree.SourceReading import readSourceCodeFromFilename
 from nuitka.utils.FileOperations import areSamePaths, removeDirectory
+from nuitka.utils.Timing import TimerReport
 from nuitka.utils.Utils import getOS
 
 
-# checks requirements needed to run each test module, according to the specified special comment
-# special comments are in the following formats:
-#     "# nuitka-skip-unless-expression: expression to be evaluated"
-#       OR
-#     "# nuitka-skip-unless-imports: module1,module2,..."
-def checkRequirements(filename):
-    for line in readSourceCodeFromFilename(None, filename).splitlines():
-        if line.startswith("# nuitka-skip-unless-"):
-            if line[21:33] == "expression: ":
-                expression = line[33:]
-                with open(os.devnull, "w") as devnull:
-                    result = subprocess.call(
-                        (
-                            os.environ["PYTHON"],
-                            "-c",
-                            "import sys, os; sys.exit(not bool(%s))" % expression,
-                        ),
-                        stdout=devnull,
-                        stderr=subprocess.STDOUT,
-                    )
-                if result != 0:
-                    return (False, "Expression '%s' evaluated to false" % expression)
-
-            elif line[21:30] == "imports: ":
-                imports_needed = line[30:].rstrip().split(",")
-                for i in imports_needed:
-                    if not hasModule(i):
-                        return (
-                            False,
-                            i
-                            + " not installed for this Python version, but test needs it",
-                        )
-    # default return value
-    return (True, "")
-
-
 def displayError(dirname, filename):
     assert dirname is None
 
-    filename = filename[:-3] + ".dist"
+    dist_path = filename[:-3] + ".dist"
+    displayFolderContents("dist folder", dist_path)
 
-    my_print("Listing of dist folder '%s':" % filename)
-
-    if os.name == "nt":
-        command = "dir /b /s /a:-D %s" % filename
-    else:
-        command = "ls -Rla %s" % filename
-
-    os.system(command)
+    inclusion_log_path = filename[:-3] + ".py.inclusion.log"
+    displayFileContents("inclusion log", inclusion_log_path)
 
 
 def main():
     # Complex stuff, even more should become common code though.
-    # pylint: disable=too-many-branches,too-many-statements
+    # pylint: disable=too-many-branches,too-many-locals,too-many-statements
 
     python_version = setup(needs_io_encoding=True)
 
@@ -130,10 +93,18 @@ def main():
         active = search_mode.consider(dirname=None, filename=filename)
 
         if not active:
-            my_print("Skipping", filename)
+            test_logger.info("Skipping %s" % filename)
             continue
 
-        extra_flags = ["expect_success", "standalone", "remove_output"]
+        extra_flags = [
+            "expect_success",
+            "--standalone",
+            "remove_output",
+            # Cache the CPython results for re-use, they will normally not change.
+            "cpython_cache",
+            # To understand what is slow.
+            "timing",
+        ]
 
         # skip each test if their respective requirements are not met
         requirements_met, error_message = checkRequirements(filename)
@@ -148,7 +119,7 @@ def main():
 
         if "Idna" in filename:
             # For the warnings of Python2.
-            if python_version.startswith("2"):
+            if python_version < (3,):
                 extra_flags.append("ignore_stderr")
 
         if filename == "CtypesUsing.py":
@@ -157,7 +128,7 @@ def main():
         if filename == "GtkUsing.py":
             # Don't test on platforms not supported by current Debian testing, and
             # which should be considered irrelevant by now.
-            if python_version.startswith("2.6"):
+            if python_version < (2, 7):
                 reportSkip("irrelevant Python version", ".", filename)
                 continue
 
@@ -191,6 +162,7 @@ def main():
         if filename == "PandasUsing.py":
             extra_flags.append("plugin_enable:numpy")
             extra_flags.append("plugin_disable:pylint-warnings")
+            extra_flags.append("plugin_disable:qt-plugins")
 
         if filename == "PmwUsing.py":
             extra_flags.append("plugin_enable:pmw-freezer")
@@ -213,7 +185,9 @@ def main():
             extra_flags.append("ignore_warnings")
 
         if filename.startswith(("PySide", "PyQt")):
-            if python_version.startswith("2.6"):
+            # Don't test on platforms not supported by current Debian testing, and
+            # which should be considered irrelevant by now.
+            if python_version < (2, 7):
                 reportSkip("irrelevant Python version", ".", filename)
                 continue
 
@@ -224,7 +198,9 @@ def main():
                 # For the plug-in not used information.
                 extra_flags.append("ignore_warnings")
 
-        my_print("Consider output of recursively compiled program:", filename)
+        test_logger.info(
+            "Consider output of standalone mode compiled program: %s" % filename
+        )
 
         # First compare so we know the program behaves identical.
         compareWithCPython(
@@ -273,7 +249,7 @@ def main():
                 found_glibc_libs.append(dist_filename)
 
         if found_glibc_libs:
-            my_print(
+            test_logger.warning(
                 "Should not ship glibc libraries with the standalone executable (found %s)"
                 % found_glibc_libs
             )
@@ -284,7 +260,10 @@ def main():
         )
 
         # Then use "strace" on the result.
-        loaded_filenames = getRuntimeTraceOfLoadedFiles(binary_filename)
+        with TimerReport(
+            "Determining run time loaded files took %.2f", logger=test_logger
+        ):
+            loaded_filenames = getRuntimeTraceOfLoadedFiles(binary_filename)
 
         current_dir = os.path.normpath(os.getcwd())
         current_dir = os.path.normcase(current_dir)
@@ -519,33 +498,27 @@ def main():
             if loaded_filename.startswith("/var/cache/"):
                 continue
 
+            lib_prefix_dir = "/usr/lib/python%d.%s" % (
+                python_version[0],
+                python_version[1],
+            )
+
             # PySide accesses its directory.
-            if (
-                loaded_filename
-                == "/usr/lib/python" + python_version[:3] + "/dist-packages/PySide"
-            ):
+            if loaded_filename == os.path.join(lib_prefix_dir, "dist-packages/PySide"):
                 continue
 
             # GTK accesses package directories only.
-            if (
-                loaded_filename
-                == "/usr/lib/python" + python_version[:3] + "/dist-packages/gtk-2.0/gtk"
+            if loaded_filename == os.path.join(
+                lib_prefix_dir, "dist-packages/gtk-2.0/gtk"
             ):
                 continue
-            if (
-                loaded_filename
-                == "/usr/lib/python" + python_version[:3] + "/dist-packages/glib"
+            if loaded_filename == os.path.join(lib_prefix_dir, "dist-packages/glib"):
+                continue
+            if loaded_filename == os.path.join(
+                lib_prefix_dir, "dist-packages/gtk-2.0/gio"
             ):
                 continue
-            if (
-                loaded_filename
-                == "/usr/lib/python" + python_version[:3] + "/dist-packages/gtk-2.0/gio"
-            ):
-                continue
-            if (
-                loaded_filename
-                == "/usr/lib/python" + python_version[:3] + "/dist-packages/gobject"
-            ):
+            if loaded_filename == os.path.join(lib_prefix_dir, "dist-packages/gobject"):
                 continue
 
             # PyQt5 seems to do this, but won't use contents then.
@@ -561,8 +534,15 @@ def main():
             ):
                 continue
 
-            # Can look at the interpreter of the system.
-            if loaded_basename == "python3":
+            # Can look at the interpreters of the system.
+            if loaded_basename in "python3":
+                continue
+            if loaded_basename in (
+                "python%s" + supported_version
+                for supported_version in (
+                    getSupportedPythonVersions() + getPartiallySupportedPythonVersions()
+                )
+            ):
                 continue
 
             # Current Python executable can actually be a symlink and
@@ -578,6 +558,10 @@ def main():
             if loaded_filename in ("/sys/fs/selinux", "/selinux"):
                 continue
 
+            # Looking at device is OK.
+            if loaded_filename.startswith("/sys/devices/"):
+                continue
+
             # Allow reading time zone info of local system.
             if loaded_filename.startswith("/usr/share/zoneinfo/"):
                 continue
@@ -606,23 +590,19 @@ def main():
             if loaded_basename.upper() in ("MSVCRT.DLL", "MSVCR90.DLL"):
                 continue
 
-            my_print("Should not access '%s'." % loaded_filename)
+            test_logger.warning("Should not access '%s'." % loaded_filename)
             illegal_access = True
 
         if illegal_access:
             if os.name != "nt":
-                my_print("Listing of dist folder:")
-                os.system("ls -Rla %s" % filename[:-3] + ".dist")
-
-                my_print("Inclusion log:")
-                os.system("cat %s" % filename[:-3] + ".py.inclusion.log")
+                displayError(None, filename)
 
                 # Run with traces to help debugging, specifically in CI environment.
                 if sys.platform == "darwin" or sys.platform.startswith("freebsd"):
-                    my_print("dtruss:")
+                    test_logger.info("dtruss:")
                     os.system("sudo dtruss %s" % binary_filename)
                 else:
-                    my_print("strace:")
+                    test_logger.info("strace:")
                     os.system("strace -s4096 -e file %s" % binary_filename)
 
             search_mode.onErrorDetected(1)
